

* calculate equity vesting variables from Equilar;
* input data: equilar data;
* Interest rate from FRED; 
* All other inputs such as WRDS Compustat/CRSP/Thomson Financial Insider Trading are specified in the code;

LIBNAME v "D:\Data";

* Read in Equilar Data;
* Read in Company Financials;
%MACRO read_company_financials;
	%DO year = 2006 %TO 2016;
		PROC IMPORT DATAFILE = "D:\Data\Equilar&year..xls" 
			OUT = cf_&year 
			DBMS = EXCELCS
			REPLACE; 
			SHEET = "Company Financials";
		RUN;
		
		DATA cf_&year;
			SET cf_&year;
			equilarfyear = &year;
			DROP f9-f38;

			company_name = UPCASE(Company_Name); 
			RENAME fye_stock_price = prc;
			FORMAT fy_end MMDDYY10.; 

			fyear = YEAR(fy_end) - (MONTH(fy_end) <= 5);
			IF company_name ^= '';
		RUN;
	%END;
	DATA cf; 
		SET cf_2007-cf_2016 cf_2006; 
	RUN;
%MEND;
%read_company_financials; 

* read in CUSIP;
PROC IMPORT DATAFILE = "D:\Data\Market Indices Constituents 2009-present vCUSIP.xls" 
	OUT = cusips 
	DBMS = EXCELCS
	REPLACE; 
	SHEET = "CUSIPs";
RUN;
PROC SORT DATA = cusips NODUPKEY; BY company_id; RUN;

PROC SQL;
	CREATE TABLE cf2 AS
	SELECT	a.*, b.cusip
	FROM	cf (DROP = cusip) a, cusips b
	WHERE	a.company_id = b.company_id;
QUIT;

DATA cf2;
	SET cf2;
	l = LENGTH(cusip);
	IF cusip IN ('XXXXXXX', 'XXXXXXXX', 'XXXXXXXXX') THEN cusip = '';
	canada = (l = 9 AND SUBSTR(cusip, 9, 1) = 'C');
	IF l = 6 THEN cusip = '00'||cusip;
	ELSE IF l = 7 THEN cusip = '0'||cusip;
	ELSE IF l = 9 THEN cusip = SUBSTR(cusip, 1, 8);
	DROP l;
RUN;

PROC SORT DATA = cf2 OUT = v.cf NODUPKEY; 
	BY company_id fy_end;
RUN;


* Read in Executive Data - ND;
%MACRO read_ednd;
	%DO year = 2006 %TO 2016;
		PROC IMPORT DATAFILE = "D:\Data\Equilar&year..xls" 
			OUT = ednd_&year 
			DBMS = EXCELCS
			REPLACE; 
			SHEET = "Executive Data - ND";
		RUN;
		DATA ednd_&year;
			SET ednd_&year;
			equilarfyear = &year;
			fyear = YEAR(fy_end) - (MONTH(fy_end) <= 5);
			FORMAT FY_End Resignation_Date MMDDYY10.;
			DROP _ _1 _2 _3 _4 _5 _6 _7;
			WHERE company ^= '' AND executive_id ^= '99999.9.N.99999';
		RUN;
	%END;
	
	DATA ednd; 
		SET ednd_2015-ednd_2016 ednd_2006-ednd_2014; 
		executive_id2 = SCAN(executive_id, 4, '.') * 1;
	RUN;
%MEND read_ednd;
%read_ednd; 

PROC SORT DATA = ednd OUT = v.ednd NODUPKEY;
	BY company_id executive_id2 fy_end;
RUN;

* Read in Option Awards - ND;
%MACRO read_option_awards;
	%DO year = 2006 %TO 2016;
		PROC IMPORT DATAFILE = "D:\Data\Equilar&year..xls" 
			OUT = oand_&year
			DBMS = EXCELCS
			REPLACE; 
			SHEET = "Option Award Details - ND";
		RUN;
		
		DATA oand_&year;
			SET oand_&year;
			RENAME 	dividend_yield___ = dividendyield
					stock_price = stockprice
					Grant_Date_Present_Value_of_Opti = GrantDatePresentValueofOpti
					Grant_Date_Present_Value_of_Opt0 = GrantDatePresentValueofOpt0
					Grant_Date_Present_Value_of_Opt1 = GrantDatePresentValueofOpt1
					Grant_Date_Present_Value_of_Opt2 = GrantDatePresentValueofOpt2;

			equilarfyear = &year;
			fyear = YEAR(fy_end) - (MONTH(fy_end) <= 5);
			executive_id2 = SCAN(executive_id, 4, '.') * 1;
			FORMAT fy_end grant_date expiration_date MMDDYY10.; 
			IF stock_price = 9999.9999 THEN stock_price = .;
			IF executive ^= '';
		RUN;
		
	%END;

	DATA v.oand; 
		SET oand_2006-oand_2016; 
	RUN;
%MEND read_option_awards;
%read_option_awards;


* Read in Stock Awards - ND;
%MACRO read_stock_awards;
	%DO year = 2006 %TO 2016;
		PROC IMPORT DATAFILE = "D:\Data\Equilar&year..xls" 
			OUT = sand_&year
			DBMS = EXCELCS
			REPLACE; 
			SHEET = "Stock Award Details - ND";
		RUN;
		
		DATA sand_&year;
			SET sand_&year;
			RENAME	Grant_Date_Stock_Price = GrantDateStockPrice
					Vesting_Period = VestingPeriod
					Grant_Date_Present_Value_of_Secu = GrantDatePresentValueofSecu;

			equilarfyear = &year; 
			fyear = YEAR(fy_end) - (MONTH(fy_end) <= 5);
			executive_id2 = SCAN(executive_id, 4, '.') * 1;

			FORMAT fy_end grant_date MMDDYY10.; 
			WHERE executive ^= "";
		RUN;
	%END;

	DATA v.sand; 
		SET sand_2013 sand_2006-sand_2012 sand_2014-sand_2016; 
	RUN;	
%MEND read_stock_awards;
%read_stock_awards;

* Read in IP Awards;
%MACRO read_ip_awards;
	%DO year = 2006 %TO 2016;
		PROC IMPORT DATAFILE = "D:\Data\Equilar&year..xls" 
			OUT = ipand_&year
			DBMS = EXCELCS
			REPLACE; 
			SHEET = "IP Award Details - ND";
		RUN;
		
		DATA ipand_&year;
			SET ipand_&year;
			equilarfyear = &year; 
			fyear = YEAR(fy_end) - (MONTH(fy_end) <= 5);
			executive_id2 = SCAN(executive_id, 4, '.') * 1;
			FORMAT fy_end grant_date MMDDYY10.; 
			WHERE executive ^= "" AND company_id ^= 9999999;
		RUN;
	%END;

	DATA v.ipand; 
		SET ipand_2015 ipand_2006-ipand_2014 ipand_2016; 
	RUN;
%MEND read_ip_awards;
%read_ip_awards;

* Read in Outstanding Equity - ND;
%MACRO read_outstanding_equity;
	%DO year = 2006 %TO 2016;
		PROC IMPORT DATAFILE = "D:\Data\Equilar&year..xls" 
			OUT = oend_&year
			DBMS = EXCELCS
			REPLACE; 
			SHEET = "Outstanding Equity - ND";
		RUN;
		
		DATA oend_&year;
			SET oend_&year;
			DROP _ _1 _2;
			equilarfyear = &year; 
			fyear = YEAR(fy_end) - (MONTH(fy_end) <= 5);
			executive_id2 = SCAN(executive_id, 4, '.') * 1;

			FORMAT fy_end expiration_date MMDDYY10.; 
			WHERE executive ^= "" AND company_id ^= 9999999;
		RUN;
	%END;

	DATA v.oend; 
		SET oend_2015 oend_2006-oend_2014 oend_2016; 
		RENAME 	remaining_term_as_of_fiscal_year = remaining_term
				value_as_of_fiscal_year_end_____ = value
				in_the_money__as_of_fiscal_year_ = in_the_money;
	RUN;
%MEND read_outstanding_equity;
%read_outstanding_equity;

* Merge Equilar with Compustat;
* Retrive Compustat;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME compna '/wrds/comp/sasdata/naa'; 

PROC SQL UNDO_POLICY = NONE;
	CREATE TABLE cominfo AS
	SELECT 	gvkey, fyear,  
			cusip AS compcusip, 
			SUBSTR(cusip, 1, LENGTH(cusip) - 1) AS cusip2,
			SUBSTR(cusip, 1, 6) AS cusip6,
			FYR, OPTDR, OPTRFR, OPTVOL, CONM, TIC, 
			datadate, CURCD, prcc_f, ADJEX_C, ADJEX_F, AJEX, AJP, cshpri
	FROM 	compna.funda 
	WHERE	indfmt = 'INDL' AND datafmt = 'STD' AND popsrc = 'D' AND consol = 'C' 
			AND fyear >= 2004; 
QUIT;	

PROC DOWNLOAD DATA = cominfo OUT = cominfo; RUN;
ENDRSUBMIT;

* match CUSIP and TICKER;
* match full cusip;
PROC SQL;
	CREATE TABLE match1 AS
	SELECT	*,
			(2 - (ticker = tic)) AS rank,
			(company_name = conm) AS namematch,
			(SUBSTR(company_name, 1, 6) = SUBSTR(conm, 1, 6)) AS namematch2,
			(fy_end = datadate) AS fyendmatch
	FROM	v.cf AS a, cominfo AS b
	WHERE	a.cusip ^= '' AND a.cusip = b.cusip2 AND a.fyear = b.fyear;
QUIT;

PROC SORT DATA = match1 NODUPKEY; BY company_id fy_end; RUN;

* manual check;
PROC SORT DATA = match1 (KEEP = company_name conm cusip compcusip tic ticker rank namematch namematch2 
		fy_end datadate fyr fyendmatch gvkey) 
	OUT = checkfye;
	BY rank DESCENDING namematch DESCENDING namematch2;
	WHERE fyendmatch = 0;
RUN;

* unmatched;
PROC SQL;
	CREATE TABLE unmatch1 AS
	SELECT	a.*, SUBSTR(cusip, 1, 6) AS cusip6
	FROM	v.cf a LEFT JOIN (SELECT company_id, fy_end FROM match1) b
	ON		a.company_id = b.company_id AND a.fy_end = b.fy_end
	HAVING	b.fy_end = .;
QUIT;

* match with ticker;
PROC SQL;
	CREATE TABLE match2 AS 
	SELECT		*, 3 AS rank,
				(company_name = conm) AS namematch,
				(SUBSTR(company_name, 1, 6) = SUBSTR(conm, 1, 6)) AS namematch2,
				(a.cusip6 = b.cusip6) AS cusip6match,
				(fy_end = datadate) AS fyendmatch
	FROM		unmatch1 a, cominfo b
	WHERE		a.ticker = b.tic AND a.fyear = b.fyear
	ORDER BY	cusip6match DESC, namematch DESC, namematch2 DESC, fyendmatch DESC;
QUIT;

PROC SORT DATA = match2 OUT = temp NODUPKEY; BY company_id fy_end; RUN;

* manual check;
PROC SORT DATA = match2 (KEEP = company_name conm cusip compcusip tic ticker rank namematch namematch2 cusip6match
		fy_end datadate fyr fyendmatch gvkey) OUT = check2; 
	BY company_name conm;
	WHERE namematch = 0 AND namematch2 = 0 AND cusip6match = 0; 
RUN;

PROC EXPORT DATA = check2
	OUTFILE = 'D:\Data\equilar_compustat_manualmatch1.xls' REPLACE
	DBMS = EXCELCS;
	SHEET = 'to_be_check';
RUN;

PROC IMPORT DATAFILE = 'D:\Data\equilar_compustat_manualmatch1.xls'
	OUT = match2manual REPLACE
	DBMS = EXCELCS; 
	SHEET = "checked_ok"; 
RUN;

PROC SQL;
	CREATE TABLE match2 AS
	SELECT	a.*, (b.fy_end ^= .) AS manual
	FROM	match2 a LEFT JOIN match2manual b
	ON		a.gvkey = b.gvkey AND a.fy_end = b.fy_end
	HAVING	namematch OR namematch2 OR cusip6match OR manual;
QUIT;

* unmatched ones;
PROC SQL;
	CREATE TABLE unmatch2 AS
	SELECT	a.*
	FROM	unmatch1 a LEFT JOIN 
			(SELECT company_id, fy_end FROM match2) b
	ON		a.company_id = b.company_id AND a.fy_end = b.fy_end
	HAVING	b.fy_end = .;
QUIT;

* match 6-digit cusip;
PROC SQL;
	CREATE TABLE match3 AS 
	SELECT		*, 4 AS rank,
				(company_name = conm) AS namematch,
				(SUBSTR(company_name, 1, 6) = SUBSTR(conm, 1, 6)) AS namematch2,
				(fy_end = datadate) AS fyendmatch
	FROM		unmatch2 a, cominfo b
	WHERE		a.cusip6 ^= '' AND a.cusip6 = b.cusip6 AND a.fyear = b.fyear
	ORDER BY	namematch DESC, namematch2 DESC;
QUIT;

PROC SQL;
	CREATE TABLE unmatch3 AS
	SELECT	a.*
	FROM	unmatch2 a LEFT JOIN (SELECT UNIQUE company_id, fy_end FROM match3) b
	ON		a.company_id = b.company_id AND a.fy_end = b.fy_end
	HAVING	b.fy_end = .;
QUIT;

* match using other matched company_id;
DATA match123;
	SET match1 match2 match3;
	KEEP company_id gvkey; 
RUN;

PROC SQL;
	CREATE TABLE match4 AS
	SELECT	a.*, b.gvkey, 5 AS rank
	FROM	unmatch3 a, (SELECT UNIQUE company_id, gvkey FROM match123) b
	WHERE	a.company_id = b.company_id;
QUIT;

* duplicates, manually correct;
PROC SQL;
	CREATE TABLE match4dup As
	SELECT		*
	FROM		match4
	GROUP BY	company_id, fy_end
	HAVING		COUNT(*) > 1;
QUIT;

DATA match4;
	SET match4;
	IF company_id = 2732 AND gvkey = '001468' THEN DELETE;
	IF company_id = 9163 AND gvkey = '166554' THEN DELETE;
RUN;

PROC SQL;
	CREATE TABLE unmatch4 AS
	SELECT	a.*
	FROM	unmatch3 a LEFT JOIN (SELECT UNIQUE company_id, fy_end FROM match4) b
	ON		a.company_id = b.company_id AND a.fy_end = b.fy_end
	HAVING	b.fy_end = .;
QUIT;

* match using company name;
PROC SQL;
	CREATE TABLE match5 AS 
	SELECT		*, 6 AS rank,
				SPEDIS(company_name, conm) AS namematch_score,
				(fy_end = datadate) AS fyendmatch
	FROM		unmatch4 a, cominfo b
	HAVING		namematch_score <= 30 AND a.fyear = b.fyear
	ORDER BY	company_name, namematch_score, fyendmatch DESC;
QUIT;

* manually investigate the match quality;
PROC EXPORT DATA = match5
	OUTFILE = 'D:\Data\equilar_compustat_manualmatch2.xls' REPLACE
	DBMS = EXCELCS;
	SHEET = 'to_be_check';
RUN;

PROC IMPORT DATAFILE = 'D:\Data\equilar_compustat_manualmatch2.xls'
	OUT = match5manual REPLACE
	DBMS = EXCELCS; 
	SHEET = "checked_ok"; 
RUN;

* matched;
PROC SQL;
	CREATE TABLE match5verified AS
	SELECT	a.*
	FROM	match5 a,  match5manual b
	WHERE	a.company_id = b.company_id AND a.fy_end = b.fy_end
			AND a.gvkey = b.gvkey;
QUIT;

* unmatched;
PROC SQL;
	CREATE TABLE unmatch5 AS
	SELECT	a.*
	FROM	unmatch4 a LEFT JOIN (SELECT UNIQUE company_id, fy_end FROM match5verified) b
	ON		a.company_id = b.company_id AND a.fy_end = b.fy_end
	HAVING	b.fy_end = .;
QUIT;

* unmatched earlier in this step because of year;
PROC SQL;
	CREATE TABLE match6 AS
	SELECT	a.*, b.gvkey, 6 AS rank, SPEDIS(company_name, conm) AS namematch_score
	FROM	unmatch5 a, (SELECT UNIQUE company_id, gvkey, conm FROM match5verified) b
	WHERE	a.company_id = b.company_id;
QUIT;

* unmatched;
PROC SQL;
	CREATE TABLE unmatch6 AS
	SELECT	a.*
	FROM	unmatch5 a LEFT JOIN (SELECT UNIQUE company_id, fy_end FROM match6) b
	ON		a.company_id = b.company_id AND a.fy_end = b.fy_end
	HAVING	b.fy_end = .;
QUIT;


* some firm-years are flagged as transition years;
DATA cfcomp;
	SET match1-match4 match5verified match6;
	DROP adjex_c adjex_f ajex ajp; 
	diff = ABS(datadate - fy_end);
RUN;

* identify transition years as those with more than one fy_end in one calendar year;
PROC SQL;
	CREATE TABLE ty AS 
	SELECT 		*
	FROM		cfcomp
	GROUP BY	company_id, equilarfyear
	HAVING		COUNT(*) > 1
	ORDER BY	company_id, equilarfyear, diff;
QUIT;

* choose the one with smaller distance of the dates for duplicates;
PROC SQL;
	CREATE TABLE ty_compustat AS 
	SELECT 		*
	FROM		ty
	GROUP BY	company_id, equilarfyear
	HAVING		COUNT(UNIQUE diff) > 1
	ORDER BY	company_id, equilarfyear, diff;
QUIT;

* verify smaller diff is the better match;
PROC SQL;
	CREATE TABLE ednd_ty_compustat AS
	SELECT	a.*, b.*
	FROM	v.ednd a, ty_compustat b
	WHERE	a.company_id = b.company_id AND a.fy_end = b.fy_end
	ORDER BY	a.company_id, a.fy_end;
QUIT;

PROC SQL;
	CREATE TABLE ty_compustat_ednd AS
	SELECT	a.*, (b.fy_end ^= .) AS ednd
	FROM	ty_compustat a 
			LEFT JOIN (SELECT UNIQUE company_id, fy_end FROM ednd_ty_compustat) b
	ON		a.company_id = b.company_id AND a.fy_end = b.fy_end;
QUIT;

PROC SQL;
	CREATE TABLE ty_compustat_ednd AS
	SELECT		*
	FROM		ty_compustat_ednd
	WHERE		ednd
	GROUP BY	company_id, equilarfyear
	HAVING		COUNT(UNIQUE fy_end) > 1;
QUIT;

DATA ty_compustat_remove;
	SET ty_compustat;
	BY company_id equilarfyear diff;
	IF LAST.equilarfyear;
RUN;

* manually examine equal matched dates;
PROC SQL;
	CREATE TABLE ty_manual AS 
	SELECT 		*
	FROM		ty
	GROUP BY	company_id, equilarfyear
	HAVING		COUNT(UNIQUE diff) <= 1
	ORDER BY	company_id, equilarfyear;
QUIT;

PROC SQL;
	CREATE TABLE ednd_ty_manual AS
	SELECT	a.*, b.*
	FROM	v.ednd a, ty_manual b
	WHERE	a.company_id = b.company_id AND a.fy_end = b.fy_end
	ORDER BY	a.company_id, a.fy_end;
QUIT;

PROC SQL;
	CREATE TABLE ty_manual_ednd AS
	SELECT	a.*, (b.fy_end ^= .) AS ednd
	FROM	ty_manual a 
			LEFT JOIN (SELECT UNIQUE company_id, fy_end FROM ednd_ty_manual) b
	ON		a.company_id = b.company_id AND a.fy_end = b.fy_end;
QUIT;

* two entries in one year in EDND;
PROC SQL;
	CREATE TABLE ty_manual_ednd2 AS
	SELECT		*
	FROM		ty_manual_ednd
	WHERE		ednd
	GROUP BY	company_id, equilarfyear
	HAVING		COUNT(UNIQUE fy_end) > 1;
QUIT;

* remove the ones that the other FY_END has entry in EDND, for the remaining ones, choose the one with December fiscal year end;
PROC SQL;
	CREATE TABLE ty_manual_ednd_remove AS
	SELECT		*
	FROM		ty_manual_ednd
	GROUP BY	company_id, equilarfyear
	HAVING		COUNT(UNIQUE ednd) = 2 AND ednd = 0 OR 
				COUNT(UNIQUE ednd) = 1 AND MONTH(fy_end) ^= 12;
RUN;

DATA ty_remove;
	SET ty_compustat_remove ty_manual_ednd_remove;
	KEEP company_id fy_end;
RUN;

* remove one obs for transition years;
PROC SQL;
	CREATE TABLE v.cfcomp (DROP = cusip2 cusip6 namematch namematch2 rank manual curcd cusip6match diff namematch_score) AS
	SELECT	a.*, YEAR(a.fy_end) AS year
	FROM	cfcomp a LEFT JOIN ty_remove b
	ON		a.company_id = b.company_id AND a.fy_end = b.fy_end
	WHERE 	curcd = 'USD'
	HAVING	b.fy_end = .;
QUIT;

* Merge cfcomp with Prices;
PROC SQL;
	CREATE TABLE cfcompprc (DROP = prc) AS
	SELECT		a.*, b.*, 
				COALESCE(c.prc * d.adjex_f, c.prc, d.prcc_f) AS lagfinalprc,
				COALESCE(a.prc * b.adjex_f, a.prc, a.prcc_f) AS finalprc,
				c.prc AS lagequilarprice, a.prc AS equilarprice,
				d.prcc_f AS lagprcc_f, d.adjex_f AS lagadjex_f,
				(CALCULATED lagfinalprc = c.prc AND CALCULATED lagfinalprc ^= .) AS priority,
				(CALCULATED finalprc = a.prc AND CALCULATED finalprc ^= .) AS priority1,
				(MONTH(a.fy_end) = MONTH(c.fy_end)) AS priority2
	FROM		v.cfcomp a 
				LEFT JOIN cominfo (KEEP = gvkey fyear adjex_f) b
	ON			a.gvkey = b.gvkey AND a.fyear = b.fyear
				LEFT JOIN v.cfcomp (KEEP = company_id equilarfyear prc fy_end) c
	ON			a.company_id = c.company_id AND a.equilarfyear = c.equilarfyear + 1
				LEFT JOIN cominfo d
	ON			a.gvkey = d.gvkey AND a.fyear = d.fyear + 1
	ORDER BY	company_id, fy_end, priority1 DESC, priority DESC, priority2 DESC;
QUIT;

PROC SORT DATA = cfcompprc (DROP = priority1 priority2) NODUPKEY;
	BY company_id fy_end;
RUN;

* Calculate newly vested shares;
* Retrieve newly vested shares;
DATA newlyvestedshares;
	SET v.ednd;
	KEEP executive_id executive_id2 company_id executive Ticker fy_end company title
		shares_acquired_on_vesting_of_st value_realized_on_vesting former_retired ceo;
	RENAME company = company_name;
	LABEL company = Company Name;
	former_retired = (COUNT(title, 'former', 'i') OR COUNT(title, 'retired', 'i'));
	ceo = (COUNT(title, 'CEO') OR COUNT(title, 'Chief Executive Officer') OR COUNT(title, 'Co-CEO') 
		OR COUNT(title, 'Chief Executive') OR COUNT(title, 'Co-Chief Executive Officer')); 
RUN;

* separate into firm-years with one or more than one executives;
* for the ones with one executive, keep it;
PROC SQL;
	CREATE TABLE newlyvestedshares_1exec AS
	SELECT		*
	FROM		newlyvestedshares
	GROUP BY	company_id, fy_end
	HAVING		COUNT(*) = 1;
QUIT;

* for the ones with more than one executive, investigate it;
PROC SQL;
	CREATE TABLE newlyvestedshares_nexec AS
	SELECT		*
	FROM		newlyvestedshares
	GROUP BY	company_id, fy_end
	HAVING		COUNT(*) > 1;
QUIT;

* Remove former/retired CEOS;
* verify these are actually former and/or retired;
PROC SQL;
	CREATE TABLE formerretiredtitles AS
	SELECT		title, COUNT(*) AS count
	FROM 		newlyvestedshares_nexec
	WHERE 		former_retired
	GROUP BY	title;
QUIT;

DATA formerretiredtitles;
	SET formerretiredtitles; 
	formerceo = (COUNT(title, 'Former CEO', 'i') OR COUNT(title, 'Former Chief Executive Officer', 'i')
	OR COUNT(title, 'Former Co-Chief Executive Officer', 'i') OR COUNT(title, 'Former Co-CEO')
	OR COUNT(title, 'Former Interim Chief Executive Officer', 'i') 
	OR COUNT(title, 'Former President and Chief Executive Officer') 
	OR COUNT(title, 'Former President and CEO', 'i'));
	former_front = UPCASE(SCAN(title, 1, ' ')) = 'FORMER';
	exec = (COUNT(title, 'Executive', 'i') > 0);
	former_before_exec = (FIND(title, 'former', 'i') < FIND(title, 'executive', 'i') 
		AND FIND(title, 'former', 'i') > 0);
	IF FIND(title, 'President and Chief Executive Officer', 'i') = 1 OR
		FIND(title, 'Chief Executive Officer', 'i') = 1 THEN notretired = 1;
RUN;
PROC SORT DATA = formerretiredtitles; 
	BY DESCENDING formerceo DESCENDING former_front DESCENDING exec 
		DESCENDING former_before_exec DESCENDING count; 
RUN;

DATA formerretiredtitles;
	SET formerretiredtitles;
	IF formerceo = 0 AND notretired = 1 THEN DELETE;
	KEEP title;
RUN;

* removed former/retired CEOs;
PROC SQL;
	CREATE TABLE newlyvestedshares2 AS
	SELECT	*
	FROM	newlyvestedshares_nexec
	WHERE	title NOT IN (SELECT title FROM formerretiredtitles);
QUIT;

PROC SORT DATA = newlyvestedshares2 OUT = temp NODUPKEY; by company_id fy_end; RUN;

* separate into firm-years with one or more than one executives;
* for the ones with only one executive, keep it;
PROC SQL;
	CREATE TABLE newlyvestedshares2_1exec AS
	SELECT		*
	FROM		newlyvestedshares2
	GROUP BY	company_id, fy_end
	HAVING		COUNT(*) = 1;
QUIT;

* for the ones with more than one executive, investigate it;
PROC SQL;
	CREATE TABLE newlyvestedshares2_nexec AS
	SELECT		*
	FROM		newlyvestedshares2
	GROUP BY	company_id, fy_end
	HAVING		COUNT(*) > 1;
QUIT;

* Remove Non-CEOs;
* some of the firm-years have a ceo and a non-CEO;
PROC SQL;
	CREATE TABLE manuallyverifyceo1 AS
	SELECT		*
	FROM		newlyvestedshares2_nexec
	GROUP BY	company_id, fy_end
	HAVING		COUNT(UNIQUE ceo) > 1;
QUIT;

DATA manuallyverifyceo1;
	SET manuallyverifyceo1;
	IF ceo = 1 OR company_id = 7547;
RUN;

* some firm-years are all ceo or all non-ceos, manually go through them;
PROC SQL;
	CREATE TABLE manuallyverifyceo2 AS
	SELECT		*, (FIND(title, 'co-c', 'i') > 0) AS potentialco
	FROM		newlyvestedshares2_nexec
	GROUP BY	company_id, fy_end
	HAVING		COUNT(UNIQUE ceo) = 1;
QUIT;
PROC SQL;
	CREATE TABLE manuallyverifyceo21 AS
	SELECT		*
	FROM		manuallyverifyceo2
	GROUP BY	company_id, fy_end
	HAVING		COUNT(UNIQUE title) = 1;
QUIT;
PROC SQL;
	CREATE TABLE manuallyverifyceo22 AS
	SELECT		*
	FROM		manuallyverifyceo2
	GROUP BY	company_id, fy_end
	HAVING		COUNT(UNIQUE title) > 1;
QUIT;
* remove CEO of a division or non-CEOs;
DATA manuallyverifyceo22;
	SET manuallyverifyceo22;
	IF executive_id IN ('2578.11.N.14383', '2578.11.N.14382', '3324.11.N.28340',
		'4925.11.N.58784', '7787.6.N.77685', '4018.8.N.41661') OR executive_id2 IN (199622, 1059679)
	THEN DELETE;
RUN;

DATA newlyvestedshares;
	SET newlyvestedshares_1exec newlyvestedshares2_1exec
		manuallyverifyceo1 manuallyverifyceo21 manuallyverifyceo22;
	DROP former_retired ceo potentialco;
RUN;
 
* calculate shares already vested and unvested;
* Retrieve Already vested Shares from EDND;
DATA alreadyvestedshares;
	SET v.ednd (KEEP = executive_id executive_id2 company_id executive Ticker FY_End FY
		company Shares_Held Options_Exercisable_within_60_Da Total_Unvested_Shares);
	alreadyvestedshares = SUM(shares_held, -1 * options_exercisable_within_60_da); 
	alreadyvestedshares2 = SUM(shares_held, -1 * total_unvested_shares, -1 * options_exercisable_within_60_da); 
RUN;

* Retrieve unvested Shares from EDND & OEND;
* retrieve unvested stocks from OEND;
PROC SQL;
	CREATE TABLE unvestedshares AS
	SELECT		company_id, executive_id2, fy_end, 
				SUM(number_of_securities) AS totalunvestedshares2 LABEL = 'TotalUnvestedShares2',
				1 AS merged
	FROM		v.oend
	WHERE		equity_type = 'Unvested Stock'
	GROUP BY	company_id, executive_id2, fy_end;
QUIT;

* merge OEND unvested stocks with unvested stocks from EDND;
PROC SQL;
	CREATE TABLE unvestedshares AS
	SELECT		a.*, b.*
	FROM		v.ednd 
				(KEEP = executive_id executive_id2 company_id executive ticker fy_end fy company 
				total_unvested_shares total_value_of_unvested_ip_share 
				total_value_of_unvested_shares Total_unvested_ip_shares) a 
				LEFT JOIN unvestedshares b
	ON			a.company_id = b.company_id AND a.executive_id2 = b.executive_id2 AND a.fy_end = b.fy_end
	ORDER BY	executive_id2, company_id, fy_end;
QUIT;


* Aggregate all measures of shares (newly vested, already vested, unvested, as well as holdings) in one file;

* Compile company-executive-fy_end index with prc;
* merge company-executive-fy_end list with compustat inputs including price;
PROC SQL;
	CREATE TABLE cfcompprcexec AS
	SELECT	a.*, executive_id2, executive
	FROM	cfcompprc (DROP = optdr optrfr optvol) a, 
			(SELECT UNIQUE company_id, executive_id2, executive, fy_end FROM v.ednd) b
	WHERE	a.company_id = b.company_id AND a.fy_end = b.fy_end;
QUIT;

* merge with lag company-executive-fy_end list to see if there exists an executive in the last year;
PROC SQL;
	CREATE TABLE cfcompprcexec AS
	SELECT	a.*, b.fy_end AS lagfy_end, (b.fy_end ^= .) AS lagexist
	FROM	cfcompprcexec a LEFT JOIN 
			(SELECT UNIQUE company_id, executive_id2, fy_end FROM v.ednd) b
	ON		a.company_id = b.company_id AND a.executive_id2 = b.executive_id2
			AND YEAR(a.fy_end) = YEAR(b.fy_end) + 1;
QUIT;
* remove 1 duplicates;
DATA cfcompprcexec;
	SET cfcompprcexec;
	IF company_id = 21075 AND lagfy_end = MDY(1, 31, 2012) THEN DELETE;
RUN;

* Merge index with newly vested stocks;
PROC SQL;
	CREATE TABLE shares1 AS
	SELECT	a.*, 
			value_realized_on_vesting, shares_acquired_on_vesting_of_st
	FROM	cfcompprcexec a, newlyvestedshares b
	WHERE	a.company_id = b.company_id AND a.fy_end = b.fy_end AND a.executive_id2 = b.executive_id2;
QUIT;

PROC SORT DATA = shares1 OUT = temp NODUPKEY; BY company_id fy_end executive_id2; RUN;

* calculate vesting share value;
DATA shares1;
	SET shares1;
	IF value_realized_on_vesting ^= . THEN valueofnewlyvestedstock = value_realized_on_vesting; 
	ELSE IF shares_acquired_on_vesting_of_st <= 0 THEN valueofnewlyvestedstock = 0;
	ELSE IF lagfinalprc ^= . THEN valueofnewlyvestedstock = shares_acquired_on_vesting_of_st * lagfinalprc; 
	ELSE IF finalprc ^= . THEN valueofnewlyvestedstock = shares_acquired_on_vesting_of_st * finalprc; 
RUN;

* remove obs without value of newly vested stocks;
DATA shares1; 
	SET shares1; 
	IF valueofnewlyvestedstock = . THEN DELETE; 
RUN;

* Merge with already vested shares in year t-1;
PROC SORT DATA = shares1 OUT = temp NODUPKEY; BY company_id lagfy_end executive_id2; RUN;

* merge with last year already vested shares measure;
* if lag exist and has no value for already vested shares, set it to zero;
PROC SQL;
	CREATE TABLE shares2 AS
	SELECT	a.*, 
			(b.company_id ^= .) AS match_last_alreadyvested,
			shares_held, options_exercisable_within_60_da,
			alreadyvestedshares, alreadyvestedshares2 
	FROM	shares1 a LEFT JOIN alreadyvestedshares b
	ON		a.company_id = b.company_id AND a.executive_id2 = b.executive_id2
			AND a.lagfy_end = b.fy_end;
QUIT;

* calculate already vested shares;
DATA shares2;
	SET shares2;
	IF lagexist AND alreadyvestedshares = . THEN alreadyvestedshares = 0;
	IF lagexist AND alreadyvestedshares <= 0 THEN valueofalreadyvestedstock = 0;
	ELSE IF lagfinalprc ^= . THEN valueofalreadyvestedstock = alreadyvestedshares * lagfinalprc;
	ELSE IF finalprc ^= . THEN valueofalreadyvestedstock = alreadyvestedshares * finalprc;
	valueofalreadyvestedstock_nomiss = COALESCE(valueofalreadyvestedstock, 0); 
RUN;

PROC SORT DATA = shares2 OUT = shares2 NODUPKEY; BY company_id executive_id2 fy_end;RUN;

* Merge with unvested shares in year t-1;
PROC SORT DATA = shares2 OUT = temp NODUPKEY; BY company_id lagfy_end executive_id2; RUN;

PROC SQL;
	CREATE TABLE shares3 AS
	SELECT	a.*, 
			(b.company_id ^= .) AS match_last_unvested,
			total_unvested_shares, total_value_of_unvested_shares,
			total_unvested_ip_shares, total_value_of_unvested_ip_share
	FROM	shares2 a LEFT JOIN unvestedshares b
	ON		a.company_id = b.company_id AND a.executive_id2 = b.executive_id2 AND a.lagfy_end = b.fy_end;
QUIT;

* calculate value of unvested stocks;
* calculate value of unvested stocks plus ip;
DATA shares3;
	SET shares3;
	IF lagexist AND total_value_of_unvested_shares ^= . THEN valueofunvestedstock = total_value_of_unvested_shares;
	ELSE IF lagexist AND total_unvested_shares <= 0 THEN valueofunvestedstock = 0;
	ELSE IF lagexist AND lagfinalprc ^= . THEN valueofunvestedstock = total_unvested_shares * lagfinalprc;
	ELSE IF lagexist AND finalprc ^= . THEN valueofunvestedstock = total_unvested_shares * finalprc; 

	IF lagexist AND SUM(total_value_of_unvested_shares, total_value_of_unvested_ip_share) ^= . THEN
		valueofunvestedstockplusip = SUM(total_value_of_unvested_shares, total_value_of_unvested_ip_share);
	ELSE IF lagexist AND SUM(total_unvested_shares, total_unvested_ip_shares) <= 0 THEN valueofunvestedstockplusip = 0;
	ELSE IF lagexist AND lagfinalprc ^= . THEN 
		valueofunvestedstockplusip = SUM(total_unvested_shares,total_unvested_ip_shares) * lagfinalprc;
	ELSE IF lagexist AND finalprc ^= . THEN 
		valueofunvestedstockplusip = SUM(total_unvested_shares,total_unvested_ip_shares) * finalprc; 

	valueofunvestedstock_nomi = COALESCE(valueofunvestedstock, 0); 
	valueofunvestedstockplusip_nomi = COALESCE(valueofunvestedstockplusip, 0); 
RUN;

PROC SORT DATA = shares3 OUT = v.shares NODUPKEY; BY company_id executive_id2 fy_end;RUN;


* Options Newly Awarded, Unexercisable and Exercisable;
* Retrieve newly awarded options from OAND;	
* aggregate options of same firm-manager-year-expdate-stkprc;

* retrieve newly awarded options from OAND;
DATA newlyawardedoptions;
	SET v.oand (KEEP = executive executive_id company_id executive_id2 ticker fy_end company type 
		number_of_options grant_date exercise_price volatility_ risk_free_rate__ term vesting_period 
		dividendYield vesting_schedule performance_ metric_s_ expiration_date);
	stkprc_aw = exercise_price;   
	expdate_aw = expiration_date; 
	IF expiration_date = . THEN expdate_aw = MDY(MONTH(grant_date), DAY(grant_date), YEAR(grant_date) + term);
	FORMAT expdate_aw MMDDYY10.;
	IF expdate_aw = . OR stkprc_aw = . OR number_of_options = . THEN DELETE;
RUN;

* Group newly awarded options by strike price/expiration date;
PROC SQL;
	CREATE TABLE newlyawardedoptions_sum AS
	SELECT		company_id, executive_id2, executive, fy_end,  
				expdate_aw AS expdate, stkprc_aw AS stkprc,
				COUNT(*) AS n_multiple, SUM(number_of_options) AS number
	FROM		newlyawardedoptions
	GROUP BY	company_id, executive_id2, executive, fy_end, expdate_aw, stkprc_aw
	ORDER BY	company_id, executive_id2, executive, fy_end, expdate, stkprc;
QUIT;

* Calculate Unexercisable Options from OAND and fill in missing expiration dates and strike prices;
* Retrieve OE-Unexercisable Options from OEND;
* combine options that everything is the same except for the numbers;
PROC SQL;
	CREATE TABLE unexercisableoptions AS
	SELECT		executive, executive_id, company_id, executive_id2, ticker, fy_end, 
				company, exercise_price, remaining_term, expiration_date,  
				expiration_date AS expdate FORMAT MMDDYY10., exercise_price AS stkprc,
				SUM(number_of_securities) AS number_of_securities, SUM(value) AS value
	FROM		v.oend 
	WHERE		equity_type = 'Unexercisable Options' AND number_of_securities ^= .
	GROUP BY	executive, executive_id, company_id, executive_id2, ticker, fy_end,  
				company, exercise_price, remaining_term, expiration_date;
QUIT;

* Fill in the missing exp date of unexercisable options with expdate adjacent unexercisable options with same strike price;
* missing expiration dates but not missing exercise price;
DATA missing_expdate;
	SET unexercisableoptions (WHERE = (expdate = . AND stkprc ^= .));
RUN;

* match these missing expdate ones with nonmissing-info options of the same executive, same stkprc and closest distance in fy_end;
PROC SQL;
	CREATE TABLE missing_expdate AS
	SELECT		executive, company, ticker, a.fy_end,  
				expiration_date, exercise_price, 
				remaining_term, executive_id, company_id, a.executive_id2, 
				b.expdate,
				a.stkprc, a.number_of_securities, value, 
				b.fy_end AS fy_end2 FORMAT MMDDYY10.,
				b.number_of_securities AS number_of_securities2
	FROM		missing_expdate a 
				LEFT JOIN 	unexercisableoptions 
					(KEEP = executive_id2 fy_end stkprc expdate number_of_securities
					WHERE = (expdate ^= . AND stkprc ^= .)) b
	ON			a.executive_id2 = b.executive_id2 AND a.stkprc = b.stkprc
	GROUP BY	executive, company, ticker, a.fy_end,  
				expiration_date, exercise_price, remaining_term, executive_id, company_id, 
				a.executive_id2, a.stkprc, a.number_of_securities, value
	HAVING		ABS(a.fy_end - b.fy_end) = MIN(ABS(a.fy_end - b.fy_end))
	ORDER BY	executive, company, ticker, fy_end, expiration_date, 
				exercise_price, remaining_term, executive_id, company_id, executive_id2, 
				stkprc, number_of_securities, value, number_of_securities2 DESC;
QUIT;

* for duplicates, pick the matched grant with a larger number of options;
DATA missing_expdate;
	SET missing_expdate;
	BY executive company ticker fy_end expiration_date exercise_price 
		remaining_term executive_id company_id executive_id2 stkprc number_of_securities value 
		DESCENDING number_of_securities2;
	IF FIRST.value;
	DROP fy_end2 number_of_securities2;
RUN;

* merge the matched missing one back with the unmissing one;
DATA unexercisableoptions2;
	SET unexercisableoptions (WHERE = (expdate ^= . OR stkprc = .)) missing_expdate;
RUN;

* Fill in the missing strike price of unexercisable options with strike price of adjacent unexercisable options with same exp date;
* non-missing expiration dates but missing exercise price;
DATA missing_stkprc;
	SET unexercisableoptions2;
	WHERE expdate ^= . AND stkprc = .;
RUN;

* match these missing expdate ones with options of the same executive, same expiration date and closest distance in fy_end;
PROC SQL;
	CREATE TABLE missing_stkprc AS
	SELECT		executive, company, ticker, a.fy_end,  
				expiration_date, exercise_price, 
				remaining_term, executive_id, company_id, a.executive_id2, 
				a.expdate,
				b.stkprc, a.number_of_securities, value, 
				b.fy_end AS fy_end2 FORMAT MMDDYY10.,
				b.number_of_securities AS number_of_securities2
	FROM		missing_stkprc a 
	LEFT JOIN 	unexercisableoptions2 
					(KEEP = executive_id2 fy_end stkprc expdate number_of_securities
					WHERE = (expdate ^= . AND stkprc ^= .)) b
	ON			a.executive_id2 = b.executive_id2 AND a.expdate = b.expdate
	GROUP BY	executive, company, ticker, a.fy_end,  
				expiration_date, exercise_price, remaining_term, executive_id, company_id, 
				a.executive_id2, a.expdate, a.number_of_securities, value
	HAVING		ABS(a.fy_end - b.fy_end) = MIN(ABS(a.fy_end - b.fy_end))
	ORDER BY	executive, company, ticker, fy_end, expiration_date, 
				exercise_price, remaining_term, executive_id, company_id, executive_id2, 
				expdate, number_of_securities, value, number_of_securities2 DESC;
QUIT;

* merge the matched missing one back with the unmissing one;
DATA unexercisableoptions3 unfilled;
	SET unexercisableoptions2 (WHERE = (expdate = . OR stkprc ^= .)) missing_stkprc (DROP = fy_end2 number_of_securities2);
	OUTPUT unexercisableoptions3;
	IF expdate = . OR stkprc = . THEN OUTPUT unfilled;
RUN;

* Match unexercisable options with earlier newly awarded options;
* using non-missing strike price/expiration date/number of options to fill in missing strike prices and expiration dates;

* require that the unexercisable option to be later than the newly awarded option date;
PROC SQL;
	CREATE TABLE filled_with_new_options AS 
	SELECT		COALESCE(a.stkprc, c.stkprc_aw, d.stkprc_aw) AS stkprc, 
				COALESCE(a.expdate, b.expdate_aw, d.expdate_aw) AS expdate FORMAT MMDDYY10., 
				b.number_of_options AS number_of_options_exp,
				c.number_of_options AS number_of_options_stk,
				d.number_of_options AS number_of_options_num,
				a.*
	FROM		unfilled a 
				LEFT JOIN newlyawardedoptions 
					(KEEP = executive_id2 fy_end stkprc_aw expdate_aw number_of_options) b
	ON			a.executive_id2 = b.executive_id2 AND a.stkprc = b.stkprc_aw AND a.fy_end >= b.fy_end
				LEFT JOIN newlyawardedoptions 
					(KEEP = executive_id2 fy_end stkprc_aw expdate_aw number_of_options) c
	ON			a.executive_id2 = c.executive_id2 AND a.expdate = c.expdate_aw AND a.fy_end >= c.fy_end	
				LEFT JOIN newlyawardedoptions 
					(KEEP = executive_id2 fy_end stkprc_aw expdate_aw number_of_options) d
	ON			a.executive_id2 = d.executive_id2 AND a.number_of_securities = d.number_of_options 
				AND a.fy_end >= d.fy_end
	ORDER BY	executive, company, ticker, fy_end, expiration_date, exercise_price, remaining_term, 
				executive_id, company_id, executive_id2, number_of_securities, value, 
				number_of_options_exp DESC, number_of_options_num DESC;
QUIT;

* remove duplicates by picking the one with the largest number_of_options;
DATA filled_with_new_options;
	SET filled_with_new_options;
	BY executive company ticker fy_end expiration_date exercise_price remaining_term 
		executive_id company_id executive_id2 number_of_securities value 
		DESCENDING number_of_options_exp DESCENDING number_of_options_num;
	IF FIRST.value;
RUN;

* fill in missing term with expiration date and grant date;
DATA unexercisableoptions4;
	SET unexercisableoptions3 (WHERE = (expdate ^= . AND stkprc ^= .)) filled_with_new_options;
	IF expdate = . OR stkprc = . OR number_of_securities = . THEN DELETE;
	filled_term = COALESCE(remaining_term, (expdate - fy_end) / 365); 
	KEEP company company_id executive_id2 executive fy_end remaining_term
		number_of_securities expdate stkprc filled_term;
RUN;

* manually verify expdate-stkprc with difference in term and remaining_term should use the remaining_term;
PROC SQL;
	CREATE TABLE dup_expdate_stkprc_diff_term AS
	SELECT		*
	FROM		unexercisableoptions4
	GROUP BY	fy_end, company_id, executive_id2, expdate, stkprc
	HAVING		COUNT(UNIQUE filled_term) > 1;
QUIT;

* Replace term with the Remaining_Term matched by STKPRC/EXPDATE;
PROC SQL;
	CREATE TABLE unexercisableoptions AS
	SELECT	COALESCE(b.remaining_term, a.filled_term) AS term, a.*
	FROM	unexercisableoptions4 a LEFT JOIN 
			dup_expdate_stkprc_diff_term (KEEP = fy_end company_id executive_id2 expdate stkprc remaining_term 
				WHERE = (remaining_term ^= .)) b
	ON		a.fy_end = b.fy_end AND a.company_id = b.company_id AND a.executive_id2 = b.executive_id2 
			AND a.expdate = b.expdate AND a.stkprc = b.stkprc;
QUIT;

* Group unexercisable options by strike price/expiration date;
* Group multiple options with the same EXPDATE/STKPRC into one;
PROC SQL;
	CREATE TABLE unexercisableoptions_sum AS
	SELECT		company, company_id, executive_id2, executive, fy_end, expdate, stkprc, term,
				COUNT(*) AS n_multiple, SUM(number_of_securities) AS number
	FROM		unexercisableoptions
	GROUP BY	company, company_id, executive_id2, executive, fy_end, expdate, stkprc, term;
QUIT;

* Calculate Exercisable Options from OAND and fill in missing expiration dates and strike prices;
* Retrieve OE-Exercisable Options from OEND;

* for some options, everything is the same except for the numbers, combine them;
PROC SQL;
	CREATE TABLE exercisableoptions AS
	SELECT		executive, executive_id, company_id, executive_id2, ticker, fy_end, 
				company, exercise_price, remaining_term, expiration_date,  
				expiration_date AS expdate FORMAT MMDDYY10., exercise_price AS stkprc,
				SUM(number_of_securities) AS number_of_securities, SUM(value) AS value
	FROM		v.oend 
	WHERE		equity_type = 'Exercisable Options' AND number_of_securities ^= .
	GROUP BY	executive, executive_id, company_id, executive_id2, ticker, fy_end,  
				company, exercise_price, remaining_term, expiration_date;
QUIT;

* Fill in the missing exp date of exercisable options with expdate adjacent exercisable options with same strike price;
* missing expiration dates but not missing exercise price;
DATA missing_expdate;
	SET exercisableoptions (WHERE = (expdate = . AND stkprc ^= .));
RUN;

* match these missing expdate ones with nonmissing-info options of the same executive, same stkprc and closest distance in fy_end;
PROC SQL;
	CREATE TABLE missing_expdate AS
	SELECT		executive, company, ticker, a.fy_end,  
				expiration_date, exercise_price, 
				remaining_term, executive_id, company_id, a.executive_id2, 
				b.expdate,
				a.stkprc, a.number_of_securities, value, 
				b.fy_end AS fy_end2 FORMAT MMDDYY10.,
				b.number_of_securities AS number_of_securities2
	FROM		missing_expdate a 
				LEFT JOIN 	exercisableoptions 
					(KEEP = executive_id2 fy_end stkprc expdate number_of_securities
					WHERE = (expdate ^= . AND stkprc ^= .)) b
	ON			a.executive_id2 = b.executive_id2 AND a.stkprc = b.stkprc
	GROUP BY	executive, company, ticker, a.fy_end,  
				expiration_date, exercise_price, remaining_term, executive_id, company_id, 
				a.executive_id2, a.stkprc, a.number_of_securities, value
	HAVING		ABS(a.fy_end - b.fy_end) = MIN(ABS(a.fy_end - b.fy_end))
	ORDER BY	executive, company, ticker, fy_end, expiration_date, 
				exercise_price, remaining_term, executive_id, company_id, executive_id2, 
				stkprc, number_of_securities, value, number_of_securities2 DESC;
QUIT;

* for duplicates, pick the matched grant with a larger number of options;
DATA missing_expdate;
	SET missing_expdate;
	BY executive company ticker fy_end expiration_date exercise_price 
		remaining_term executive_id company_id executive_id2 stkprc number_of_securities value 
		DESCENDING number_of_securities2;
	IF FIRST.value;
	DROP fy_end2 number_of_securities2;
RUN;

* merge the matched missing one back with the unmissing one;
DATA exercisableoptions2;
	SET exercisableoptions (WHERE = (expdate ^= . OR stkprc = .)) missing_expdate;
RUN;

* Fill in the missing strike price of exercisable options with strike price of adjacent exercisable options with same exp date;
* non-missing expiration dates but missing exercise price;
DATA missing_stkprc;
	SET exercisableoptions2;
	WHERE expdate ^= . AND stkprc = .;
RUN;

* match these missing stkprc ones with options of the same executive, same expiration date and closest distance in fy_end;
PROC SQL;
	CREATE TABLE missing_stkprc AS
	SELECT		executive, company, ticker, a.fy_end,  
				expiration_date, exercise_price, 
				remaining_term, executive_id, company_id, a.executive_id2, 
				a.expdate,
				b.stkprc, a.number_of_securities, value, 
				b.fy_end AS fy_end2 FORMAT MMDDYY10.,
				b.number_of_securities AS number_of_securities2
	FROM		missing_stkprc a 
				LEFT JOIN 	exercisableoptions2 
					(KEEP = executive_id2 fy_end stkprc expdate number_of_securities
					WHERE = (expdate ^= . AND stkprc ^= .)) b
	ON			a.executive_id2 = b.executive_id2 AND a.expdate = b.expdate
	GROUP BY	executive, company, ticker, a.fy_end,  
				expiration_date, exercise_price, remaining_term, executive_id, company_id, 
				a.executive_id2, a.expdate, a.number_of_securities, value
	HAVING		ABS(a.fy_end - b.fy_end) = MIN(ABS(a.fy_end - b.fy_end))
	ORDER BY	executive, company, ticker, fy_end, expiration_date, 
				exercise_price, remaining_term, executive_id, company_id, executive_id2, 
				expdate, number_of_securities, value, number_of_securities2 DESC;
QUIT;

DATA missing_stkprc; 
	SET missing_stkprc;
	BY executive company ticker fy_end expiration_date 
		exercise_price remaining_term executive_id company_id executive_id2 
		expdate number_of_securities value DESCENDING number_of_securities2;
	IF FIRST.value;
RUN;

* merge the matched missing one back with the unmissing one;
DATA exercisableoptions3 unfilled;
	SET exercisableoptions2 (WHERE = (expdate = . OR stkprc ^= .)) missing_stkprc (DROP = fy_end2 number_of_securities2);
	OUTPUT exercisableoptions3;
	IF expdate = . OR stkprc = . THEN OUTPUT unfilled;
RUN;

* Match exercisable options with earlier newly awarded options;
* using non-missing strike price/expiration date/number of options to fill in missing strike prices and expiration dates;

* require that the exercisable option to be later than the newly awarded option date;
PROC SQL;
	CREATE TABLE filled_with_new_options AS 
	SELECT		COALESCE(a.stkprc, c.stkprc_aw, d.stkprc_aw) AS stkprc, 
				COALESCE(a.expdate, b.expdate_aw, d.expdate_aw) AS expdate FORMAT MMDDYY10., 
				b.number_of_options AS number_of_options_exp,
				c.number_of_options AS number_of_options_stk,
				d.number_of_options AS number_of_options_num,
				a.*
	FROM		unfilled a 
				LEFT JOIN newlyawardedoptions 
					(KEEP = executive_id2 fy_end stkprc_aw expdate_aw number_of_options) b
	ON			a.executive_id2 = b.executive_id2 AND a.stkprc = b.stkprc_aw AND a.fy_end >= b.fy_end
				LEFT JOIN newlyawardedoptions 
					(KEEP = executive_id2 fy_end stkprc_aw expdate_aw number_of_options) c
	ON			a.executive_id2 = c.executive_id2 AND a.expdate = c.expdate_aw AND a.fy_end >= c.fy_end	
				LEFT JOIN newlyawardedoptions 
					(KEEP = executive_id2 fy_end stkprc_aw expdate_aw number_of_options) d
	ON			a.executive_id2 = d.executive_id2 AND a.number_of_securities = d.number_of_options 
				AND a.fy_end >= d.fy_end
	ORDER BY	executive, company, ticker, fy_end, expiration_date, exercise_price, remaining_term, 
				executive_id, company_id, executive_id2, number_of_securities, value, 
				number_of_options_exp DESC, number_of_options_num DESC;
QUIT;

* remove duplicates by picking the one with the largest number_of_options;
DATA filled_with_new_options;
	SET filled_with_new_options;
	BY executive company ticker fy_end expiration_date exercise_price remaining_term 
		executive_id company_id executive_id2 number_of_securities value 
		DESCENDING number_of_options_exp DESCENDING number_of_options_num;
	IF FIRST.value;
RUN;

* fill in missing term with expiration date and grant date;
DATA exercisableoptions4;
	SET exercisableoptions3 (WHERE = (expdate ^= . AND stkprc ^= .)) filled_with_new_options;
	IF expdate = . OR stkprc = . OR number_of_securities = . THEN DELETE;	
	filled_term = COALESCE(remaining_term, (expdate - fy_end) / 365); 
	KEEP company company_id executive_id2 executive fy_end remaining_term
		number_of_securities expdate stkprc filled_term;
RUN;

* verify that expdate-stkprc with difference in term and remaining_term should use the remaining_term;
PROC SQL;
	CREATE TABLE dup_expdate_stkprc_diff_term AS
	SELECT		*
	FROM		exercisableoptions4
	GROUP BY	fy_end, company_id, executive_id2, expdate, stkprc
	HAVING		COUNT(UNIQUE filled_term) > 1;
QUIT;

* Replace term with the Remaining_Term matched by STKPRC/EXPDATE;
PROC SQL;
	CREATE TABLE exercisableoptions AS
	SELECT	COALESCE(b.remaining_term, a.filled_term) AS term, a.*
	FROM	exercisableoptions4 a LEFT JOIN 
			dup_expdate_stkprc_diff_term (KEEP = fy_end company_id executive_id2 expdate stkprc remaining_term 
				WHERE = (Remaining_Term ^= .)) b
	ON		a.fy_end = b.fy_end AND a.company_id = b.company_id AND a.executive_id2 = b.executive_id2 
			AND a.expdate = b.expdate AND a.stkprc = b.stkprc;
QUIT;

* Group exercisable options by strike price/expiration date;
* Group multiple options with the same EXPDATE/STKPRC into one;
PROC SQL;
	CREATE TABLE exercisableoptions_sum AS
	SELECT		company, company_id, executive_id2, executive, fy_end, expdate, stkprc, term,
				COUNT(*) AS n_multiple, SUM(number_of_securities) AS number
	FROM		exercisableoptions
	GROUP BY	company, company_id, executive_id2, executive, fy_end, expdate, stkprc, term;
QUIT;


* vesting option number (t) = unvested options (t-1) + new options (t) - unvested options (t);
* Combine unexercisable options in t-1 and newly awarded options in t;
DATA lagoptions1;
	SET unexercisableoptions_sum (KEEP =  company_id executive_id2 executive fy_end expdate stkprc number term IN = uo)
		newlyawardedoptions_sum (KEEP = company_id executive_id2 executive fy_end expdate stkprc number IN = aw);
	IF uo THEN DO;
		lagyear = YEAR(fy_end);
		uooraw = 'UO';
	END;
	IF aw THEN DO;
		lagyear = YEAR(fy_end) - 1;
		uooraw = 'AW';
	END;
	fyr = MONTH(fy_end);
RUN;

* Combine unexercisable options in the last year and newly awarded options in the current year with the same EXPDATE/STKPRC combination;
PROC SQL;
	CREATE TABLE dupkeyoptions AS
	SELECT		company_id, executive_id2, executive, lagyear, expdate, stkprc,
				SUM(number) AS number2, 1 AS combined
	FROM		lagoptions1 
	GROUP BY	company_id, executive_id2, executive, lagyear, expdate, stkprc
	HAVING		COUNT(*) > 1;
QUIT;

* for the duplicate key options, keep the UO one but change the number to the combined number;
PROC SQL;
	CREATE TABLE lagoptions2 AS
	SELECT	COALESCE(number2, number) AS number, a.*
	FROM	lagoptions1 a LEFT JOIN dupkeyoptions b
	ON		a.company_id = b.company_id AND a.executive_id2 = b.executive_id2
			AND a.executive = b.executive AND a.lagyear = b.lagyear AND a.expdate = b.expdate
			AND a.stkprc = b.stkprc
	HAVING	b.company_id = . OR (uooraw = 'UO' AND b.company_id ^= .);
QUIT;

* Match with unexercisable options in t to calculate newly vested options;
* match with unexercisable options in year t, if no unexercisable options, assume it is zero, also calculate next fy_end and term as of that date;
PROC SQL;
	CREATE TABLE newlyvestedoptions AS
	SELECT	a.*, 
			b.fy_end AS fy_end_nextyear FORMAT MMDDYY10.,
			(b.fy_end = .) AS missing,
			b.term AS term_nextyear,
			COALESCE(b.number, 0) AS number_nextyear,
			a.number - CALCULATED number_nextyear AS vestedoptions	
	FROM	lagoptions2 a LEFT JOIN unexercisableoptions_sum b
	ON		a.company_id = b.company_id AND a.executive_id2 = b.executive_id2 AND a.executive = b.executive
			AND a.lagyear = YEAR(b.fy_end) - 1 AND a.expdate = b.expdate AND a.stkprc = b.stkprc;
QUIT;

* if no unexercisable options in the next year, then fy_end_nextyear and term_nextyear are missing;
* in this case, assume fy_end_nextyear is 12 months after current fy_end;
* assume term_next year is the distance between expiration date and fy_end_nextyear;
DATA newlyvestedoptions;
	SET newlyvestedoptions;
	IF missing THEN DO;
		IF uooraw = 'UO' THEN fy_end_nextyear = INTNX('MONTH', fy_end, 12, 'END');
		ELSE fy_end_nextyear = fy_end;
		term_nextyear = (expdate - fy_end_nextyear) / 365;
	END;
RUN;

* set negative vested option numbers to zero;
DATA newlyvestedoptions;
	SET newlyvestedoptions;
	DROP fy_end;
	KEEP company_id executive_id2 executive lagyear expdate stkprc 
		fy_end_nextyear number number_nextyear vestedoptions term_nextyear fyr; 
	* set negative vestedoptions to zero;
	IF vestedoptions < 0 THEN vestedoptions = 0;
	RENAME 	number = lagnumber
			number_nextyear = number
			fy_end_nextyear = fy_end
			term_nextyear = term;
	LABEL 	number = lagnumber
			number_nextyear = number
			fy_end_nextyear = fy_end
			term_nextyear = term;
RUN;

* Options already vested and unvested;
* Retrieve Exercisable Options from OEND and EDND;
* exercisable options from EDND;
DATA exercisableoptions1;	
	SET v.ednd (KEEP = company_id executive_id2 executive fy_end company ticker 
		total_exercisable_options total_value_of_exercisable_optio);
RUN;

* calculate exercisable option number from OEND;
PROC SQL;
	CREATE TABLE exercisableoptions_sum AS
	SELECT		company_id, executive_id2, fy_end, 
				SUM(number_of_securities) AS total_exercisable_options2
	FROM		v.oend
	WHERE		equity_type = 'Exercisable Options'
	GROUP BY	company_id, executive_id2, fy_end;
QUIT;

* match the exercisable options from two places;
PROC SQL;
	CREATE TABLE alreadyvestedoptions AS
	SELECT	a.*, b.total_exercisable_options2
	FROM	exercisableoptions1 a LEFT JOIN exercisableoptions_sum b
	ON		a.company_id = b.company_id AND a.executive_id2 = b.executive_id2 AND a.fy_end = b.fy_end;
QUIT;

* Retrieve Unexercisable Options from OEND and EDND;
* unexercisable options from EDND;
DATA  unexercisableoptions1;	
	SET v.ednd (KEEP = company_id executive_id2 executive fy_end company ticker 
		total_unexercisable_options total_value_of_unexercisable_opt);
RUN;

* calculate unexercisable option number from OEND;
PROC SQL;
	CREATE TABLE unexercisableoptions_sum AS
	SELECT		company_id, executive_id2, fy_end, 
				SUM(number_of_securities) AS total_unexercisable_options2
	FROM		v.oend
	WHERE		equity_type = 'Unexercisable Options' 
	GROUP BY	company_id, executive_id2, fy_end;
QUIT;

* match the unvested options from the two places;
PROC SQL;
	CREATE TABLE unvestedoptions AS
	SELECT	a.*, b.total_unexercisable_options2
	FROM	unexercisableoptions1 a LEFT JOIN unexercisableoptions_sum b
	ON		a.company_id = b.company_id AND a.executive_id2 = b.executive_id2 AND a.fy_end = b.fy_end;
QUIT;


* Black Scholes assumptions from Equilar, Compustat, ExecuComp;
* BS assumptions from OAND;
DATA bsinputsequilar; 
	SET v.oand (KEEP = fyear company_id fy_end volatility_ risk_free_rate__ dividendyield grant_date);
RUN;

* Keep year t first grant inputs and year t-1 last grant inputs;
PROC SORT DATA = bsinputsequilar; 
	BY company_id fy_end grant_date;
RUN;

* for year t, keep the BS input of the first grant;
DATA bsinputs;
	SET bsinputsequilar; 
	BY company_id fy_end grant_date; 
	IF FIRST.fy_end; 
	DROP grant_date fy_end; 
	RENAME company_id = company_id;
	LABEL company_id = company_id; 
RUN;

* for year t-1, keep the BS input of the last grant;
DATA lagbsinputs;
	SET bsinputsequilar; 
	BY company_id fy_end grant_date; 
	IF LAST.fy_end; 
	DROP grant_date fy_end;
	RENAME 	fyear = lagfyear
			volatility_ = lagvolatility_
			risk_free_rate__ = lagrisk_free_rate__
			dividendyield = lagdividendyield
			company_id = company_id;
RUN;

* ExecuComp & Compustat BS assumptions;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME compna '/wrds/comp/sasdata/naa'; 
LIBNAME exec '/wrds/comp/sasdata/execcomp';

* B/S assumptions from ExecuComp;
DATA bsinputsexec;
	SET exec.codirfin (KEEP = year gvkey bs_volatility bs_yield);
	bs_yield = bs_yield / 100;
	WHERE year >= 2004;
RUN;

* B/S assumptions from Compustat;
DATA bsinputscomp;
	SET compna.funda (KEEP = gvkey fyear optdr optrfr optvol indfmt datafmt popsrc consol);
	optdr = optdr / 100;
	optrfr = optrfr / 100;
	optvol = optvol / 100;
	DROP indfmt datafmt popsrc consol;
	WHERE indfmt = 'INDL' AND datafmt = 'STD' AND popsrc = 'D' AND consol = 'C' AND fyear >= 2004;
RUN;

* merge the two;
PROC SQL;
	CREATE TABLE bsinputsboth AS
	SELECT	COALESCE(a.gvkey, b.gvkey) AS gvkey,
			COALESCE(a.year, b.fyear) AS fyear,
			bs_volatility, bs_yield,
			optdr, optrfr, optvol
	FROM	bsinputsexec a FULL JOIN bsinputscomp b
	ON		a.gvkey = b.gvkey AND a.year = b.fyear;
QUIT;

DATA bsinputsboth; 
	SET bsinputsboth; 
	IF bs_volatility = . AND bs_yield = . AND optdr = . AND optrfr = . AND optvol = . THEN DELETE; 
RUN;

PROC DOWNLOAD DATA = bsinputsboth OUT = bsinputsboth; RUN; 

ENDRSUBMIT;

* compustat CRSP link file;

/**********************************************************************************
* Program         :   ccm_lnktable.sas                                            *
* Author          :   WRDS/M. Duan                                                *
* Date Created    :   April 2009                                                  *
*                                                                                 *
* Usage           :                                                               *
*   This program shows the main processing part of the CCM web queries.           *
* XpressFeed version CCM product provides more detailed information on linking    *
* history, in the meanwhile it also populates much more observations than the old *
* version, among which there are some consecutive records (consecutive linking    *
* date range) with the same GVKEY/IID/PERMNO pair but different LINKTYPE or       *
* USEDFLAG or LINKPRIM value. This program will collapse such consecutive rows    *
* and combine them into one.                                                      *
* An example might be helpful:                                                    *
* Two records can be found in the link table (ccmxpf_linktable):                  *
* GVKEY  LPERMNO    LINKDT  LINKENDDT  LINKPRIM  LIID  LINKTYPE  LPERMCO USEDFLAG *
* 001043   18980  19490801   19581230         C   00X        LN    20009       -1 *
* 001043   18980  19581231   19620130         C   00X        LC    20009        1 *
* After this program, there is only one line left:                                *
* 001043   18980  19490801   19620130         C   00X        LC    20009        1 *
*                                                                                 *
* Notes:                                                                          *
*   Include LIID in all the BY statements ONLY IF the LNK table is to be used     *
* to merge with Security Monthly table. Otherwise, LIID should be removed from    *
* the BY statements.                                                         *
***********************************************************************************/
 
*  if using SAS Connect, use the following:;
 
    %let wrds = wrds-cloud.wharton.upenn.edu 4016;
    options comamid=TCP remote=wrds;
    signon username=_prompt_;
 
    rsubmit;
 
libname ccm '/wrds/crsp/sasdata/a_ccm';
 
/* ---------------------------------------------------------------- */
/* create the "andlu" macro variable that indicates if the usedflag */
/* = 1 links will be used exclusively or not.                       */
/* Set to %str( ) if using both usedflag=1 or -1.                   */
/* ---------------------------------------------------------------- */
 
%let andlu = %str(usedflag = 1);
 
/* ---------------------------------------------------------------- */
/* create the "andlt" macro variable that will keep only the        */
/*  desired linktypes.                                              */
/* ---------------------------------------------------------------- */
 
%let andlt = %str(linktype in ("LC" "LN" "LU" "LX" "LD" "LS"));

/* ---------------------------------------------------------------- */
/* create the "andlp" macro variable that will keep only the        */
/*  desired linkprims.                                              */
/* ---------------------------------------------------------------- */
 
%let andlp = %str(linkprim in ("P" "C"));
 
proc sql;
     create table lnk1 as select *
     from ccm.ccmxpf_linktable
     where &andlu and &andlt and &andlp
      order by gvkey, lpermno, lpermco, linkdt, linkenddt
      ;
quit;
    
/* ---------------------------------------------------------------- */
/* Processing LNK table to collapse                                 */
/* ---------------------------------------------------------------- */
 
data lnk2;
    set lnk1;
    by gvkey lpermno lpermco linkdt linkenddt;
    format prev_ldt prev_ledt yymmddn8.;
    retain prev_ldt prev_ledt;
    if first.lpermno then do;
        if last.lpermno then do;
            /* Keep this obs if it's the first and last matching permno pair */
            output;           
        end;
        else do;
            /* If it's the first but not the last pair, retain the dates for future use */
            prev_ldt = linkdt;
            prev_ledt = linkenddt;
            output;
            end;
        end;   
    else do;
        if linkdt=prev_ledt+1 or linkdt=prev_ledt then do;
            /* If the date range follows the previous one, assign the previous linkdt value
               to the current - will remove the redundant in later steps. Also retain the
               link end date value */
            linkdt = prev_ldt;
            prev_ledt = linkenddt;
            output;
            end;
        else do;
            /* If it doesn't fall into any of the above conditions, just keep it and retain the
               link date range for future use*/
            output;
            prev_ldt = linkdt;
            prev_ledt = linkenddt;
            end;
        end;
    drop prev_ldt prev_ledt;
run;
 
DATA lnk;
	SET lnk2;
	BY gvkey lpermno linkdt;
	IF LAST.linkdt;
  /* remove redundant observations with identical LINKDT (result of the previous data step), so that
     each consecutive pair of observations will have either different GVKEY-IID-PERMNO match, or
     non-consecutive link date range
   */
RUN;

PROC DOWNLOAD DATA = lnk OUT = compustat_crsp_link;
RUN;
 
ENDRSUBMIT;

* BS Assumptions Newly Vested Options;
* Merge Shares with newly vested options;

* merge shares with newly vested options; 
* use shares because it has an index of firm-year-managers;
PROC SQL;
	CREATE TABLE newlyvested1 AS
	SELECT	a.*, b.*, a.fyear - 1 AS lagfyear
	FROM	v.shares (KEEP = executive gvkey fyear datadate fy_end company_id 
			lagfy_end executive_id2 lagfinalprc finalprc) a LEFT JOIN newlyvestedoptions b
	ON		a.company_id = b.company_id AND a.executive_id2 = b.executive_id2 
			AND a.fy_end = b.fy_end;
QUIT;

* Merge with BS assumptions;
* merge with Equilar BS assumptions;
PROC SQL;
	CREATE TABLE newlyvested2 AS
	SELECT	a.*, b.*, c.*, term + 1 AS lagterm
	FROM	newlyvested1 a LEFT JOIN bsinputs b
	ON		a.company_id = b.company_id AND a.fyear = b.fyear
			LEFT JOIN lagbsinputs c
	ON		a.company_id = c.company_id AND a.lagfyear = c.lagfyear;
QUIT;

* merge with ExecuComp/Compustat BS assumptions;
* pick equilar over execucomp over compustat;
* Equilar(t-1) -> Equilar(t) -> ExecuComp(t-1) -> ExecuComp(t) -> Compustat(t-1) -> Compustat(t);
PROC SQL;
	CREATE TABLE newlyvested3 AS
	SELECT	a.*, b.*, 
			c.bs_volatility AS lagbs_volatility,
			c.bs_yield AS lagbs_yield,
			c.optdr AS lagoptdr,
			c.optrfr AS lagoptrfr,
			c.optvol AS lagoptvol,

			COALESCE(lagdividendyield, dividendyield, c.bs_yield, b.bs_yield, c.optdr, b.optdr) AS finaldivyield1,
			COALESCE(lagvolatility_, volatility_, c.bs_volatility, b.bs_volatility, c.optvol, b.optvol) AS finalvol1,
			COALESCE(lagrisk_free_rate__, risk_free_rate__, c.optrfr, b.optrfr) AS finalrfr1,

			(CALCULATED finaldivyield1 = .) AS missingdiv,
			(CALCULATED finalvol1 = .) AS missingvol,
			(CALCULATED finalrfr1 = .) AS missingrfr
	FROM	newlyvested2 a LEFT JOIN bsinputsboth b
	ON		a.gvkey = b.gvkey AND a.fyear = b.fyear
			LEFT JOIN bsinputsboth c
	ON		a.gvkey = c.gvkey AND a.lagfyear = c.fyear;
QUIT;

* Try to fill in missing volatility;
* generate list of gvkey-fyear that we need volatility;
PROC SORT DATA = newlyvested3 OUT = gvkeyindex1 (KEEP = gvkey datadate fyear) NODUPKEY;
	BY gvkey fyear;
	WHERE missingvol AND vestedoptions > 0;
RUN;

DATA gvkeyindex1;
	SET gvkeyindex1;
	lagdatadate = INTNX('MONTH', datadate, -12, 'END');
	FORMAT lagdatadate MMDDYY10.;
RUN;

* obtain permno;
PROC SQL;
	CREATE TABLE permnoindex1 AS
	SELECT	*
	FROM	gvkeyindex1 a, compustat_crsp_link b
	WHERE	a.gvkey = b.gvkey;
QUIT;

* calculate standard deviation of daily returns from CRSP;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME crspsd '/wrds/crsp/sasdata/a_stock'; 
PROC UPLOAD DATA = permnoindex1 OUT = permnoindex1; RUN;

* Merge with CRSP daily stock file and calculate annalized standard deviation of daily return;
PROC SQL;
	CREATE TABLE vol1 AS
	SELECT		gvkey, permno, fyear, 
				COUNT(*) AS n_ret, STD(LOG(1 + ret)) AS std_ret,
				CALCULATED std_ret * SQRT(252) AS annualret
	FROM		permnoindex1 a, crspsd.dsf (KEEP = date permno ret prc) b
	WHERE		a.lpermno = b.permno AND b.date BETWEEN a.lagdatadate - 365 * 3 AND a.lagdatadate
	GROUP BY	gvkey, permno, fyear
	ORDER BY	gvkey, fyear, annualret;
QUIT;
PROC DOWNLOAD DATA = vol1 OUT = vol1; RUN;

ENDRSUBMIT;

* for these stocks, pick the bigger return volatility;
PROC SQL;
	CREATE TABLE dualstock AS
	SELECT		*
	FROM		vol1
	GROUP BY	gvkey, fyear
	HAVING		COUNT(*) > 1
	ORDER BY	gvkey, fyear, annualret;
QUIT;

DATA filledvol1;
	SET vol1;
	BY gvkey fyear annualret;
	IF LAST.fyear;
RUN;

* Merge back to newlyvested4; 
PROC SQL;
	CREATE TABLE newlyvested4 AS
	SELECT	*
	FROM	newlyvested3 a LEFT JOIN filledvol1 (KEEP = gvkey fyear annualret) b
	ON		a.gvkey = b.gvkey AND a.fyear = b.fyear;
QUIT;

* if missing, set it to annualret from CRSP;
DATA newlyvested4;
	SET newlyvested4;
	IF missingvol THEN DO;
		finalvol1 = annualret;
	END;
RUN;

* Try one more time to fill in VOL with current year VOL;
* need volatility for these;
PROC SORT DATA = newlyvested4 OUT = gvkeyindex2 (KEEP = gvkey datadate fyear) NODUPKEY;
	BY gvkey fyear;
	WHERE finalvol1 = . AND vestedoptions > 0;
RUN;

DATA gvkeyindex2;
	SET gvkeyindex2;
	lagdatadate = INTNX('MONTH', datadate, -12, 'END');
	FORMAT lagdatadate MMDDYY10.;
RUN;

* obtain permno;
PROC SQL;
	CREATE TABLE permnoindex2 AS
	SELECT	*
	FROM	gvkeyindex2 a, compustat_crsp_link b
	WHERE	a.gvkey = b.gvkey;
QUIT;


* calculate standard deviation of daily returns from CRSP;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME crspsd '/wrds/crsp/sasdata/a_stock'; 
PROC UPLOAD DATA = permnoindex2 OUT = permnoindex2; RUN;

* Merge with CRSP daily stock file and calculate annalized standard deviation of daily return;
PROC SQL;
	CREATE TABLE vol2 AS
	SELECT		gvkey, permno, fyear, 
				COUNT(*) AS n_ret, STD(LOG(1 + ret)) AS std_ret,
				CALCULATED std_ret * SQRT(252) AS annualret
	FROM		permnoindex2 a, crspsd.dsf (KEEP = date permno ret prc) b
	WHERE		a.lpermno = b.permno AND b.date BETWEEN a.lagdatadate - 365 * 4 AND a.lagdatadate + 365
	GROUP BY	gvkey, permno, fyear
	ORDER BY	gvkey, fyear, annualret;
QUIT;

PROC DOWNLOAD DATA = vol2 OUT = vol2; RUN;
ENDRSUBMIT;

PROC SORT DATA = vol2 NODUPKEY; BY fyear gvkey; RUN;

* try to fill in the missing volatilities;
PROC SQL;
	CREATE TABLE newlyvested5 AS
	SELECT	a.*, b.annualret AS annualret_extwin
	FROM	newlyvested4 a LEFT JOIN vol2 (KEEP = gvkey fyear annualret) b
	ON		a.gvkey = b.gvkey AND a.fyear = b.fyear;
QUIT;

* if missing, set it to annualret from CRSP;
DATA newlyvested5;
	SET newlyvested5;
	IF finalvol1 = . THEN DO;
		finalvol1 = annualret_extwin;
	END;
RUN;

* Try to fill in missing risk free rates;
PROC SORT DATA = newlyvested5 OUT = gvkeyindex3 (KEEP = gvkey datadate fyear lagterm) NODUPKEY;
	BY fyear gvkey lagterm datadate;
	WHERE missingrfr AND vestedoptions > 0;
RUN;

/*
risk free interest rate source:
v.interestrates_2017:

Data List: Interest Rates
Data Updated: 2017-05-23

FRED (Federal Reserve Economic Data)
Link: https://fred.stlouisfed.org
Help: https://fred.stlouisfed.org/help-faq
Economic Research Division
Federal Reserve Bank of St. Louis
*/

PROC SQL;
	CREATE TABLE filledinterest AS
	SELECT		a.*, b.*
	FROM		gvkeyindex3 a, 
				v.interestrates_2017 (WHERE = (YEAR(date) >= 2004 AND dgs1 ^= .)
				KEEP = date DGS1 DGS10 DGS1MO DGS2 DGS20 DGS3 DGS30 DGS3MO DGS5 DGS6MO DGS7) b
	WHERE		b.date BETWEEN 
				INTNX('MONTH', datadate, -12, 'END') - 5 AND INTNX('MONTH', datadate, -12, 'END')
	GROUP BY	fyear, gvkey, lagterm, datadate
	HAVING		date = MAX(date);
QUIT;

DATA filledinterest; 
	SET filledinterest;
	mindis = MIN(ABS(lagterm - 1), ABS(lagterm - 2), ABS(lagterm - 3), ABS(lagterm - 5), ABS(lagterm - 7),
		ABS(lagterm - 10), ABS(lagterm - 20), ABS(lagterm - 30), ABS(lagterm - 0.083), 
		ABS(lagterm - 0.25), ABS(lagterm - 0.5));
	IF mindis = ABS(lagterm - 1) THEN rfr = DGS1 * 0.01;
	ELSE IF mindis = ABS(lagterm - 2) THEN rfr = DGS2 * 0.01;
	ELSE IF mindis = ABS(lagterm - 3) THEN rfr = DGS3 * 0.01;
	ELSE IF mindis = ABS(lagterm - 5) THEN rfr = DGS5 * 0.01;
	ELSE IF mindis = ABS(lagterm - 7) THEN rfr = DGS7 * 0.01;
	ELSE IF mindis = ABS(lagterm - 10) THEN rfr = DGS10 * 0.01;
	ELSE IF mindis = ABS(lagterm - 20) THEN rfr = DGS20 * 0.01;
	ELSE IF mindis = ABS(lagterm - 30) THEN rfr = DGS30 * 0.01;
	ELSE IF mindis = ABS(lagterm - 0.083) THEN rfr = DGS1MO * 0.01;
	ELSE IF mindis = ABS(lagterm - 0.25) THEN rfr = DGS3MO * 0.01;
	ELSE IF mindis = ABS(lagterm - 0.5) THEN rfr = DGS6MO * 0.01; 
RUN;
	
* fill in the missing ones;
PROC SQL;
	CREATE TABLE newlyvested6 AS
	SELECT	a.*, b.rfr
	FROM	newlyvested5 a LEFT JOIN filledinterest b
	ON		a.fyear = b.fyear AND a.gvkey = b.gvkey AND a.lagterm = b.lagterm;
QUIT;

DATA newlyvested6;
	SET newlyvested6;
	IF missingrfr THEN DO;
		finalrfr1 = rfr;
	END;
RUN;

* Try to fill in missing dividend yield;
PROC SORT DATA = newlyvested6 OUT = gvkeyindex4 (KEEP = gvkey fyear) NODUPKEY;
	BY gvkey fyear;
	WHERE missingdiv AND vestedoptions > 0;
RUN;

%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME compna '/wrds/comp/sasdata/naa'; 
PROC UPLOAD DATA = gvkeyindex4 OUT = gvkeyindex4; RUN;

* Merge with Compustat to obtain dividend yield;
* use the average of the last 5 years;
PROC SQL;
	CREATE TABLE dividends AS
	SELECT		a.gvkey, a.fyear, 
				AVG(COALESCE(dvpsp_f / prcc_f, dvc / (prcc_f * csho))) AS mean_div
	FROM		gvkeyindex4 a, 
				compna.funda (KEEP = gvkey fyear dvt dv dvc dvpsp_f prcc_f csho
				indfmt datafmt popsrc consol
				WHERE = (indfmt = 'INDL' AND datafmt = 'STD' AND popsrc = 'D' 
				AND consol = 'C' AND fyear >= 1998)) b
	WHERE		a.gvkey = b.gvkey AND b.fyear BETWEEN a.fyear - 5 AND a.fyear - 1
	GROUP BY	a.gvkey, a.fyear
	HAVING		mean_div ^= .;
QUIT;

PROC DOWNLOAD DATA = dividends OUT = dividends; RUN;
ENDRSUBMIT;

PROC SQL;
	CREATE TABLE newlyvested7 AS
	SELECT	*
	FROM	newlyvested6 a LEFT JOIN dividends b
	ON		a.gvkey = b.gvkey AND A.fyear = b.fyear;
QUIT;

DATA newlyvested7;
	SET newlyvested7;
	IF missingdiv THEN DO;
		finaldivyield1 = mean_div;
	END;
RUN;

* set missing dividend yield to zero;
DATA bsassumptions_newlyvestedoptions;
	SET newlyvested7;
	finaldivyield1 = COALESCE(finaldivyield1, 0);
RUN;

* BS Assumptions Already Vested Options;
* Pull out the already vested options from OEND;
* merge shares with already vested options at the end of the previous year; 
* use shares because it has an index of firm-year-managers;
PROC SQL;
	CREATE TABLE alreadyvested1 AS
	SELECT	a.*, a.fyear - 1 AS lagfyear,
			expiration_date AS expdate, exercise_price AS stkprc, 
			number_of_securities AS number, remaining_term AS term, value
	FROM	v.shares (KEEP = executive gvkey fyear datadate fy_end company_id lagfy_end executive_id2
			WHERE = (lagfy_end ^= .)) a 
			LEFT JOIN v.oend (KEEP = company_id executive_id2 fy_end expiration_date exercise_price number_of_securities
			remaining_term value equity_type trans_pd
			WHERE = (equity_type = 'Exercisable Options')) b
	ON		a.company_id = b.company_id AND a.executive_id2 = b.executive_id2 AND a.lagfy_end = b.fy_end;
QUIT;

* Merge with BS assumptions;
* merge with Equilar BS assumptions;
PROC SQL;
	CREATE TABLE alreadyvested2 AS
	SELECT	a.*, b.*, c.*
	FROM	alreadyvested1 a LEFT JOIN bsinputs b
	ON		a.company_id = b.company_id AND a.fyear = b.fyear
			LEFT JOIN lagbsinputs c
	ON		a.company_id = c.company_id AND a.lagfyear = c.lagfyear;
QUIT;

* merge with ExecuComp/Compustat BS assumptions;
* pick equilar over execucomp over compustat;
* Equilar(t-1) -> Equilar(t) -> ExecuComp(t-1) -> ExecuComp(t) -> Compustat(t-1) -> Compustat(t);
PROC SQL;
	CREATE TABLE alreadyvested3 AS
	SELECT	a.*, b.*, 
			c.bs_volatility AS lagbs_volatility,
			c.bs_yield AS lagbs_yield,
			c.optdr AS lagoptdr,
			c.optrfr AS lagoptrfr,
			c.optvol AS lagoptvol,

			COALESCE(lagdividendyield, dividendyield, c.bs_yield, b.bs_yield, c.optdr, b.optdr) AS finaldivyield1,
			COALESCE(lagvolatility_, volatility_, c.bs_volatility, b.bs_volatility, c.optvol, b.optvol) AS finalvol1,
			COALESCE(lagrisk_free_rate__, risk_free_rate__, c.optrfr, b.optrfr) AS finalrfr1,

			(CALCULATED finaldivyield1 = .) AS missingdiv,
			(CALCULATED finalvol1 = .) AS missingvol,
			(CALCULATED finalrfr1 = .) AS missingrfr
	FROM	alreadyvested2 a LEFT JOIN bsinputsboth b
	ON		a.gvkey = b.gvkey AND a.fyear = b.fyear
			LEFT JOIN bsinputsboth c
	ON		a.gvkey = c.gvkey AND a.lagfyear = c.fyear;
QUIT;

* Try to fill in missing volatility;
* generate list of gvkey-fyear that we need volatility;
PROC SORT DATA = alreadyvested3 OUT = gvkeyindex1 (KEEP = gvkey datadate fyear) NODUPKEY;
	BY gvkey fyear;
	WHERE missingvol AND number > 0;
RUN;

DATA gvkeyindex1;
	SET gvkeyindex1;
	lagdatadate = INTNX('MONTH', datadate, -12, 'END');
	FORMAT lagdatadate MMDDYY10.;
RUN;

* obtain permno;
PROC SQL;
	CREATE TABLE permnoindex1 AS
	SELECT	*
	FROM	gvkeyindex1 a, compustat_crsp_link b
	WHERE	a.gvkey = b.gvkey;
QUIT;

* calculate standard deviation of daily returns from CRSP;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME crspsd '/wrds/crsp/sasdata/a_stock'; 
PROC UPLOAD DATA = permnoindex1 OUT = permnoindex1; RUN;

* Merge with CRSP daily stock file and calculate annalized standard deviation of daily return;
PROC SQL;
	CREATE TABLE vol1 AS
	SELECT		gvkey, permno, fyear, 
				COUNT(*) AS n_ret, STD(LOG(1 + ret)) AS std_ret,
				CALCULATED std_ret * SQRT(252) AS annualret
	FROM		permnoindex1 a, crspsd.dsf (KEEP = date permno ret prc) b
	WHERE		a.lpermno = b.permno AND b.date BETWEEN a.lagdatadate - 365 * 3 AND a.lagdatadate
	GROUP BY	gvkey, permno, fyear
	ORDER BY	gvkey, fyear, annualret;
QUIT;

PROC DOWNLOAD DATA = vol1 OUT = vol1; RUN;
ENDRSUBMIT;

* pick the bigger return volatility for these;
PROC SQL;
	CREATE TABLE dualstock AS
	SELECT		*
	FROM		vol1
	GROUP BY	gvkey, fyear
	HAVING		COUNT(*) > 1
	ORDER BY	gvkey, fyear, annualret;
QUIT;

DATA filledvol1;
	SET vol1;
	BY gvkey fyear annualret;
	IF LAST.fyear;
RUN;

* Merge back to alreadyvested14; 
PROC SQL;
	CREATE TABLE alreadyvested4 AS
	SELECT	*
	FROM	alreadyvested3 a LEFT JOIN filledvol1 (KEEP = gvkey fyear annualret) b
	ON		a.gvkey = b.gvkey AND a.fyear = b.fyear;
QUIT;

* if missing, set it to annualret from CRSP;
DATA alreadyvested4;
	SET alreadyvested4;
	IF missingvol THEN DO;
		finalvol1 = annualret;
	END;
RUN;

* Try one more time to fill in VOL with current year VOL;
* need volatility for these;
PROC SORT DATA = alreadyvested4 OUT = gvkeyindex2 (KEEP = gvkey datadate fyear) NODUPKEY;
	BY gvkey fyear;
	WHERE finalvol1 = . AND number > 0;
RUN;

DATA gvkeyindex2;
	SET gvkeyindex2;
	lagdatadate = INTNX('MONTH', datadate, -12, 'END');
	FORMAT lagdatadate MMDDYY10.;
RUN;

* obtain permno;
PROC SQL;
	CREATE TABLE permnoindex2 AS
	SELECT	*
	FROM	gvkeyindex2 a, compustat_crsp_link b
	WHERE	a.gvkey = b.gvkey;
QUIT;

* calculate standard deviation of daily returns from CRSP;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME crspsd '/wrds/crsp/sasdata/a_stock'; 
PROC UPLOAD DATA = permnoindex2 OUT = permnoindex2; RUN;

* Merge with CRSP daily stock file and calculate annalized standard deviation of daily return;
PROC SQL;
	CREATE TABLE vol2 AS
	SELECT		gvkey, permno, fyear, 
				COUNT(*) AS n_ret, STD(LOG(1 + ret)) AS std_ret,
				CALCULATED std_ret * SQRT(252) AS annualret
	FROM		permnoindex2 a, crspsd.dsf (KEEP = date permno ret prc) b
	WHERE		a.lpermno = b.permno AND b.date BETWEEN a.lagdatadate - 365 * 4 AND a.lagdatadate + 365
	GROUP BY	gvkey, permno, fyear
	ORDER BY	gvkey, fyear, annualret;
QUIT;

PROC DOWNLOAD DATA = vol2 OUT = vol2; RUN;
ENDRSUBMIT;

PROC SORT DATA = vol2 NODUPKEY; BY fyear gvkey; RUN;

* try to fill in the missing volatilities;
PROC SQL;
	CREATE TABLE alreadyvested5 AS
	SELECT	a.*, b.annualret AS annualret_extwin
	FROM	alreadyvested4 a LEFT JOIN vol2 (KEEP = gvkey fyear annualret) b
	ON		a.gvkey = b.gvkey AND a.fyear = b.fyear;
QUIT;

* if missing, set it to annualret from CRSP;
DATA alreadyvested5;
	SET alreadyvested5;
	IF finalvol1 = . THEN DO;
		finalvol1 = annualret_extwin;
	END;
RUN;

* Try to fill in missing risk free rates;
PROC SORT DATA = alreadyvested5 OUT = gvkeyindex3 (KEEP = gvkey datadate fyear term) NODUPKEY;
	BY fyear gvkey term datadate;
	WHERE missingrfr AND number > 0 AND term ^= .;
RUN;

* adding datadate to the key does not change anything;
PROC SQL;
	CREATE TABLE filledinterest AS
	SELECT		a.*, b.*
	FROM		gvkeyindex3 a, 
				v.interestrates_2017 (WHERE = (YEAR(date) >= 2004 AND dgs1 ^= .)
				KEEP = date DGS1 DGS10 DGS1MO DGS2 DGS20 DGS3 DGS30 DGS3MO DGS5 DGS6MO DGS7) b
	WHERE		b.date BETWEEN 
				INTNX('MONTH', datadate, -12, 'END') - 5 AND INTNX('MONTH', datadate, -12, 'END')
	GROUP BY	fyear, gvkey, term, datadate
	HAVING		date = MAX(date);
QUIT;

DATA filledinterest; 
	SET filledinterest;
	mindis = MIN(ABS(term - 1), ABS(term - 2), ABS(term - 3), ABS(term - 5), ABS(term - 7),
		ABS(term - 10), ABS(term - 20), ABS(term - 30), ABS(term - 0.083), 
		ABS(term - 0.25), ABS(term - 0.5));
	IF mindis = ABS(term - 1) THEN rfr = DGS1 * 0.01;
	ELSE IF mindis = ABS(term - 2) THEN rfr = DGS2 * 0.01;
	ELSE IF mindis = ABS(term - 3) THEN rfr = DGS3 * 0.01;
	ELSE IF mindis = ABS(term - 5) THEN rfr = DGS5 * 0.01;
	ELSE IF mindis = ABS(term - 7) THEN rfr = DGS7 * 0.01;
	ELSE IF mindis = ABS(term - 10) THEN rfr = DGS10 * 0.01;
	ELSE IF mindis = ABS(term - 20) THEN rfr = DGS20 * 0.01;
	ELSE IF mindis = ABS(term - 30) THEN rfr = DGS30 * 0.01;
	ELSE IF mindis = ABS(term - 0.083) THEN rfr = DGS1MO * 0.01;
	ELSE IF mindis = ABS(term - 0.25) THEN rfr = DGS3MO * 0.01;
	ELSE IF mindis = ABS(term - 0.5) THEN rfr = DGS6MO * 0.01; 
RUN;

* fill in the missing ones;
PROC SQL;
	CREATE TABLE alreadyvested6 AS
	SELECT	a.*, b.rfr
	FROM	alreadyvested5 a LEFT JOIN filledinterest b
	ON		a.fyear = b.fyear AND a.gvkey = b.gvkey AND a.term = b.term;
QUIT;

DATA alreadyvested6;
	SET alreadyvested6;
	IF missingrfr THEN DO;
		finalrfr1 = rfr;
	END;
RUN;

* Try to fill in missing dividend yield;
PROC SORT DATA = alreadyvested6 OUT = gvkeyindex4 (KEEP = gvkey fyear) NODUPKEY;
	BY gvkey fyear;
	WHERE missingdiv AND number > 0;
RUN;

%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME compna '/wrds/comp/sasdata/naa'; 
PROC UPLOAD DATA = gvkeyindex4 OUT = gvkeyindex4; RUN;

* Merge with Compustat to obtain dividend yield;
* use the average of the last 5 years;
PROC SQL;
	CREATE TABLE dividends AS
	SELECT		a.gvkey, a.fyear, 
				AVG(COALESCE(dvpsp_f / prcc_f, dvc / (prcc_f * csho))) AS mean_div
	FROM		gvkeyindex4 a, 
				compna.funda (KEEP = gvkey fyear dvt dv dvc dvpsp_f prcc_f csho
				indfmt datafmt popsrc consol
				WHERE = (indfmt = 'INDL' AND datafmt = 'STD' AND popsrc = 'D' 
				AND consol = 'C' AND fyear >= 1998)) b
	WHERE		a.gvkey = b.gvkey AND b.fyear BETWEEN a.fyear - 5 AND a.fyear - 1
	GROUP BY	a.gvkey, a.fyear
	HAVING		mean_div ^= .;
QUIT;

PROC DOWNLOAD DATA = dividends OUT = dividends; RUN;
ENDRSUBMIT;

PROC SQL;
	CREATE TABLE alreadyvested7 AS
	SELECT	*
	FROM	alreadyvested6 a LEFT JOIN dividends b
	ON		a.gvkey = b.gvkey AND A.fyear = b.fyear;
QUIT;

DATA alreadyvested7;
	SET alreadyvested7;
	IF missingdiv THEN DO;
		finaldivyield1 = mean_div;
	END;
RUN;

* set missing dividend yield to zero;
DATA bsassumptions_alreadyvestedopt;
	SET alreadyvested7;
	finaldivyield1 = COALESCE(finaldivyield1, 0);
RUN;

* BS Assumptions Unvested Options;
* Pull out the unvested options from OEND;
* merge shares with unvested options; 
* use shares because it has an index of firm-year-managers;
PROC SQL;
	CREATE TABLE unvested1 AS
	SELECT	a.*, a.fyear - 1 AS lagfyear,
			expiration_date AS expdate, exercise_price AS stkprc, 
			number_of_securities AS number, remaining_term AS term, value
	FROM	v.shares (KEEP = executive gvkey fyear datadate fy_end company_id lagfy_end executive_id2
			WHERE = (lagfy_end ^= .)) a 
			LEFT JOIN v.oend (KEEP = company_id executive_id2 fy_end expiration_date exercise_price number_of_securities
			remaining_term value equity_type trans_pd
			WHERE = (equity_type = 'Unexercisable Options')) b
	ON		a.company_id = b.company_id AND a.executive_id2 = b.executive_id2 AND a.lagfy_end = b.fy_end;
QUIT;

* Merge with BS assumptions;
* merge with Equilar BS assumptions;
PROC SQL;
	CREATE TABLE unvested2 AS
	SELECT	a.*, b.*, c.*
	FROM	unvested1 a LEFT JOIN bsinputs b
	ON		a.company_id = b.company_id AND a.fyear = b.fyear
			LEFT JOIN lagbsinputs c
	ON		a.company_id = c.company_id AND a.lagfyear = c.lagfyear;
QUIT;

* merge with ExecuComp/Compustat BS assumptions;
* pick equilar over execucomp over compustat;
* Equilar(t-1) -> Equilar(t) -> ExecuComp(t-1) -> ExecuComp(t) -> Compustat(t-1) -> Compustat(t);
PROC SQL;
	CREATE TABLE unvested3 AS
	SELECT	a.*, b.*, 
			c.bs_volatility AS lagbs_volatility,
			c.bs_yield AS lagbs_yield,
			c.optdr AS lagoptdr,
			c.optrfr AS lagoptrfr,
			c.optvol AS lagoptvol,

			COALESCE(lagdividendyield, dividendyield, c.bs_yield, b.bs_yield, c.optdr, b.optdr) AS finaldivyield1,
			COALESCE(lagvolatility_, volatility_, c.bs_volatility, b.bs_volatility, c.optvol, b.optvol) AS finalvol1,
			COALESCE(lagrisk_free_rate__, risk_free_rate__, c.optrfr, b.optrfr) AS finalrfr1,

			(CALCULATED finaldivyield1 = .) AS missingdiv,
			(CALCULATED finalvol1 = .) AS missingvol,
			(CALCULATED finalrfr1 = .) AS missingrfr
	FROM	unvested2 a LEFT JOIN bsinputsboth b
	ON		a.gvkey = b.gvkey AND a.fyear = b.fyear
			LEFT JOIN bsinputsboth c
	ON		a.gvkey = c.gvkey AND a.lagfyear = c.fyear;
QUIT;

* Try to fill in missing volatility;
* generate list of gvkey-fyear that we need volatility;
PROC SORT DATA = unvested3 OUT = gvkeyindex1 (KEEP = gvkey datadate fyear) NODUPKEY;
	BY gvkey fyear;
	WHERE missingvol AND number > 0;
RUN;

DATA gvkeyindex1;
	SET gvkeyindex1;
	lagdatadate = INTNX('MONTH', datadate, -12, 'END');
	FORMAT lagdatadate MMDDYY10.;
RUN;

* obtain permno;
PROC SQL;
	CREATE TABLE permnoindex1 AS
	SELECT	*
	FROM	gvkeyindex1 a, compustat_crsp_link b
	WHERE	a.gvkey = b.gvkey;
QUIT;

* calculate standard deviation of daily returns from CRSP;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME crspsd '/wrds/crsp/sasdata/a_stock'; 
PROC UPLOAD DATA = permnoindex1 OUT = permnoindex1; RUN;

* Merge with CRSP daily stock file and calculate annalized standard deviation of daily return;
PROC SQL;
	CREATE TABLE vol1 AS
	SELECT		gvkey, permno, fyear, 
				COUNT(*) AS n_ret, STD(LOG(1 + ret)) AS std_ret,
				CALCULATED std_ret * SQRT(252) AS annualret
	FROM		permnoindex1 a, crspsd.dsf (KEEP = date permno ret prc) b
	WHERE		a.lpermno = b.permno AND b.date BETWEEN a.lagdatadate - 365 * 3 AND a.lagdatadate
	GROUP BY	gvkey, permno, fyear
	ORDER BY	gvkey, fyear, annualret;
QUIT;
PROC DOWNLOAD DATA = vol1 OUT = vol1; RUN;
ENDRSUBMIT;

* pick bigger return volatility for these;
PROC SQL;
	CREATE TABLE dualstock AS
	SELECT		*
	FROM		vol1
	GROUP BY	gvkey, fyear
	HAVING		COUNT(*) > 1
	ORDER BY	gvkey, fyear, annualret;
QUIT;

DATA filledvol1;
	SET vol1;
	BY gvkey fyear annualret;
	IF LAST.fyear;
RUN;

* Merge back to unvested14; 
PROC SQL;
	CREATE TABLE unvested4 AS
	SELECT	*
	FROM	unvested3 a LEFT JOIN filledvol1 (KEEP = gvkey fyear annualret) b
	ON		a.gvkey = b.gvkey AND a.fyear = b.fyear;
QUIT;

* if missing, set it to annualret from CRSP;
DATA unvested4;
	SET unvested4;
	IF missingvol THEN DO;
		finalvol1 = annualret;
	END;
RUN;

* Try one more time to fill in VOL with current year VOL;
* need volatility for these;
PROC SORT DATA = unvested4 OUT = gvkeyindex2 (KEEP = gvkey datadate fyear) NODUPKEY;
	BY gvkey fyear;
	WHERE finalvol1 = . AND number > 0;
RUN;

DATA gvkeyindex2;
	SET gvkeyindex2;
	lagdatadate = INTNX('MONTH', datadate, -12, 'END');
	FORMAT lagdatadate MMDDYY10.;
RUN;

* obtain permno;
PROC SQL;
	CREATE TABLE permnoindex2 AS
	SELECT	*
	FROM	gvkeyindex2 a, compustat_crsp_link b
	WHERE	a.gvkey = b.gvkey;
QUIT;

* calculate standard deviation of daily returns from CRSP;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME crspsd '/wrds/crsp/sasdata/a_stock'; 
PROC UPLOAD DATA = permnoindex2 OUT = permnoindex2; RUN;

* Merge with CRSP daily stock file and calculate annalized standard deviation of daily return;
PROC SQL;
	CREATE TABLE vol2 AS
	SELECT		gvkey, permno, fyear, 
				COUNT(*) AS n_ret, STD(LOG(1 + ret)) AS std_ret,
				CALCULATED std_ret * SQRT(252) AS annualret
	FROM		permnoindex2 a, crspsd.dsf (KEEP = date permno ret prc) b
	WHERE		a.lpermno = b.permno AND b.date BETWEEN a.lagdatadate - 365 * 4 AND a.lagdatadate + 365
	GROUP BY	gvkey, permno, fyear
	ORDER BY	gvkey, fyear, annualret;
QUIT;

PROC DOWNLOAD DATA = vol2 OUT = vol2; RUN;
ENDRSUBMIT;

* pick bigger volatility for these;
PROC SQL;
	CREATE TABLE dualstock AS
	SELECT		*
	FROM		vol2
	GROUP BY	gvkey, fyear
	HAVING		COUNT(*) > 1
	ORDER BY	gvkey, fyear, annualret;
QUIT;

DATA filledvol2;
	SET vol2;
	BY gvkey fyear annualret;
	IF LAST.fyear;
RUN;

* try to fill in the missing volatilities;
PROC SQL;
	CREATE TABLE unvested5 AS
	SELECT	a.*, b.annualret AS annualret_extwin
	FROM	unvested4 a LEFT JOIN filledvol2 (KEEP = gvkey fyear annualret) b
	ON		a.gvkey = b.gvkey AND a.fyear = b.fyear;
QUIT;

* if missing, set it to annualret from CRSP;
DATA unvested5;
	SET unvested5;
	IF finalvol1 = . THEN DO;
		finalvol1 = annualret_extwin;
	END;
RUN;

* Try to fill in missing risk free rates;
PROC SORT DATA = unvested5 OUT = gvkeyindex3 (KEEP = gvkey datadate fyear term) NODUPKEY;
	BY fyear gvkey term datadate;
	WHERE missingrfr AND number > 0 AND term ^= .;
RUN;

PROC SQL;
	CREATE TABLE filledinterest AS
	SELECT		a.*, b.*
	FROM		gvkeyindex3 a, 
				v.interestrates_2017 (WHERE = (YEAR(date) >= 2004 AND dgs1 ^= .)
				KEEP = date DGS1 DGS10 DGS1MO DGS2 DGS20 DGS3 DGS30 DGS3MO DGS5 DGS6MO DGS7) b
	WHERE		b.date BETWEEN 
				INTNX('MONTH', datadate, -12, 'END') - 5 AND INTNX('MONTH', datadate, -12, 'END')
	GROUP BY	fyear, gvkey, term, datadate
	HAVING		date = MAX(date);
QUIT;

DATA filledinterest; 
	SET filledinterest;
	mindis = MIN(ABS(term - 1), ABS(term - 2), ABS(term - 3), ABS(term - 5), ABS(term - 7),
		ABS(term - 10), ABS(term - 20), ABS(term - 30), ABS(term - 0.083), 
		ABS(term - 0.25), ABS(term - 0.5));
	IF mindis = ABS(term - 1) THEN rfr = DGS1 * 0.01;
	ELSE IF mindis = ABS(term - 2) THEN rfr = DGS2 * 0.01;
	ELSE IF mindis = ABS(term - 3) THEN rfr = DGS3 * 0.01;
	ELSE IF mindis = ABS(term - 5) THEN rfr = DGS5 * 0.01;
	ELSE IF mindis = ABS(term - 7) THEN rfr = DGS7 * 0.01;
	ELSE IF mindis = ABS(term - 10) THEN rfr = DGS10 * 0.01;
	ELSE IF mindis = ABS(term - 20) THEN rfr = DGS20 * 0.01;
	ELSE IF mindis = ABS(term - 30) THEN rfr = DGS30 * 0.01;
	ELSE IF mindis = ABS(term - 0.083) THEN rfr = DGS1MO * 0.01;
	ELSE IF mindis = ABS(term - 0.25) THEN rfr = DGS3MO * 0.01;
	ELSE IF mindis = ABS(term - 0.5) THEN rfr = DGS6MO * 0.01; 
RUN;

* fill in the missing ones;
PROC SQL;
	CREATE TABLE unvested6 AS
	SELECT	a.*, b.rfr
	FROM	unvested5 a LEFT JOIN filledinterest b
	ON		a.fyear = b.fyear AND a.gvkey = b.gvkey AND a.term = b.term;
QUIT;

DATA unvested6;
	SET unvested6;
	IF missingrfr THEN DO;
		finalrfr1 = rfr;
	END;
RUN;

* Try to fill in missing dividend yield;
PROC SORT DATA = unvested6 OUT = gvkeyindex4 (KEEP = gvkey fyear) NODUPKEY;
	BY gvkey fyear;
	WHERE missingdiv AND number > 0;
RUN;

%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME compna '/wrds/comp/sasdata/naa'; 
PROC UPLOAD DATA = gvkeyindex4 OUT = gvkeyindex4; RUN;

* Merge with Compustat to obtain dividend yield;
* use the average of the last 5 years;
PROC SQL;
	CREATE TABLE dividends AS
	SELECT		a.gvkey, a.fyear, 
				AVG(COALESCE(dvpsp_f / prcc_f, dvc / (prcc_f * csho))) AS mean_div
	FROM		gvkeyindex4 a, 
				compna.funda (KEEP = gvkey fyear dvt dv dvc dvpsp_f prcc_f csho
				indfmt datafmt popsrc consol
				WHERE = (indfmt = 'INDL' AND datafmt = 'STD' AND popsrc = 'D' 
				AND consol = 'C' AND fyear >= 1998)) b
	WHERE		a.gvkey = b.gvkey AND b.fyear BETWEEN a.fyear - 5 AND a.fyear - 1
	GROUP BY	a.gvkey, a.fyear
	HAVING		mean_div ^= .;
QUIT;

PROC DOWNLOAD DATA = dividends OUT = dividends; RUN;
ENDRSUBMIT;

PROC SQL;
	CREATE TABLE unvested7 AS
	SELECT	*
	FROM	unvested6 a LEFT JOIN dividends b
	ON		a.gvkey = b.gvkey AND A.fyear = b.fyear;
QUIT;

DATA unvested7;
	SET unvested7;
	IF missingdiv THEN DO;
		finaldivyield1 = mean_div;
	END;
RUN;

* set missing dividend yield to zero;
DATA bsassumptions_unvestedopt;
	SET unvested7;
	finaldivyield1 = COALESCE(finaldivyield1, 0);
RUN;
PROC SORT DATA = bsassumptions_unvestedopt;
	BY company_id fy_end executive_id2;
RUN;

* BS Assumptions Newly Awarded Options;
* merge index with newly awarded options;
* merge shares with newly awarded options; 
* use shares because it has an index of firm-year-managers;
PROC SQL;
	CREATE TABLE newlyawarded1 (RENAME = (exercise_price = stkprc expiration_date = expdate)) AS
	SELECT	a.*, b.*, a.fyear - 1 AS lagfyear
	FROM	v.shares (KEEP = executive gvkey fyear datadate fy_end company_id 
			lagfy_end executive_id2 lagfinalprc finalprc) a LEFT JOIN newlyawardedoptions b
	ON		a.company_id = b.company_id AND a.executive_id2 = b.executive_id2 AND a.fy_end = b.fy_end;
QUIT;

* Merge with Equilar BS assumptions;

* risk free rates in the newlyawardedoptions and bsinputs are the same;
* difference in volatility and dividendyield;
* bsinputs BS assumption data is from v.oand;
* newlyawardedoptions BS assumption data is from v.oand too;
* the reason these two can be different is becasue bsinputs uses the first grant of the year while 
	newlyawardedoptions uses the actual grant;
* since both can be missing, use whichever is available;
PROC SQL;
	CREATE TABLE newlyawarded2 AS
	SELECT	COALESCE(a.volatility_, b.volatility_) AS volatility_,
			COALESCE(a.risk_free_rate__, b.risk_free_rate__) AS risk_free_rate__,
			COALESCE(a.dividendyield, b.dividendyield) AS dividendyield,
			a.*, 
			c.*, term + 1 AS lagterm
	FROM	newlyawarded1 a LEFT JOIN bsinputs b
	ON		a.company_id = b.company_id AND a.fyear = b.fyear
			LEFT JOIN lagbsinputs c
	ON		a.company_id = c.company_id AND a.lagfyear = c.lagfyear;
QUIT;

* merge with ExecuComp/Compustat BS assumptions;
* pick equilar over execucomp over compustat;

* Equilar(t-1) -> Equilar(t) -> ExecuComp(t-1) -> ExecuComp(t) -> Compustat(t-1) -> Compustat(t);
PROC SQL;
	CREATE TABLE newlyawarded3 AS
	SELECT	a.*, b.*, 
			c.bs_volatility AS lagbs_volatility,
			c.bs_yield AS lagbs_yield,
			c.optdr AS lagoptdr,
			c.optrfr AS lagoptrfr,
			c.optvol AS lagoptvol,
			COALESCE(lagdividendyield, dividendyield, c.bs_yield, b.bs_yield, c.optdr, b.optdr) AS finaldivyield1,
			COALESCE(lagvolatility_, volatility_, c.bs_volatility, b.bs_volatility, c.optvol, b.optvol) AS finalvol1,
			COALESCE(lagrisk_free_rate__, risk_free_rate__, c.optrfr, b.optrfr) AS finalrfr1,
			(CALCULATED finaldivyield1 = .) AS missingdiv,
			(CALCULATED finalvol1 = .) AS missingvol,
			(CALCULATED finalrfr1 = .) AS missingrfr
	FROM	newlyawarded2 a LEFT JOIN bsinputsboth b
	ON		a.gvkey = b.gvkey AND a.fyear = b.fyear
			LEFT JOIN bsinputsboth c
	ON		a.gvkey = c.gvkey AND a.lagfyear = c.fyear;
QUIT;

* Try to fill in missing volatility;
* generate list of gvkey-fyear that we need volatility;
PROC SORT DATA = newlyawarded3 OUT = gvkeyindex1 (KEEP = gvkey datadate fyear) NODUPKEY;
	BY gvkey fyear;
	WHERE missingvol AND number_of_options > 0;
RUN;

DATA gvkeyindex1;
	SET gvkeyindex1;
	lagdatadate = INTNX('MONTH', datadate, -12, 'END');
	FORMAT lagdatadate MMDDYY10.;
RUN;

* obtain permno;
PROC SQL;
	CREATE TABLE permnoindex1 AS
	SELECT	*
	FROM	gvkeyindex1 a, compustat_crsp_link b
	WHERE	a.gvkey = b.gvkey;
QUIT;

* calculate standard deviation of daily returns from CRSP;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME crspsd '/wrds/crsp/sasdata/a_stock'; 
PROC UPLOAD DATA = permnoindex1 OUT = permnoindex1; RUN;

*** Merge with CRSP daily stock file and calculate annalized standard deviation of daily return;
PROC SQL;
	CREATE TABLE vol1 AS
	SELECT		gvkey, permno, fyear, 
				COUNT(*) AS n_ret, STD(LOG(1 + ret)) AS std_ret,
				CALCULATED std_ret * SQRT(252) AS annualret
	FROM		permnoindex1 a, crspsd.dsf (KEEP = date permno ret prc) b
	WHERE		a.lpermno = b.permno AND b.date BETWEEN a.lagdatadate - 365 * 3 AND a.lagdatadate
	GROUP BY	gvkey, permno, fyear
	ORDER BY	gvkey, fyear, annualret;
QUIT;
PROC DOWNLOAD DATA = vol1 OUT = vol1; RUN;
ENDRSUBMIT;

* pick the bigger return volatility for these;
PROC SQL;
	CREATE TABLE dualstock AS
	SELECT		*
	FROM		vol1
	GROUP BY	gvkey, fyear
	HAVING		COUNT(*) > 1
	ORDER BY	gvkey, fyear, annualret;
QUIT;

DATA filledvol1;
	SET vol1;
	BY gvkey fyear annualret;
	IF LAST.fyear;
RUN;

* Merge back to newlyawarded14; 
PROC SQL;
	CREATE TABLE newlyawarded4 AS
	SELECT	*
	FROM	newlyawarded3 a LEFT JOIN filledvol1 (KEEP = gvkey fyear annualret) b
	ON		a.gvkey = b.gvkey AND a.fyear = b.fyear;
QUIT;

* if missing, set it to annualret from CRSP;
DATA newlyawarded4;
	SET newlyawarded4;
	IF missingvol THEN DO;
		finalvol1 = annualret;
	END;
RUN;

* Try one more time to fill in VOL with current year VOL;
* need volatility for these;
PROC SORT DATA = newlyawarded4 OUT = gvkeyindex2 (KEEP = gvkey datadate fyear) NODUPKEY;
	BY gvkey fyear;
	WHERE finalvol1 = . AND number_of_options > 0;
RUN;

DATA gvkeyindex2;
	SET gvkeyindex2;
	lagdatadate = INTNX('MONTH', datadate, -12, 'END');
	FORMAT lagdatadate MMDDYY10.;
RUN;

* obtain permno;
PROC SQL;
	CREATE TABLE permnoindex2 AS
	SELECT	*
	FROM	gvkeyindex2 a, compustat_crsp_link b
	WHERE	a.gvkey = b.gvkey;
QUIT;

* calculate standard deviation of daily returns from CRSP;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME crspsd '/wrds/crsp/sasdata/a_stock'; 
PROC UPLOAD DATA = permnoindex2 OUT = permnoindex2; RUN;

* Merge with CRSP daily stock file and calculate annalized standard deviation of daily return;
PROC SQL;
	CREATE TABLE vol2 AS
	SELECT		gvkey, permno, fyear, 
				COUNT(*) AS n_ret, STD(LOG(1 + ret)) AS std_ret,
				CALCULATED std_ret * SQRT(252) AS annualret
	FROM		permnoindex2 a, crspsd.dsf (KEEP = date permno ret prc) b
	WHERE		a.lpermno = b.permno AND b.date BETWEEN a.lagdatadate - 365 * 4 AND a.lagdatadate + 365
	GROUP BY	gvkey, permno, fyear
	ORDER BY	gvkey, fyear, annualret;
QUIT;

PROC DOWNLOAD DATA = vol2 OUT = vol2; RUN;
ENDRSUBMIT;

* pick the bigger return volatility for these;
PROC SQL;
	CREATE TABLE dualstock AS
	SELECT		*
	FROM		vol2
	GROUP BY	gvkey, fyear
	HAVING		COUNT(*) > 1
	ORDER BY	gvkey, fyear, annualret;
QUIT;

DATA filledvol2;
	SET vol2;
	BY gvkey fyear annualret;
	IF LAST.fyear;
RUN;

* try to fill in the missing volatilities;
PROC SQL;
	CREATE TABLE newlyawarded5 AS
	SELECT	a.*, b.annualret AS annualret_extwin
	FROM	newlyawarded4 a LEFT JOIN filledvol2 (KEEP = gvkey fyear annualret) b
	ON		a.gvkey = b.gvkey AND a.fyear = b.fyear;
QUIT;

* if missing, set it to annualret from CRSP;
DATA newlyawarded5;
	SET newlyawarded5;
	IF finalvol1 = . THEN DO;
		finalvol1 = annualret_extwin;
	END;
RUN;

* Try to fill in missing risk free rates;
PROC SORT DATA = newlyawarded5 OUT = gvkeyindex3 (KEEP = gvkey datadate fyear lagterm) NODUPKEY;
	BY fyear gvkey lagterm datadate;
	WHERE missingrfr AND number_of_options > 0;
RUN;

PROC SQL;
	CREATE TABLE filledinterest AS
	SELECT		a.*, b.*
	FROM		gvkeyindex3 a, 
				v.interestrates_2017 (WHERE = (YEAR(date) >= 2004 AND dgs1 ^= .)
				KEEP = date DGS1 DGS10 DGS1MO DGS2 DGS20 DGS3 DGS30 DGS3MO DGS5 DGS6MO DGS7) b
	WHERE		b.date BETWEEN 
				INTNX('MONTH', datadate, -12, 'END') - 5 AND INTNX('MONTH', datadate, -12, 'END')
	GROUP BY	fyear, gvkey, lagterm, datadate
	HAVING		date = MAX(date);
QUIT;

DATA filledinterest; 
	SET filledinterest;
	mindis = MIN(ABS(lagterm - 1), ABS(lagterm - 2), ABS(lagterm - 3), ABS(lagterm - 5), ABS(lagterm - 7),
		ABS(lagterm - 10), ABS(lagterm - 20), ABS(lagterm - 30), ABS(lagterm - 0.083), 
		ABS(lagterm - 0.25), ABS(lagterm - 0.5));
	IF mindis = ABS(lagterm - 1) THEN rfr = DGS1 * 0.01;
	ELSE IF mindis = ABS(lagterm - 2) THEN rfr = DGS2 * 0.01;
	ELSE IF mindis = ABS(lagterm - 3) THEN rfr = DGS3 * 0.01;
	ELSE IF mindis = ABS(lagterm - 5) THEN rfr = DGS5 * 0.01;
	ELSE IF mindis = ABS(lagterm - 7) THEN rfr = DGS7 * 0.01;
	ELSE IF mindis = ABS(lagterm - 10) THEN rfr = DGS10 * 0.01;
	ELSE IF mindis = ABS(lagterm - 20) THEN rfr = DGS20 * 0.01;
	ELSE IF mindis = ABS(lagterm - 30) THEN rfr = DGS30 * 0.01;
	ELSE IF mindis = ABS(lagterm - 0.083) THEN rfr = DGS1MO * 0.01;
	ELSE IF mindis = ABS(lagterm - 0.25) THEN rfr = DGS3MO * 0.01;
	ELSE IF mindis = ABS(lagterm - 0.5) THEN rfr = DGS6MO * 0.01; 
RUN;

* fill in the missing ones;
PROC SQL;
	CREATE TABLE newlyawarded6 AS
	SELECT	a.*, b.rfr
	FROM	newlyawarded5 a LEFT JOIN filledinterest b
	ON		a.fyear = b.fyear AND a.gvkey = b.gvkey AND a.lagterm = b.lagterm;
QUIT;

DATA newlyawarded6;
	SET newlyawarded6;
	IF missingrfr THEN DO;
		finalrfr1 = rfr;
	END;
RUN;

* Try to fill in missing dividend yield;
PROC SORT DATA = newlyawarded6 OUT = gvkeyindex4 (KEEP = gvkey fyear) NODUPKEY;
	BY gvkey fyear;
	WHERE missingdiv AND number_of_options > 0;
RUN;

%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME compna '/wrds/comp/sasdata/naa'; 
PROC UPLOAD DATA = gvkeyindex4 OUT = gvkeyindex4; RUN;

* Merge with Compustat to obtain dividend yield;
* use the average of the last 5 years;
PROC SQL;
	CREATE TABLE dividends AS
	SELECT		a.gvkey, a.fyear, 
				AVG(COALESCE(dvpsp_f / prcc_f, dvc / (prcc_f * csho))) AS mean_div
	FROM		gvkeyindex4 a, 
				compna.funda (KEEP = gvkey fyear dvt dv dvc dvpsp_f prcc_f csho
				indfmt datafmt popsrc consol
				WHERE = (indfmt = 'INDL' AND datafmt = 'STD' AND popsrc = 'D' 
				AND consol = 'C' AND fyear >= 1998)) b
	WHERE		a.gvkey = b.gvkey AND b.fyear BETWEEN a.fyear - 5 AND a.fyear - 1
	GROUP BY	a.gvkey, a.fyear
	HAVING		mean_div ^= .;
QUIT;

PROC DOWNLOAD DATA = dividends OUT = dividends; RUN;
ENDRSUBMIT;

PROC SQL;
	CREATE TABLE newlyawarded7 AS
	SELECT	*
	FROM	newlyawarded6 a LEFT JOIN dividends b
	ON		a.gvkey = b.gvkey AND A.fyear = b.fyear;
QUIT;

DATA newlyawarded7;
	SET newlyawarded7;
	IF missingdiv THEN DO;
		finaldivyield1 = mean_div;
	END;
RUN;

* set missing dividend yield to zero;
DATA bsassumptions_newlyawardedopt;
	SET newlyawarded7;
	finaldivyield1 = COALESCE(finaldivyield1, 0);
RUN;

* Options Incentives;
* Calculate values of newly vested options;
* Calculate Z, Delta;
DATA bs_calc_newlyvestedoptions;
	SET bsassumptions_newlyvestedoptions;
	IF lagterm > 0 AND finalvol1 > 0 AND stkprc ^= 0 THEN DO;
		Z1 = (LOG(lagfinalprc / stkprc) + lagterm * (finalrfr1 - finaldivyield1 + finalvol1**2 / 2)) / (finalvol1 * SQRT(lagterm));
		IF lagfinalprc = . AND finalprc ^= . THEN DO;
			Z1 = (LOG(finalprc / stkprc) + lagterm * (finalrfr1 - finaldivyield1 + finalvol1**2 / 2)) / (finalvol1 * SQRT(lagterm));
		END;
		Delta1 = EXP(-finaldivyield1 * lagterm) * CDF("NORMAL",Z1) * vestedoptions;
	END;
RUN;

* remove ones without positive number of options; 
DATA bs_calc_newlyvestedoptions;
	SET bs_calc_newlyvestedoptions (WHERE = (vestedoptions > 0));
RUN;

* calculate option values and aggregate it to company-manager-fy_end level;
PROC SQL;
	CREATE TABLE newlyvestedoptionvalue AS
	SELECT		company_id, fy_end, executive_id2, executive, lagfinalprc, finalprc,
				COUNT(delta1) AS n_delta1, SUM(delta1) AS newlyvestedsum_delta1,
				COALESCE(CALCULATED newlyvestedsum_delta1 * lagfinalprc, CALCULATED newlyvestedsum_delta1 * finalprc) 
					AS newlyvestedoptionvalue1
	FROM		bs_calc_newlyvestedoptions
	GROUP BY	company_id, fy_end, executive_id2, executive, lagfinalprc, finalprc;
QUIT;

* Calculate values of newly awarded options;
* Calculate Z, Delta;
DATA bs_calc_newlyawardedoptions;
	SET bsassumptions_newlyawardedopt;
	IF lagterm > 0 AND finalvol1 > 0 AND stkprc ^= 0 THEN DO;
		Z1 = (LOG(lagfinalprc / stkprc) + lagterm * (finalrfr1 - finaldivyield1 + finalvol1**2 / 2)) / (finalvol1 * SQRT(lagterm));
		IF lagfinalprc = . AND finalprc ^= . THEN DO;
			Z1 = (LOG(finalprc / stkprc) + lagterm * (finalrfr1 - finaldivyield1 + finalvol1**2 / 2)) / (finalvol1 * SQRT(lagterm));
		END;
		Delta1 = EXP(-finaldivyield1 * lagterm) * CDF("NORMAL",Z1) * number_of_options;
	END;
	IF number_of_options > 0;
RUN;

PROC SQL;
	CREATE TABLE newlyawardedoptionvalue AS
	SELECT		company_id, fy_end, executive_id2, executive, lagfinalprc, finalprc,
				COUNT(delta1) AS n_delta1, SUM(delta1) AS newlyawardedsum_delta1,
				COALESCE(CALCULATED newlyawardedsum_delta1 * lagfinalprc, CALCULATED newlyawardedsum_delta1 * finalprc) 
					AS newlyawardedoptionvalue1
	FROM		bs_calc_newlyawardedoptions
	GROUP BY	company_id, fy_end, executive_id2, executive, lagfinalprc, finalprc;
QUIT;

* Calculate values of already vested options;
* merge with v.shares to obtain lagfinalprc and finalprc;
PROC SQL;
	CREATE TABLE alreadyvestedoptions AS
	SELECT	*
	FROM	bsassumptions_alreadyvestedopt a, 
			v.shares (KEEP = company_id fy_end lagfy_end executive_id2 lagfinalprc finalprc) b
	WHERE	a.company_id = b.company_id AND a.fy_end = b.fy_end 
			AND a.executive_id2 = b.executive_id2 AND a.lagfy_end = b.lagfy_end;
QUIT;

* calculate Z, Delta;
DATA bs_calc_alreadyvestedoptions; 
	SET alreadyvestedoptions;
	IF term > 0 AND finalvol1 > 0 AND stkprc ^= 0 THEN DO;
		Z1 = (LOG(lagfinalprc / stkprc) + term * (finalrfr1 - finaldivyield1 + finalvol1**2 / 2)) / (finalvol1 * SQRT(term));
		IF lagfinalprc = . AND finalprc ^= . THEN DO;
			Z1 = (LOG(finalprc / stkprc) + term * (finalrfr1 - finaldivyield1 + finalvol1**2 / 2)) / (finalvol1 * SQRT(term));
		END;
		Delta1 = EXP(-finaldivyield1 * term) * CDF("NORMAL",Z1) * number;
	END;
	IF number > 0;
RUN;

* calculate option values and aggregate it to company-manager-fy_end level;
PROC SQL;
	CREATE TABLE alreadyvestedoptionvalue AS
	SELECT		company_id, fy_end, executive_id2, executive, lagfinalprc, finalprc,
				COUNT(delta1) AS n_delta1, SUM(delta1) AS alreadyvestedsum_delta1,
				COALESCE(CALCULATED alreadyvestedsum_delta1 * lagfinalprc, CALCULATED alreadyvestedsum_delta1 * finalprc) 
					AS alreadyvestedoptionvalue1
	FROM		bs_calc_alreadyvestedoptions
	GROUP BY	company_id, fy_end, executive_id2, executive, lagfinalprc, finalprc;
QUIT;

* Calculate values of unvested options;
* merge with v.shares to obtain lagfinalprc and finalprc;
PROC SQL;
	CREATE TABLE unvestedoptions AS
	SELECT	*
	FROM	bsassumptions_unvestedopt a, 
			v.shares (KEEP = company_id fy_end lagfy_end executive_id2 lagfinalprc finalprc) b
	WHERE	a.company_id = b.company_id AND a.fy_end = b.fy_end 
			AND a.executive_id2 = b.executive_id2 AND a.lagfy_end = b.lagfy_end;
QUIT;

* calculate Z, Delta;
DATA bs_calc_unvestedoptions; 
	SET unvestedoptions;
	IF term > 0 AND finalvol1 > 0 AND stkprc ^= 0 THEN DO;
		Z1 = (LOG(lagfinalprc / stkprc) + term * (finalrfr1 - finaldivyield1 + finalvol1**2 / 2)) / (finalvol1 * SQRT(term));
		IF lagfinalprc = . AND finalprc ^= . THEN DO;
			Z1 = (LOG(finalprc / stkprc) + term * (finalrfr1 - finaldivyield1 + finalvol1**2 / 2)) / (finalvol1 * SQRT(term));
		END;
		Delta1 = EXP(-finaldivyield1 * term) * CDF("NORMAL",Z1) * number;
	END;
	IF number > 0;
RUN;

* calculate option values and aggregate it to company-manager-fy_end level;
PROC SQL;
	CREATE TABLE unvestedoptionvalue AS
	SELECT		company_id, fy_end, executive_id2, executive, lagfinalprc, finalprc,
				COUNT(delta1) AS n_delta1, SUM(delta1) AS unvestedsum_delta1,
				COALESCE(CALCULATED unvestedsum_delta1 * lagfinalprc, CALCULATED unvestedsum_delta1 * finalprc) 
					AS unvestedoptionvalue1
	FROM		bs_calc_unvestedoptions
	GROUP BY	company_id, fy_end, executive_id2, executive, lagfinalprc, finalprc;
QUIT;

* Aggregate Shares and Options Incentives;
* Merge vested shares with newly vested, already, unvested options;
PROC SORT DATA = v.shares NODUPKEY; BY company_id fy_end executive_id2; RUN;

* merge v.shares with BS calculated value of newly vested, already vested, unvested options in year t,
	and Equilar reported value of already vested and unvested options in year t-1;
PROC SQL;
	CREATE TABLE myopia AS
	SELECT	a.*, b.*, c.*, d.*
	FROM	v.shares a LEFT JOIN newlyvestedoptionvalue b
	ON		a.company_id = b.company_id AND a.fy_end = b.fy_end 
			AND a.executive_id2 = b.executive_id2 AND a.executive = b.executive
			LEFT JOIN alreadyvestedoptionvalue c
	ON		a.company_id = c.company_id AND a.fy_end = c.fy_end 
			AND a.executive_id2 = c.executive_id2 AND a.executive = c.executive
			LEFT JOIN unvestedoptionvalue d
	ON		a.company_id = d.company_id AND a.fy_end = d.fy_end 
			AND a.executive_id2 = d.executive_id2 AND a.executive = d.executive;
QUIT;

* Calculate total value of vested equity;
DATA v.mm;
	SET myopia;
	newlyvestednumber1 = SUM(shares_acquired_on_vesting_of_st, newlyvestedsum_delta1, 0);
	alreadyvestednumber1 = SUM(alreadyvestedshares, alreadyvestedsum_delta1, 0);
	unvestednumber1 = SUM(total_unvested_shares, unvestedsum_delta1, 0);
RUN;

* Vesting equity date, V1;
* vesting equity at the monthly level, using vesting date to assign month;
* for options, we assume that they vest and expire on the annversary of the grant;
* for stocks, post-2006 with grant dates, we use grant date;
* for stocks, pre-2006 ones and post-2006 without grant dates ones, we evenly distribute them across the post-2006 grant dates;
* V1: using all "Graded" awards first, then all "Cliff" awards;
* V1: after using Graded and Cliff, assign remaining to All stock grant dates;

* Post 2006 vesting options - from Equilar;
* Using grant date to figure out exactly which month an option vests in;
DATA post2006vestingoptions_wdate; 
	SET newlyvestedoptions;
	vestingmonth = MONTH(expdate); vestingday = DAY(expdate);
RUN;

* Post 2006 vesting stocks - from Equilar;
* obtain the newly vested stocks in each year and match them back to stock awards to figure out their grant dates;
DATA post2006posvestingstocks; 
	SET newlyvestedshares (KEEP = company_id fy_end executive_id2 shares_acquired_on_vesting_of_st);
	lagfy_end = INTNX('MONTH', fy_end, -12, 'END');
	FORMAT lagfy_end MMDDYY10.; 
	IF shares_acquired_on_vesting_of_st > 0;
RUN;

* Retrieve all stock awards and calculate their vesting begin and end date;
* Vesting begin date is one year after grant date, vesting end date is vestingperiod year after grant date;
DATA sand;
	SET v.sand (KEEP = company_id executive_id2 fy_end fyear executive company ticker 
		grant_date number_of_securities vesting_schedule vestingPeriod performance_);
	
	vestingmonth = MONTH(grant_date); vestingday = DAY(grant_date); 

	* for 2/29 situation, since it does not happen every year;
	vestingday2 = vestingday; IF vestingmonth = 2 AND vestingday = 29 THEN vestingday2 = 28; 
	vestingday3 = vestingday; IF vestingmonth = 2 AND vestingday = 29 
		AND vestingperiod / 4 ^= INT(vestingperiod / 4) THEN vestingday3 = 28; 

	* set vesting begin date to be one year after the grant date and end date to be vestingperiod year after grant date;
	vestingbegdate = MDY(vestingmonth, vestingday2, YEAR(grant_date) + 1);
	vestingenddate = MDY(vestingmonth, vestingday3, YEAR(grant_date) + VestingPeriod); 

	FORMAT vestingbegdate vestingenddate MMDDYY10.; 

	DROP vestingday2 vestingday3;

	IF grant_date ^= .;
	WHERE vesting_schedule ^= "Retirement";
RUN;

* Assighed using "Graded" stock grants;
* merge post-2006 vested stocks with stock awards with Graded vesting schedule;
* make sure that the vesting period and the lagfy_end to fy_end period overlaps;
* assume for stock awards with Graded vesting schedule, an equal number of shares vested in each year;
* calculate the total number of vesting shares for each firm-manager-fy_end based on its grant history and this assumption;
PROC SQL;
	CREATE TABLE gradedawards AS
	SELECT		a.company_id, a.fy_end, a.lagfy_end, a.executive_id2, 
				a.shares_acquired_on_vesting_of_st AS totalvesting,
				number_of_securities / vestingperiod AS implied_vesting,
				SUM(number_of_securities / vestingperiod) AS sum_implied_vesting,
				vestingmonth, vestingday
	FROM		post2006posvestingstocks a LEFT JOIN sand (WHERE = (vesting_schedule = "Graded")) b
	ON			a.company_id = b.company_id AND a.executive_id2 = b.executive_id2
				AND b.vestingenddate >= a.lagfy_end AND b.vestingbegdate <= a.fy_end
	GROUP BY	a.company_id, a.fy_end, a.lagfy_end, a.executive_id2;
QUIT;

* the total shares vesting based on award / grant date can be more than the reported shares vesting;
* adjust the number of vesting shares in this case;
DATA gradedawards;
	SET gradedawards;
	* if merged with any grants, then can use the grant date to assign shares;
	IF sum_implied_vesting > 0 THEN DO;
		* cannot assign more than the reported shares vesting;
		maxassigned_vesting = MIN(sum_implied_vesting, totalvesting);
		leftvesting = totalvesting - maxassigned_vesting;
	END;
	IF sum_implied_vesting = . THEN leftvesting = totalvesting;

	* ratio of shares already assigned vesting schedule, set to zero if no match to stock awards;
	vestingratio = COALESCE(maxassigned_vesting, 0) / totalvesting;

	FORMAT vestingratio PERCENT8.3;
	
	* if less than 10 shares left and vesting ratio > 99% then count as fully merged;
	fullymerged = (leftvesting <= 10 AND vestingratio > 0.99);

	IF fullymerged THEN adj = (maxassigned_vesting + leftvesting) / sum_implied_vesting;
	ELSE adj = 1;

	* adjust the implied vesting number;
	vesting = implied_vesting * adj; 
	
	KEEP company_id executive_id2 fy_end lagfy_end vesting leftvesting vestingmonth vestingday fullymerged; 
RUN;

* these are the assigned ones based on "Graded" stock awards;
DATA post2006posvestingstocks_g;
	SET gradedawards (WHERE = (vesting > 0) KEEP = company_id executive_id2 fy_end vesting vestingmonth vestingday fullymerged);
	type = 'Graded';
RUN;

* these are the unassigned number of vesting stocks;
PROC SORT DATA = gradedawards (KEEP = company_id executive_id2 fy_end lagfy_end leftvesting fullymerged) 
		OUT = left_g NODUPKEY;
	BY company_id executive_id2 fy_end leftvesting;
	WHERE fullymerged ^= 1;
RUN;

* these are the number of assigned ones;
PROC SORT DATA = gradedawards OUT = x NODUPKEY;
	BY company_id executive_id2 fy_end;
	WHERE fullymerged;
RUN;

* Assign using "Cliff" stock grants;
* merge unassigned stock awards with "Cliff" stock grants;
* make sure that the vesting end date falls between lagfy_end and fy_end;
* assign all "Cliff" stock grants vest on the vesting end date;
PROC SQL;
	CREATE TABLE cliffawards AS
	SELECT		a.company_id, a.fy_end, a.lagfy_end, a.executive_id2, 
				a.leftvesting AS totalvesting,
				number_of_securities AS implied_vesting,
				SUM(number_of_securities) AS sum_implied_vesting,
				vestingmonth, vestingday
	FROM		left_g a LEFT JOIN sand (WHERE = (vesting_schedule = "Cliff")) b
	ON			a.company_id = b.company_id AND a.executive_id2 = b.executive_id2
				AND b.vestingenddate BETWEEN a.lagfy_end + 1 AND a.fy_end
	GROUP BY	a.company_id, a.fy_end, a.lagfy_end, a.executive_id2;
QUIT;

* the total shares vesting based on award / grant date can be more than the reported shares vesting;
* adjust the number of vesting shares in this case;
DATA cliffawards;
	SET cliffawards;
	* if merged with any grants, then can use the grant date to assign shares;
	IF sum_implied_vesting > 0 THEN DO;
		* cannot assign more than the reported shares vesting;
		maxassigned_vesting = MIN(sum_implied_vesting, totalvesting);
		leftvesting = totalvesting - maxassigned_vesting;
	END;
	IF sum_implied_vesting = . THEN leftvesting = totalvesting;

	* ratio of shares already assigned vesting schedule, set to zero if no match to stock awards;
	vestingratio = COALESCE(maxassigned_vesting, 0) / totalvesting;

	FORMAT vestingratio PERCENT8.3;
	
	* if less than 10 shares left and vesting ratio > 99% then count as fully merged;
	fullymerged = (leftvesting <= 10 AND vestingratio > 0.99);

	IF fullymerged THEN adj = (maxassigned_vesting + leftvesting) / sum_implied_vesting;
	ELSE adj = 1;

	* adjust the implied vesting number;
	vesting = implied_vesting * adj; 
	
	KEEP company_id executive_id2 fy_end lagfy_end vesting leftvesting vestingmonth vestingday fullymerged; 
RUN;

* these are the assigned ones based on "Cliff" stock awards;
DATA post2006posvestingstocks_c;
	SET cliffawards (WHERE = (vesting > 0) KEEP = company_id executive_id2 fy_end vesting vestingmonth vestingday fullymerged);
	type = 'Cliff';
RUN;

* these are the unassigned number of vesting stocks;
PROC SORT DATA = cliffawards (KEEP = company_id executive_id2 fy_end lagfy_end leftvesting fullymerged) OUT = left_c NODUPKEY;
	BY company_id executive_id2 fy_end leftvesting;
	WHERE fullymerged ^= 1;
RUN;

PROC SORT DATA = cliffawards OUT = x NODUPKEY;
	BY company_id executive_id2 fy_end;
	WHERE fullymerged;
RUN;

* More assumptiosn, v1 keeps all grant dates for all grants;
* assign the remaining stock awards equally to all grant dates of each company-executive;
* note that this include grants in all years;
* obtain grant dates of all options from a company-executive;
PROC SORT DATA = sand (KEEP = company_id executive_id2 vestingmonth vestingday) OUT = grantdate NODUPKEY;
	BY company_id executive_id2 vestingmonth vestingday;
RUN;

* calculate number of grant date in each year, as divider later;
PROC SQL;
	CREATE TABLE ndate AS
	SELECT		*, COUNT(*) AS ndate
	FROM		grantdate
	GROUP BY	company_id, executive_id2;
QUIT;

* merge unassigned stock awards evenly to each grant dates;
PROC SQL;
	CREATE TABLE post2006posvestingstocks_n AS
	SELECT		a.company_id, a.fy_end, a.lagfy_end, a.executive_id2, 
				vestingmonth, vestingday,
				leftvesting / COALESCE(ndate, 1) AS vesting,
				CASE WHEN ndate ^= . THEN "Asum"
					 ELSE "Del"
				END	AS type
	FROM		left_c a LEFT JOIN ndate b
	ON			a.company_id = b.company_id AND a.executive_id2 = b.executive_id2;
QUIT;

* not merged;
PROC SORT DATA = post2006posvestingstocks_n (WHERE = (type = "Del")) NODUPKEY OUT = notmerged;
	BY company_id fy_end executive_id2;
RUN;

* Combine stock awards from all three assumptions;
DATA post2006vestingstock_v1;
	SET post2006posvestingstocks_g (KEEP = company_id fy_end executive_id2 vesting vestingmonth vestingday type)
		post2006posvestingstocks_c (KEEP = company_id fy_end executive_id2 vesting vestingmonth vestingday type)
		post2006posvestingstocks_n (KEEP = company_id fy_end executive_id2 vesting vestingmonth vestingday type);
RUN;

PROC SQL;
	CREATE TABLE post2006vestingstocks_wdate_v1 AS
	SELECT	*
	FROM	newlyvestedshares a LEFT JOIN post2006vestingstock_v1 b
	ON		a.company_id = b.company_id AND a.executive_id2 = b.executive_id2
			AND a.fy_end = b.fy_end;
QUIT;

* Vesting equity date, other versions;
* vesting equity at the monthly level, using vesting date to assign month;
* for options, we assume that they vest and expire on the annversary of the grant;
* for stocks, post-2006 with grant dates, we use grant date;
* for stocks, pre-2006 ones and post-2006 without grant dates ones, we evenly distribute them across the post-2006 grant dates;
* NP: using Performance_ = "N/A";
* NP: also, do NOT use the last assumption because these can be non-time-based vestings;
* Only include no performance vesting options;
* retrieve option award types;
* NP: Keeping only non-performance vesting grants;
PROC SORT DATA = v.oand (KEEP = company_id executive_id2 grant_date expiration_date 
	exercise_price vesting_schedule performance_) OUT = option_np;
	BY company_id executive_id2 expiration_date exercise_price grant_date;
	WHERE grant_date ^= . AND expiration_date ^= . AND exercise_price ^= . AND performance_ = "N/A";
RUN;

* for each company-manager-expdate-stkprice, keep the first grant date;
DATA option_np;
	SET option_np;
	BY company_id executive_id2 expiration_date exercise_price grant_date;
	IF FIRST.exercise_price;
RUN;

PROC SQL;
	CREATE TABLE post2006vestingoptions_np AS
	SELECT	a.*, b.*
	FROM	post2006vestingoptions_wdate a, option_np b
	WHERE	a.company_id = b.company_id AND a.executive_id2 = b.executive_id2
			AND a.expdate = b.expiration_date AND a.stkprc = b.exercise_price
			AND vestedoptions > 0;
QUIT;

* Post 2006 vesting stocks - from Equilar;
* obtain the newly vested stocks in each year and match them back to stock awards to figure out their grant dates;
DATA post2006posvestingstocks; 
	SET newlyvestedshares (KEEP = company_id fy_end executive_id2 shares_acquired_on_vesting_of_st);
	lagfy_end = INTNX('MONTH', fy_end, -12, 'END');
	FORMAT lagfy_end MMDDYY10.; 
	IF shares_acquired_on_vesting_of_st > 0;
RUN;

* Retrieve all stock awards and calculate their vesting begin and end date;
* Vesting begin date is one year after grant date, vesting end date is vestingperiod year after grant date;
DATA sand;
	SET v.sand (KEEP = company_id executive_id2 fy_end fyear executive company ticker 
		grant_date number_of_securities vesting_schedule vestingPeriod performance_);
	
	vestingmonth = MONTH(grant_date); vestingday = DAY(grant_date); 

	* for 2/29 situation, since it does not happen every year;
	vestingday2 = vestingday; IF vestingmonth = 2 AND vestingday = 29 THEN vestingday2 = 28; 
	vestingday3 = vestingday; IF vestingmonth = 2 AND vestingday = 29 
		AND vestingperiod / 4 ^= INT(vestingperiod / 4) THEN vestingday3 = 28; 

	* set vesting begin date to be one year after the grant date and end date to be vestingperiod year after grant date;
	vestingbegdate = MDY(vestingmonth, vestingday2, YEAR(grant_date) + 1);
	vestingenddate = MDY(vestingmonth, vestingday3, YEAR(grant_date) + VestingPeriod); 

	FORMAT vestingbegdate vestingenddate MMDDYY10.; 

	DROP vestingday2 vestingday3;

	IF grant_date ^= .;
	WHERE vesting_schedule ^= "Retirement";
RUN;

* Assighed using "Graded" stock grants;
* merge post-2006 vested stocks with stock awards with Graded vesting schedule;
* make sure that the vesting period and the lagfy_end to fy_end period overlaps;
* assume for stock awards with Graded vesting schedule, an equal number of shares vested in each year;
* calculate the total number of vesting shares for each firm-manager-fy_end based on its grant history and this assumption;
PROC SQL;
	CREATE TABLE gradedawards AS
	SELECT		a.company_id, a.fy_end, a.lagfy_end, a.executive_id2, 
				a.shares_acquired_on_vesting_of_st AS totalvesting,
				number_of_securities / vestingperiod AS implied_vesting,
				SUM(number_of_securities / vestingperiod) AS sum_implied_vesting,
				vestingmonth, vestingday
	FROM		post2006posvestingstocks a LEFT JOIN 
				sand (WHERE = (vesting_schedule = "Graded" AND performance_ = "N/A")) b
	ON			a.company_id = b.company_id AND a.executive_id2 = b.executive_id2
				AND b.vestingenddate >= a.lagfy_end AND b.vestingbegdate <= a.fy_end
	GROUP BY	a.company_id, a.fy_end, a.lagfy_end, a.executive_id2;
QUIT;

* the total shares vesting based on award / grant date can be more than the reported shares vesting;
* adjust the number of vesting shares in this case;
DATA gradedawards;
	SET gradedawards;
	* if merged with any grants, then can use the grant date to assign shares;
	IF sum_implied_vesting > 0 THEN DO;
		* cannot assign more than the reported shares vesting;
		maxassigned_vesting = MIN(sum_implied_vesting, totalvesting);
		leftvesting = totalvesting - maxassigned_vesting;
	END;
	IF sum_implied_vesting = . THEN leftvesting = totalvesting;

	* ratio of shares already assigned vesting schedule, set to zero if no match to stock awards;
	vestingratio = COALESCE(maxassigned_vesting, 0) / totalvesting;

	FORMAT vestingratio PERCENT8.3;
	
	* if less than 10 shares left and vesting ratio > 99% then count as fully merged;
	fullymerged = (leftvesting <= 10 AND vestingratio > 0.99);

	IF fullymerged THEN adj = (maxassigned_vesting + leftvesting) / sum_implied_vesting;
	ELSE adj = 1;

	* adjust the implied vesting number;
	vesting = implied_vesting * adj; 
	
	KEEP company_id executive_id2 fy_end lagfy_end vesting leftvesting vestingmonth vestingday fullymerged; 
RUN;

* these are the assigned ones based on "Graded" stock awards;
DATA post2006posvestingstocks_g;
	SET gradedawards (WHERE = (vesting > 0) KEEP = company_id executive_id2 fy_end vesting vestingmonth vestingday fullymerged);
	type = 'Graded';
RUN;

* these are the unassigned number of vesting stocks;
PROC SORT DATA = gradedawards (KEEP = company_id executive_id2 fy_end lagfy_end leftvesting fullymerged) OUT = left_g NODUPKEY;
	BY company_id executive_id2 fy_end leftvesting;
	WHERE fullymerged ^= 1;
RUN;

PROC SORT DATA = gradedawards OUT = x NODUPKEY;
	BY company_id executive_id2 fy_end;
	WHERE fullymerged;
RUN;

* Assign using "Cliff" stock grants;
* merge unassigned stock awards with "Cliff" stock grants;
* make sure that the vesting end date falls between lagfy_end and fy_end;
* assign all "Cliff" stock grants vest on the vesting end date;
PROC SQL;
	CREATE TABLE cliffawards AS
	SELECT		a.company_id, a.fy_end, a.lagfy_end, a.executive_id2, 
				a.leftvesting AS totalvesting,
				number_of_securities AS implied_vesting,
				SUM(number_of_securities) AS sum_implied_vesting,
				vestingmonth, vestingday
	FROM		left_g a LEFT JOIN 
				sand (WHERE = (vesting_schedule = "Cliff" AND performance_ = "N/A")) b
	ON			a.company_id = b.company_id AND a.executive_id2 = b.executive_id2
				AND b.vestingenddate BETWEEN a.lagfy_end + 1 AND a.fy_end
	GROUP BY	a.company_id, a.fy_end, a.lagfy_end, a.executive_id2;
QUIT;

* the total shares vesting based on award / grant date can be more than the reported shares vesting;
* adjust the number of vesting shares in this case;
DATA cliffawards;
	SET cliffawards;
	* if merged with any grants, then can use the grant date to assign shares;
	IF sum_implied_vesting > 0 THEN DO;
		* cannot assign more than the reported shares vesting;
		maxassigned_vesting = MIN(sum_implied_vesting, totalvesting);
		leftvesting = totalvesting - maxassigned_vesting;
	END;
	IF sum_implied_vesting = . THEN leftvesting = totalvesting;

	* ratio of shares already assigned vesting schedule, set to zero if no match to stock awards;
	vestingratio = COALESCE(maxassigned_vesting, 0) / totalvesting;

	FORMAT vestingratio PERCENT8.3;
	
	* if less than 10 shares left and vesting ratio > 99% then count as fully merged;
	fullymerged = (leftvesting <= 10 AND vestingratio > 0.99);

	IF fullymerged THEN adj = (maxassigned_vesting + leftvesting) / sum_implied_vesting;
	ELSE adj = 1;

	* adjust the implied vesting number;
	vesting = implied_vesting * adj; 
	
	KEEP company_id executive_id2 fy_end lagfy_end vesting leftvesting vestingmonth vestingday fullymerged; 
RUN;

* these are the assigned ones based on "Cliff" stock awards;
DATA post2006posvestingstocks_c;
	SET cliffawards (WHERE = (vesting > 0) KEEP = company_id executive_id2 fy_end vesting vestingmonth vestingday fullymerged);
	type = 'Cliff';
RUN;

* these are the unassigned number of vesting stocks;
PROC SORT DATA = cliffawards (KEEP = company_id executive_id2 fy_end lagfy_end leftvesting fullymerged) OUT = left_c NODUPKEY;
	BY company_id executive_id2 fy_end leftvesting;
	WHERE fullymerged ^= 1;
RUN;

PROC SORT DATA = cliffawards OUT = x NODUPKEY;
	BY company_id executive_id2 fy_end;
	WHERE fullymerged;
RUN;

* alternative definition of Time-based, still assign the remaining ones;
* assign the remaining stock awards equally to all grant dates of each company-executive;
* note that this include grants in all years;

* obtain grant dates of all options from a company-executive;
PROC SORT DATA = sand (KEEP = company_id executive_id2 vestingmonth vestingday vesting_schedule performance_) 
		OUT = grantdate (DROP = vesting_schedule performance_) NODUPKEY;
	BY company_id executive_id2 vestingmonth vestingday;
RUN;

* calculate number of grant date in each year, as divider later;
PROC SQL;
	CREATE TABLE ndate AS
	SELECT		*, COUNT(*) AS ndate
	FROM		grantdate
	GROUP BY	company_id, executive_id2;
QUIT;

* merge unassigned stock awards evenly to each grant dates;
PROC SQL;
	CREATE TABLE post2006posvestingstocks_n AS
	SELECT		a.company_id, a.fy_end, a.lagfy_end, a.executive_id2, 
				vestingmonth, vestingday,
				leftvesting / COALESCE(ndate, 1) AS vesting,
				CASE WHEN ndate ^= . THEN "Asum"
					 ELSE "Del"
				END	AS type
	FROM		left_c a LEFT JOIN ndate b
	ON			a.company_id = b.company_id AND a.executive_id2 = b.executive_id2;
QUIT;

* not merged;
PROC SORT DATA = post2006posvestingstocks_n (WHERE = (type = "Del")) NODUPKEY OUT = notmerged;
	BY company_id fy_end executive_id2;
RUN;

* Combine stock awards from the two assumptions;
* alternative version of time-based;
DATA post2006vestingstock_v4;
	SET post2006posvestingstocks_g (KEEP = company_id fy_end executive_id2 vesting vestingmonth vestingday type)
		post2006posvestingstocks_c (KEEP = company_id fy_end executive_id2 vesting vestingmonth vestingday type)
		post2006posvestingstocks_n (KEEP = company_id fy_end executive_id2 vesting vestingmonth vestingday type);
RUN;

PROC SQL;
	CREATE TABLE post2006vestingstocks_wdate_v4 AS
	SELECT	*
	FROM	newlyvestedshares a LEFT JOIN post2006vestingstock_v4 b
	ON		a.company_id = b.company_id AND a.executive_id2 = b.executive_id2
			AND a.fy_end = b.fy_end;
QUIT;

* Create monthly index;
* Use v.shares annual to populate qtr data;
* copy to Q4;
PROC SQL;
	CREATE TABLE q4 AS
	SELECT	company_id, executive, executive_id2, fy_end, gvkey, fyear, company_name,
			conm, shares_acquired_on_vesting_of_st, value_realized_on_vesting,
			finalprc AS oldprc, lagfinalprc AS oldlagprc, 4 AS fqtr,
			datadate, INTNX('MONTH', datadate, -3, 'END') AS lagdatadate FORMAT MMDDYY10.
	FROM	v.shares;
QUIT;

* copy to Q3;
PROC SQL;
	CREATE TABLE q3 AS
	SELECT	company_id, executive, executive_id2, fy_end, gvkey, fyear, company_name,
			conm, shares_acquired_on_vesting_of_st, value_realized_on_vesting,
			finalprc AS oldprc, lagfinalprc AS oldlagprc, 3 AS fqtr,
			INTNX('MONTH', datadate, -3, 'END') AS datadate FORMAT MMDDYY10., 
			INTNX('MONTH', datadate, -6, 'END') AS lagdatadate FORMAT MMDDYY10.
	FROM	v.shares;
QUIT;

* copy to Q2;
PROC SQL;
	CREATE TABLE q2 AS
	SELECT	company_id, executive, executive_id2, fy_end, gvkey, fyear, company_name,
			conm, shares_acquired_on_vesting_of_st, value_realized_on_vesting,
			finalprc AS oldprc, lagfinalprc AS oldlagprc, 2 AS fqtr,
			INTNX('MONTH', datadate, -6, 'END') AS datadate FORMAT MMDDYY10., 
			INTNX('MONTH', datadate, -9, 'END') AS lagdatadate FORMAT MMDDYY10.
	FROM	v.shares;
QUIT;

* copy to Q1;
PROC SQL;
	CREATE TABLE q1 AS
	SELECT	company_id, executive, executive_id2, fy_end, gvkey, fyear, company_name,
			conm, shares_acquired_on_vesting_of_st, value_realized_on_vesting,
			finalprc AS oldprc, lagfinalprc AS oldlagprc, 1 AS fqtr,
			INTNX('MONTH', datadate, -9, 'END') AS datadate FORMAT MMDDYY10.,
			INTNX('MONTH', datadate, -12, 'END') AS lagdatadate FORMAT MMDDYY10.
	FROM	v.shares;
QUIT;

* merge all quarters;
DATA qall;
	SET q1 q2 q3 q4;
RUN;

* remove duplicate;
PROC SORT DATA = qall; 
	BY gvkey executive_id2 datadate fy_end;
RUN;
DATA qall;
	SET qall;
	BY gvkey executive_id2 datadate fy_end;
	IF LAST.datadate;
RUN;

* Create monthly level data from quarterly index;
* copy to M1;
PROC SQL;
	CREATE TABLE m1 AS
	SELECT	gvkey, company_id, company_name, conm, executive, executive_id2, 
			fy_end, fyear, fqtr, 3 AS fmonth,
			INTNX('MONTH', datadate, -2, 'END') AS datadate FORMAT MMDDYY10., 
			INTNX('MONTH', datadate, -3, 'END') AS lagdatadate FORMAT MMDDYY10.,
			shares_acquired_on_vesting_of_st, value_realized_on_vesting
	FROM	qall;
QUIT;

* copy to M2;
PROC SQL;
	CREATE TABLE m2 AS
	SELECT	gvkey, company_id, company_name, conm, executive, executive_id2, 
			fy_end, fyear, fqtr, 3 AS fmonth,
			INTNX('MONTH', datadate, -1, 'END') AS datadate FORMAT MMDDYY10., 
			INTNX('MONTH', datadate, -2, 'END') AS lagdatadate FORMAT MMDDYY10.,
			shares_acquired_on_vesting_of_st, value_realized_on_vesting
	FROM	qall;
QUIT;

* copy to M3;
PROC SQL;
	CREATE TABLE m3 AS
	SELECT	gvkey, company_id, company_name, conm, executive, executive_id2, 
			fy_end, fyear, fqtr, 3 AS fmonth,
			datadate, INTNX('MONTH', datadate, -1, 'END') AS lagdatadate FORMAT MMDDYY10.,
			shares_acquired_on_vesting_of_st, value_realized_on_vesting
	FROM	qall;
QUIT;

* merge all months;
DATA mall;
	SET m1 m2 m3;
RUN;

* remove duplicates;
PROC SORT DATA = mall; 
	BY gvkey executive_id2 datadate fy_end;
RUN;
DATA mall;
	SET mall;
	BY gvkey executive_id2 datadate fy_end;
	IF LAST.datadate;
RUN;

* add permnos;
PROC SQL;
	CREATE TABLE mall AS
	SELECT	a.*, lpermno, 
			lagdatadate >= linkdt AND (lagdatadate <= linkenddt OR linkenddt = .E) AS link_lagdatadate,
			datadate >= linkdt AND (datadate <= linkenddt OR linkenddt = .E) AS link_datadate
	FROM	mall a,	compustat_crsp_link b
	WHERE	a.gvkey = b.gvkey AND 
			(lagdatadate >= linkdt AND (lagdatadate <= linkenddt OR linkenddt = .E)
			OR datadate >= linkdt AND (datadate <= linkenddt OR linkenddt = .E));
QUIT;
PROC SQL; 
	CREATE TABLE mall (DROP = link_lagdatadate link_datadate) AS
	SELECT		*
	FROM		mall
	GROUP BY	gvkey, executive_id2, datadate, fy_end
	HAVING		COUNT(*) = 1 OR link_lagdatadate
	ORDER BY	datadate;
QUIT;

* Match with CRSP Month End Price;
%LET wrds = wrds-cloud.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME crspa '/wrds/crsp/sasdata/a_stock'; 

PROC SQL UNDO_POLICY = NONE;
	CREATE TABLE mprice AS
	SELECT 		permno, date, prc, shrout, cfacpr, cfacshr
	FROM 		crspa.msf 
	WHERE		YEAR(date) >= 2005
	ORDER BY	permno, date, prc DESC; 
QUIT;	

PROC DOWNLOAD DATA = mprice OUT = mprice; RUN;
ENDRSUBMIT;

* datadate is the month end and lagdatadate is the last month end;
PROC SQL;
	CREATE TABLE mallp AS
	SELECT	a.*, ABS(b.prc) AS prc, ABS(c.prc) AS lagprc, 
			b.cfacpr, b.cfacshr, c.cfacpr AS cfacpr_lag, c.cfacshr AS cfacshr_lag
	FROM	mall a LEFT JOIN mprice b
	ON		a.lpermno = b.permno AND a.datadate = INTNX('MONTH', b.date, 0, 'END')
			LEFT JOIN mprice c
	ON		a.lpermno = c.permno AND a.lagdatadate = INTNX('MONTH', c.date, 0, 'END');
QUIT;

PROC SORT DATA = mallp NODUPKEY; 
	BY gvkey executive_id2 datadate;
RUN;

* adjust the price when lagprc is missing (so we have to use prc) and then the cumulative adjustment factor changed;
DATA v.mallp;
	SET mallp;
	IF prc ^= . AND lagprc = . AND 
			(cfacpr ^= cfacpr_lag AND cfacpr_lag ^= . OR cfacshr ^= cfacshr_lag AND cfacshr_lag ^= .) 
		THEN prc = prc * (cfacshr_lag / cfacshr);
	DROP cfacpr cfacshr cfacpr_lag cfacshr_lag;
RUN;

* Newly Vesting Shares and Options; 
* Merge diff versions of shares vesting schedule with monthly index;
%MACRO vestingshares_m(var);
	DATA vestingshares; 
		SET post2006vestingstocks_&var.;
		lagfy_end = INTNX('MONTH', fy_end, -12, 'END');
		vestingyear = YEAR(fy_end) - (vestingmonth > MONTH(fy_end));
		IF vestingmonth = 2 AND vestingday = 29 AND vestingyear / 4 ^= INT(vestingyear / 4) THEN vestingday = 28;
		vestingtime = MDY(vestingmonth, vestingday, vestingyear);
		FORMAT lagfy_end vestingtime MMDDYY10.; 
		DROP shares_acquired_on_vesting_of_st value_realized_on_vesting executive company_name
			vestingyear vestingmonth vestingday;
		WHERE vesting > 0; 
	RUN;

	* aggregate all vesting shares in each month;
	PROC SQL;
		CREATE TABLE mallpshares_&var AS 
		SELECT		a.company_id, executive, a.executive_id2, a.fy_end, gvkey, lagfy_end, prc, lagprc, 
					fyear, datadate, lagdatadate, fqtr, fmonth, lpermno, ticker, company_name, 
					shares_acquired_on_vesting_of_st, value_realized_on_vesting,
					SUM(vesting) AS vestingshares 
		FROM		v.mallp a LEFT JOIN vestingshares b
		ON			a.company_id = b.company_id AND a.fy_end = b.fy_end AND a.executive_id2 = b.executive_id2
					AND b.vestingtime BETWEEN a.lagdatadate + 1 AND a.datadate
		GROUP BY 	a.company_id, executive, a.executive_id2, a.fy_end, gvkey, lagfy_end, prc, lagprc, 
					fyear, datadate, lagdatadate, fqtr, fmonth, lpermno, ticker, company_name, 
					shares_acquired_on_vesting_of_st, value_realized_on_vesting;
	QUIT;
%MEND vestingshares_m;

* Merge diff versions of option vesting schedule with monthly index;
%MACRO newlyvesting_m (var1, var2, var3);

	* vesting options on grant level, define exact vesting date;
	DATA vestingoptions_&var1; 
		SET post2006vestingoptions_&var1;
		lagfy_end = INTNX('MONTH', fy_end, -12, 'END');
		vestingyear = YEAR(fy_end) - (vestingmonth > MONTH(fy_end));
		IF vestingmonth = 2 AND vestingday = 29 AND vestingyear / 4 ^= INT(vestingyear / 4) THEN vestingday = 28;
		vestingtime = MDY(vestingmonth, vestingday, vestingyear);
		FORMAT lagfy_end vestingtime MMDDYY10.; 
		DROP vestingyear vestingmonth vestingday;
		WHERE vestedoptions > 0; 
	RUN;
	
	* merge vesting options with their BS assumptions;
	PROC SQL;
		CREATE TABLE mallpoptionsbs_&var3 AS
		SELECT	a.*, b.expdate, b.stkprc, b.vestedoptions,
				b.term AS oldterm, b.vestingtime,
				(b.expdate - datadate) / 365 AS term,
				CALCULATED term + 0.25 AS lagterm,
				c.*
		FROM 	mallpshares_&var2 a LEFT JOIN vestingoptions_&var1 b
		ON		a.company_id = b.company_id AND a.executive_id2 = b.executive_id2 AND a.fy_end = b.fy_end
				AND b.vestingtime BETWEEN a.lagdatadate + 1 AND a.datadate
				LEFT JOIN bsassumptions c
		ON		a.company_id = c.company_id AND a.fy_end = c.fy_end AND a.executive_id2 = c.executive_id2
				AND b.expdate = c.expdate AND b.stkprc = c.stkprc AND b.vestedoptions = c.vestedoptions;
	QUIT;
		
	* Calculate value of newly vested options;
	* Calculate Z, Delta, and Option values;
	DATA mallpoptionsbs_cal_&var3; 
		SET mallpoptionsbs_&var3;
		IF lagterm > 0 AND stkprc > 0 AND finalvol1 ^= 0 THEN DO;
			Z1 = (LOG(lagprc / stkprc) + lagterm * (finalrfr1 - finaldivyield1 + finalvol1**2 / 2)) / (finalvol1 * SQRT(lagterm));
			
			IF lagprc = . AND prc ^= . THEN DO;
				Z1 = (LOG(prc / stkprc) + lagterm * (finalrfr1 - finaldivyield1 + finalvol1**2 / 2)) / (finalvol1 * SQRT(lagterm));
			END;
			
			delta1_un = EXP(-finaldivyield1 * lagterm) * CDF("NORMAL", Z1);
			
			delta1 = EXP(-finaldivyield1 * lagterm) * CDF("NORMAL", Z1) * vestedoptions;
		END;
		* For v2, assume delta is replaced with their intrinsic value;
		%IF &var3 = v2 %THEN %DO;
			delta1_un = (stkprc <= lagprc);
			delta1 = delta1_un * vestedoptions;
		%END;
	
		* For v3, assume all options are at the money;
		%IF &var3 = v3 %THEN %DO;
			IF lagterm > 0 AND finalvol1 ^= 0 THEN DO;			
				Z1 = (LOG(1) + lagterm * (finalrfr1 - finaldivyield1 + finalvol1**2 / 2)) / (finalvol1 * SQRT(lagterm));
		
				delta1_un = EXP(-finaldivyield1 * lagterm) * CDF("NORMAL", Z1);
		
				delta1 = EXP(-finaldivyield1 * lagterm) * CDF("NORMAL", Z1) * vestedoptions;
			END;
		%END;
	RUN;

	PROC MEANS DATA = mallpoptionsbs_cal_&var3;
		VAR delta1_un;
		WHERE vestedoptions > 0;
	RUN;

	* aggregate total vesting option value in each month;
	PROC SQL;
		CREATE TABLE mnewlyvesting_&var3 AS
		SELECT		company_id, fy_end, lagfy_end, gvkey, lpermno, prc, lagprc, fyear, fmonth, datadate, lagdatadate, fqtr,
					executive, ticker, company_name, shares_acquired_on_vesting_of_st,
					value_realized_on_vesting, executive_id2, vestingshares, SUM(vestedoptions) AS vestingoptions, 
					SUM(delta1) AS delta1
		FROM		mallpoptionsbs_cal_&var3
		GROUP BY	company_id, fy_end, lagfy_end, gvkey, lpermno, prc, lagprc, fyear, fmonth, datadate, lagdatadate, fqtr,
					executive, ticker, company_name, shares_acquired_on_vesting_of_st,
					value_realized_on_vesting, executive_id2, vestingshares; 
	QUIT;
%MEND newlyvesting_m;

* retrieve the calculated BS assumptions, only keep the ones with positive vestedoptions;
DATA bsassumptions; 
	SET bsassumptions_newlyvestedoptions; 
	KEEP company_id executive_id2 fy_end expdate stkprc vestedoptions
	finaldivyield1 finalvol1 finalrfr1 
	missingdiv missingvol missingrfr;
	IF vestedoptions > 0;
RUN;

PROC SORT DATA = bsassumptions NODUPKEY;
	BY company_id fy_end executive_id2 expdate stkprc vestedoptions;
RUN;

%vestingshares_m(wdate_v1); 
%vestingshares_m(wdate_v4);
%newlyvesting_m (wdate, wdate_v1, v1); 
%newlyvesting_m (wdate, wdate_v1, v2); 
%newlyvesting_m (wdate, wdate_v1, v3);
%newlyvesting_m (np, wdate_v4, v4);

* match insider trading with equilar;
* Download insider trading data from TFN;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME insider '/wrds/tfn/sasdata/insiders'; 

PROC DOWNLOAD DATA = insider.table1 (WHERE = (YEAR(cdate) >= 2005)) OUT = insider_table1; RUN;

ENDRSUBMIT;

* Match TFN with Equilar Exeuctives;
* match insider trading with MM using 6-digit cusip;
PROC SQL;
	CREATE TABLE cusip_match AS
	SELECT		company_id, executive_id2, fy_end, lagfy_end, cusip, cusip6, 
				conm, company_name, cname, personid, executive, owner, rolecode1
	FROM		v.mm (WHERE = (lagexist = 1)) a, insider_table1 b
	WHERE		SUBSTR(cusip, 1, 6) = cusip6 AND cusip6 ^= '' AND
				b.trandate BETWEEN a.lagfy_end AND a.fy_end;
QUIT;
PROC SORT DATA = cusip_match NODUPKEY;
	BY company_id executive_id2 fy_end lagfy_end cusip executive cusip6
		conm company_name personid owner;
RUN;

* name matches;
DATA cusip_match2;
	SET cusip_match;
	name1 = COMPRESS(SCAN(executive, 1, ',. -'));
	name2 = COMPRESS(SCAN(executive, 2, ',. -'));
	name3 = COMPRESS(SCAN(executive, 3, ',. -'));
	ownname1 = COMPRESS(SCAN(owner, 1, ' '));
	ownname2 = COMPRESS(SCAN(owner, 2, ' '));
	ownname3 = COMPRESS(SCAN(owner, 3, ' '));
	ownname4 = COMPRESS(SCAN(owner, 4, ' '));
	match1 = (name1 = ownname1 OR name1 = ownname2 OR name1 = ownname3 OR name1 = ownname4);
	IF LENGTH(name2) > 1 THEN match2 = (name2 = ownname1 OR name2 = ownname2 OR name2 = ownname3 OR name2 = ownname4);
	IF LENGTH(name3) > 1 THEN match3 = (name3 = ownname1 OR name3 = ownname3 OR name3 = ownname3 OR name3 = ownname4);
	IF match1 OR match2 OR match3;
	DROP name1 name2 name3 ownname1 ownname2 ownname3 ownname4;
RUN;

* Classify these matches;
* Perfect matches without dupkey;
DATA namematch;
	SET cusip_match2;
	IF match1 AND match2 OR match1 AND match3;
RUN;

PROC SQL;
	CREATE TABLE namematch_nodupkey AS
	SELECT		*, COUNT(*) AS n_personid
	FROM		namematch
	GROUP BY	executive_id2, fy_end
	HAVING		COUNT(*) = 1;
QUIT;

* Perfect matches BUT with dupkey;
PROC SQL;
	CREATE TABLE namematch_dupkey AS
	SELECT		lagfy_end, cusip, cusip6, conm, company_name, cname, personid, company_id,
				executive_id2, fy_end, executive, owner, rolecode1, match1, match2, match3
	FROM		namematch
	WHERE		executive_id2 NOT IN (SELECT UNIQUE executive_id2 FROM namematch_nodupkey)
	ORDER BY	company_id, executive_id2, fy_end, personid;
QUIT;
DATA namematch_dupkey;
	SET namematch_dupkey;
	new = (company_id ^= LAG(company_id) OR fy_end ^= LAG(fy_end));
RUN;

PROC EXPORT DATA = namematch_dupkey
	OUTFILE = "D:\Data\TFNnamematch_namematch_dupkey.xls" REPLACE
	DBMS = EXCELCS; 
	SHEET = "namematch_dupkey";
RUN;

* Non perfect matches;
PROC SQL;
	CREATE TABLE non_namematch AS
	SELECT		lagfy_end, cusip, cusip6, conm, company_name, cname, personid, company_id,
				executive_id2, fy_end, executive, owner, rolecode1, match1, match2, match3
	FROM		cusip_match2
	WHERE		executive_id2 NOT IN (SELECT UNIQUE executive_id2 FROM namematch)
	ORDER BY	company_id, executive_id2, fy_end, personid, lagfy_end, cusip, executive,
				cusip6, conm, company_name, cusip6, owner;
QUIT;
DATA non_namematch;
	SET non_namematch;
	new = (company_id ^= LAG(company_id) OR fy_end ^= LAG(fy_end));
RUN;

* output for further manual confirmation;
PROC EXPORT DATA = non_namematch
	OUTFILE = "D:\Data\TFNnamematch_non_namematch.xls" REPLACE
	DBMS = EXCELCS; 
	SHEET = "non_namematch";
RUN;

* Import files after verifying;
PROC IMPORT DATAFILE = "D:\Data\TFNnamematch_namematch_dupkey.xls" 
	OUT = namematch_dupkey_checked
	DBMS = EXCELCS
	REPLACE; 
	SHEET = "checked_ok";
RUN;
PROC SORT DATA = namematch_dupkey_checked OUT = temp NODUPKEY;
	BY company_id fy_end;
	WHERE verified;
RUN;

PROC IMPORT DATAFILE = "D:\Data\TFNnamematch_non_namematch.xls" 
	OUT = non_namematch_checked
	DBMS = EXCELCS
	REPLACE; 
	SHEET = "checked_ok";
RUN;
PROC SORT DATA = non_namematch_checked OUT = temp NODUPKEY;
	BY company_id fy_end;
	WHERE verified;
RUN;

DATA mergedname;
	SET namematch_nodupkey (KEEP = executive_id2 company_id fy_end lagfy_end cusip executive
		cusip6 conm company_name personid owner rolecode1)
		namematch_dupkey_checked (KEEP = executive_id2 company_id fy_end lagfy_end cusip executive
			cusip6 conm company_name personid owner rolecode1 verified WHERE = (verified = 1))
		non_namematch_checked (KEEP = executive_id2 company_id fy_end lagfy_end cusip executive
			cusip6 conm company_name personid owner rolecode1 verified WHERE = (verified = 1));
	RENAME 	owner = ownermatched
			rolecode1 = rolecodematched;
	LABEL 	owner = ownermatched
			rolecode1 = rolecodematched;
	DROP verified;
RUN;

* calculate compustat related variables;
* Retrive data from Compustat quarterly file;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME compna '/wrds/comp/sasdata/naa'; 

PROC SQL UNDO_POLICY = NONE;
	CREATE TABLE comvarq_wrds AS
	SELECT 	gvkey, datadate, fyearq, fqtr, fyr, cik, cusip, conm, datacqtr, datafqtr, apdedateq, fdateq, rdq,
			atq, saleq, saley, 
			cshopq, prcraq, cshiq, sstky, spstkcy, scstkcy,
			cshoq, prccq, cshprq, cshfdq, ceqq,
			chq, cheq, niq, oibdpq, oiadpq, nopiq, oancfy, dpq, sretq, actq, lctq,
			ltq, dlcq, dlttq, dltisy, dltry, fincfy, pstkq, txditcq, dvpq, dvpy, pdvcy, cdvcy, dvy,
			tstknq, tstkq,
			ppegtq, ppentq, capxy, xrdq, req
	FROM 	compna.fundq 
	WHERE	indfmt = 'INDL' AND datafmt = 'STD' AND popsrc = 'D' AND consol = 'C' 
			AND fyearq >= 2003; 
QUIT;	

PROC DOWNLOAD DATA = comvarq_wrds OUT = comvarq; RUN;
ENDRSUBMIT;

* delete firm-qtrs with missing total assets;
DATA comvarq;
	SET comvarq;
	absxrdq = ABS(xrdq); 
	abscapxy = ABS(capxy);
	mvq = cshoq * prccq;
	WHERE atq ^= .;
RUN;

PROC SORT DATA = comvarq NODUPKEY;
	BY gvkey fyearq fqtr datadate;
RUN;

PROC SQL;
	CREATE TABLE duplicate_firmyear AS
	SELECT		*, COUNT(*) AS ncompany
	FROM		comvarq
	GROUP BY	gvkey, fyearq, fqtr
	HAVING		COUNT(*) > 1;
QUIT;

PROC SQL;
	CREATE TABLE duplicate_firmdate AS
	SELECT		*, COUNT(*) AS ncompany
	FROM		comvarq
	GROUP BY	gvkey, datadate
	HAVING		COUNT(*) > 1;
QUIT;

* remove duplicate firm-datadate;
PROC SORT DATA = comvarq;
	BY gvkey datadate DESCENDING absxrdq DESCENDING abscapxy;
RUN;

DATA comvarq;
	SET comvarq;
	BY gvkey datadate DESCENDING absxrdq DESCENDING abscapxy;
	IF FIRST.datadate;
RUN;

* Calculate investment measures;
PROC SQL;
	CREATE TABLE investmentq1 AS
	SELECT	a.*, b.datadate AS lagdatadate, c.datadate AS lag2datadate,
			b.fqtr AS lagfqtr, b.atq AS lagatq, b.saleq AS lagsaleq, b.ppegtq AS lagppegtq, 
			b.ppentq AS lagppentq, b.capxy AS lagcapxy, b.xrdq AS lagxrdq,
			c.fqtr AS lag2fqtr, c.atq AS lag2atq, c.saleq AS lag2saleq, c.ppegtq AS lag2ppegtq, 
			c.ppentq AS lag2ppentq, c.capxy AS lag2capxy, c.xrdq AS lag2xrdq
	FROM	comvarq (KEEP = gvkey datadate fyearq fqtr fyr cik cusip conm datacqtr datafqtr
			rdq atq ppegtq ppentq capxy xrdq saleq saley) a,
			comvarq (KEEP = gvkey datadate fqtr atq saleq ppegtq ppentq capxy xrdq) b,
			comvarq (KEEP = gvkey datadate fqtr atq saleq ppegtq ppentq capxy xrdq) c
	WHERE	a.gvkey = b.gvkey AND a.gvkey = c.gvkey 
			AND INTNX('MONTH',a.datadate, -3, 'END') = b.datadate
			AND INTNX('MONTH',a.datadate, -6, 'END') = c.datadate;
QUIT;

PROC SQL;
	CREATE TABLE investmentq2 AS
	SELECT	a.*, 
			d.fqtr AS lag4fqtr, d.atq AS lag4atq, d.saleq AS lag4saleq, d.ppegtq AS lag4ppegtq, 
			d.ppentq AS lag4ppentq, d.capxy AS lag4capxy, d.xrdq AS lag4xrdq,
			e.fqtr AS lag5fqtr, e.atq AS lag5atq, e.saleq AS lag5saleq, d.ppegtq AS lag5ppegtq, 
			e.ppentq AS lag5ppentq, e.capxy AS lag5capxy, e.xrdq AS lag5xrdq
	FROM	investmentq1 a LEFT JOIN
			comvarq (KEEP = gvkey datadate fqtr atq saleq ppegtq ppentq capxy xrdq) d
	ON		a.gvkey = d.gvkey AND INTNX('MONTH',a.datadate, -12, 'END') = d.datadate
			LEFT JOIN
			comvarq (KEEP = gvkey datadate fqtr atq saleq ppegtq ppentq capxy xrdq) e
	ON		a.gvkey = e.gvkey AND INTNX('MONTH',a.datadate, -15, 'END') = e.datadate;
QUIT;

DATA qtrinvestment;
	SET investmentq2;
	IF lagatq ^= 0 THEN DO;
		rdatq = COALESCE(xrdq, 0) / lagatq;	
		deltardatq = SUM(xrdq, -1 * lagxrdq, 0) / lagatq; 
		IF lag4fqtr ^= . THEN delta4rdatq = SUM(xrdq, -1 * lag4xrdq, 0) / lagatq;
	END;
	missingrdatq = (xrdq = .);
	missingdeltardatq = (xrdq = . OR lagxrdq = .);
	missingdelta4rdatq = (xrdq = . OR lag4xrdq = .);

	IF fqtr = 1 THEN capxq = capxy;
	ELSE capxq = capxy - lagcapxy;
	IF lagfqtr = 1 THEN lagcapxq = lagcapxy;
	ELSE lagcapxq = lagcapxy - lag2capxy;

	IF lagatq ^= 0 THEN DO;
		capxatq = COALESCE(capxq, 0) / lagatq; 
		deltacapxatq = SUM(capxq, -1 * lagcapxq, 0) / lagatq;
	END;
	missingcapxatq = (capxq = .);
	missingdeltacapxatq = (capxq = . OR lagcapxq = .);

	IF lag4fqtr = 1 THEN lag4capxq = lag4capxy;
	ELSE lag4capxq = lag4capxy - lag5capxy;
	IF lag4fqtr ^= . AND lag5fqtr ^= . AND lagatq ^= 0 THEN delta4capxatq = SUM(capxq, -1 * lag4capxq, 0) / lagatq;
	missingdelta4capxatq = (capxq = . OR lag4capxq = .);

	IF lagatq ^= 0 THEN DO;
		netinvatq = SUM(ppentq, -1 * lagppentq, 0) / lagatq; 
		deltanetinvatq = SUM(ppentq, lag2ppentq, -2 * lagppentq, 0) / lagatq; 
	END;
	missingnetinvatq = (ppentq = . OR lagppentq = .);
	missingdeltanetinvatq = (ppentq = . OR lagppentq = . OR lag2ppentq = .);

	IF lag4fqtr ^= . AND lag5fqtr ^= . AND lagatq ^= 0 THEN 
		delta4netinvatq = SUM(ppentq, -1 * lagppentq, lag5ppentq, -lag4ppentq, 0) / lagatq; 
	missingdelta4netinvatq = (ppentq = . OR lagppentq = . OR lag4ppentq = . OR lag5ppentq = .);

	rdnetinvatq = SUM(rdatq, netinvatq);
	rdcapxatq = SUM(rdatq, capxatq);
	deltardnetinvatq = SUM(deltardatq, deltanetinvatq);
	deltardcapxatq = SUM(deltardatq, deltacapxatq);
	delta4rdnetinvatq = SUM(delta4rdatq, delta4netinvatq);
	delta4rdcapxatq = SUM(delta4rdatq, delta4capxatq);

	KEEP gvkey datadate deltardatq deltacapxatq deltardcapxatq; 
RUN;

* Remove duplicate firm-datadate;
* Delete firm-quarters with missing AT or mvq;
DATA comvarq;
	SET comvarq;
	mvq = cshoq * prccq;
	IF atq ^= . AND mvq ^= .;
RUN;

* Keep the ones with largest CSHOPQ and SALEQ;
PROC SORT DATA = comvarq;
	BY gvkey datadate DESCENDING cshopq DESCENDING saleq;
RUN;
DATA comvarq;
	SET comvarq;
	BY gvkey datadate DESCENDING cshopq DESCENDING saleq;
	IF FIRST.datadate;
RUN;

* Controls from Compustat;
PROC SQL;
	CREATE TABLE controlq AS
	SELECT	a.gvkey, a.datadate, a.fyearq, mvq, a.atq,
			SUM(mvq, dlttq, dlcq) / a.atq AS mkbk,
			LOG(a.saleq) AS logsaleq, a.saleq,
			SUM(dlttq, 0) / a.atq AS debt,
			2 * oibdpq / (a.atq + b.atq) AS operatingroaq,
			2 * nopiq / (a.atq + b.atq) AS noperatingroaq,
			SUM(dlttq, dlcq, 0) / SUM(mvq, pstkq, dlttq, dlcq, -1 * txditcq) AS market_lev2
	FROM	comvarq a, comvarq (KEEP = gvkey datadate atq saleq) b
	WHERE	a.gvkey = b.gvkey AND INTNX('MONTH', a.datadate, -3, 'END') = b.datadate;
QUIT;

* rolling average of market leverage;
PROC SQL;
	CREATE TABLE controlq AS
	SELECT	*
	FROM	controlq a,
			(SELECT		a.*, AVG(b.market_lev2) AS mklev
			FROM		controlq (KEEP = gvkey datadate) a LEFT JOIN controlq (KEEP = gvkey datadate market_lev2) b
			ON			a.gvkey = b.gvkey AND b.datadate BETWEEN a.datadate - 364 AND a.datadate
			GROUP BY	a.gvkey, a.datadate) b
	WHERE	a.gvkey = b.gvkey AND a.datadate = b.datadate;
QUIT;

* download industry code;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME compna '/wrds/comp/sasdata/naa';
* historical industry classification;
PROC SQL;
	CREATE TABLE sich AS
	SELECT	gvkey, fyear, sich
	FROM	compna.funda
	WHERE	indfmt = 'INDL' AND datafmt = 'STD' AND popsrc = 'D' AND consol = 'C'
			AND fyear >= 1993;
QUIT;

PROC DOWNLOAD data = sich OUT = sich; RUN;

LIBNAME company '/wrds/comp/sasdata/naa/company';
* industry classification;
PROC SQL;
	CREATE TABLE sic AS
	SELECT	gvkey, sic
	FROM	company.company;
QUIT;

PROC DOWNLOAD data = sic OUT = sic; RUN;
ENDRSUBMIT;

DATA sic;
	SET sic;
	sic_num = sic * 1;
RUN;

* HHI sales;
PROC SQL;
	CREATE TABLE ind_adj_control AS
	SELECT		a.gvkey, a.fyearq, a.datadate, saleq,
				CASE 
					WHEN saleq < 0 AND saleq ^= . THEN 0
					ELSE saleq
				END AS saleq_pos, 
				COALESCE(INT(b.sich / 10), INT(c.sic_num / 10), 0) AS sic3
	FROM		controlq (KEEP = gvkey fyearq datadate saleq) a 
				LEFT JOIN sich b
	ON			a.gvkey = b.gvkey AND (a.fyearq = b.fyear OR a.fyearq = 2016 AND b.fyear = 2015)
				LEFT JOIN sic c
	ON			a.gvkey = c.gvkey
	ORDER BY	sic3, fyearq;
QUIT;

* HHI Industry;
PROC SQL;
	CREATE TABLE hhisic3 AS
	SELECT		sic3, fyearq, 
				SUM(saleq_pos**2) / SUM(saleq_pos)**2 AS hhisic3
	FROM		ind_adj_control
	GROUP BY	sic3, fyearq;
QUIT;

PROC SQL;
	CREATE TABLE ind_adj_control AS
	SELECT	a.gvkey, datadate, a.sic3,
			hhisic3
	FROM	ind_adj_control a LEFT JOIN hhisic3 d
	ON		a.sic3 = d.sic3 AND A.fyearq = d.fyearq;
QUIT;

PROC SORT DATA = ind_adj_control NODUPKEY;
	BY gvkey datadate;
RUN;

* Merge controls from Compustat;
PROC SQL;
	CREATE TABLE controlq AS
	SELECT	a.*, d.*
	FROM	controlq a LEFT JOIN ind_adj_control d
	ON		a.gvkey = d.gvkey AND a.datadate = d.datadate;
QUIT;

* Calculate Industry M&A Liquidity;
* calculate MA liquidity;
* Import SDC data;
%MACRO read_in_SDC_complete1;
	%DO year = 2006 %TO 2016;
		* read in SDC;
		PROC IMPORT DATAFILE = "D:\Data\complete1_&year..xls" 
			OUT = ma_complete1_&year
			DBMS = EXCELCS
			REPLACE; 
			SHEET = "Request 2";
		RUN;

		* clean up variables;
		DATA ma_complete1_&year;
			SET ma_complete1_&year;
			RENAME 	Acquiror_Primary___SIC___Code = acquiror_sic
					Target_Primary___SIC__Code = target_sic
					Acquiror__CUSIP = acquiror_cusip
					__Date_Announced = anndate
					__Date_Effective = effdate
					_deal_number = deal_number
					Target_Total_Assets__One__Year_P = target_at_1y
					Target_Total_Assets___mil_ = target_at
					Acquiror__Total__Assets____mil_ = acquiror_at
					Acquiror_Market_Val_4_Weeks_Prio = acquiror_mv_4w
					Tgt__Market_Val_4_Wwks_Prior_to_ = target_mv_4w
					Value__Cash___mil_ = value_cash
					__of_Cash = pct_cash
					__of_Other = pct_other
					__of_Stock = pct_stock
					__of_Unknown = pct_unknown;
			LABEL 	Acquiror_Primary___SIC___Code = Acquiror SIC
					Target_Primary___SIC__Code = Target SIC
					Acquiror__CUSIP = Acquiror_CUSIP
					__Date_Announced = anndate
					__Date_Effective = effdate
					Target_CUSIP = Target_CUSIP
					Target_Name = Target_Name
					Target_Total_Assets__One__Year_P = Target Total Assets 1 year prior
					Target_Total_Assets___mil_ = Target Total Assets
					Acquiror__Total__Assets____mil_ = Acquiror Total Assets
					Acquiror_Market_Val_4_Weeks_Prio = Acquiror Market Value 4 weeks prior
					Tgt__Market_Val_4_Wwks_Prior_to_ = Target Market Value 4 weeks prior; 
			* remove one empty observation in the last row;
			IF target_cusip ^= '';
			IF _deal_number ^= 9999999999;
			KEEP _deal_number Acquiror__CUSIP Target_CUSIP __Date_Announced __Date_Effective tender_offer target_public_status
				Acquiror_Primary___SIC___Code Target_Primary___SIC__Code
				status Value__Cash___mil_ __of_Cash __of_Other __of_Stock __of_Unknown
				Target_Total_Assets__One__Year_P Target_Total_Assets___mil_ Acquiror__Total__Assets____mil_ Acquiror_Market_Val_4_Weeks_Prio Tgt__Market_Val_4_Wwks_Prior_to_; 
		RUN;
	%END;
	DATA ma_complete1;
		SET ma_complete1_2006-ma_complete1_2016;
	RUN;
%MEND read_in_SDC_complete1;
%read_in_SDC_complete1;

%MACRO read_in_SDC_complete2;
	%DO year = 2006 %TO 2016;
		PROC IMPORT DATAFILE = "D:\Data\complete2_&year..xls" 
			OUT = ma_complete2_&year
			DBMS = EXCELCS
			REPLACE; 
			SHEET = "Request 2";
		RUN;

		* clean up variables;
		DATA ma_complete2_&year;
			SET ma_complete2_&year;
			deal_size = Value_of_Transaction___mil_ * 1;

			RENAME 	_deal_number = deal_number
					____sought = pct_sought
					Price__Per_Share = bid_offered
					Initial__Pr_Sh = initial_bid_offered
					Offer_Price_to_Target_Stock_Pric = premium_1day
					Offer_Price_to_Target_Stock_Pri0 = premium_1week
					Offer_Price_to_Target_Stock_Pri1 = premium_4week;
			LABEL 	Value_of_Transaction___mil_ = Deal Size
					____sought = PCT sought
					Price__Per_Share = bid_offered
					Initial__Pr_Sh = initial_bid_offered
					Offer_Price_to_Target_Stock_Pric = premium_1day
					Offer_Price_to_Target_Stock_Pri0 = premium_1week
					Offer_Price_to_Target_Stock_Pri1 = premium_4week; 
			* remove one empty observation in the last row;
			IF target_cusip ^= '';
			IF _deal_number ^= 9999999999;
			KEEP _deal_number deal_size ____sought Price__Per_Share Initial__Pr_Sh Offer_Price_to_Target_Stock_Pric
				Offer_Price_to_Target_Stock_Pri0 Offer_Price_to_Target_Stock_Pri1; 
		RUN;
		
	%END;
	DATA ma_complete2;
		SET ma_complete2_2006-ma_complete2_2016;
	RUN;
%MEND read_in_SDC_complete2;
%read_in_SDC_complete2;

* Process SDC Data;
PROC SQL;
	CREATE TABLE ma_dealsize AS
	SELECT	*
	FROM	ma_complete1 a, ma_complete2 b
	WHERE	a.deal_number = b.deal_number;
QUIT;

* Download name file from CRSP;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME crspm '/wrds/crsp/sasdata/a_stock'; 
DATA namefile;
	SET crspm.dsenames (KEEP = permno ncusip namedt nameendt cusip);
	ncusip6 = SUBSTR(ncusip, 1, 6);
RUN;
PROC DOWNLOAD DATA = namefile OUT = namefile; RUN;
ENDRSUBMIT;

* Match SDC acquiror with CRSP using historical CUSIP;
PROC SQL;
	CREATE TABLE ma_dealsize AS
	SELECT		a.*, b.permno AS permno_acquiror,
				(anndate >= namedt AND anndate <= nameendt) AS merged,
				ABS(anndate - namedt) AS distance
	FROM		ma_dealsize a, namefile b
	WHERE		a.acquiror_cusip = b.ncusip6 
	ORDER BY	deal_number, merged DESC, distance;
QUIT;

DATA ma_dealsize;
	SET ma_dealsize;
	BY deal_number DESCENDING merged distance; 
	IF FIRST.deal_number;
	DROP merged distance;
RUN;

PROC SQL;
	CREATE TABLE ma_dealsize AS
	SELECT		a.*, gvkey AS gvkey_acquiror, 
				(a.anndate >= linkdt AND (a.anndate <= linkenddt OR linkenddt = .E)) AS merged1,
				(a.effdate >= linkdt AND (a.effdate <= linkenddt OR linkenddt = .E)) AS merged2
	FROM		ma_dealsize a, compustat_crsp_link b
	WHERE		a.permno_acquiror = b.lpermno
	ORDER BY	deal_number, merged1 DESC, merged2 DESC, gvkey_acquiror;
QUIT;

DATA ma_dealsize;
	SET ma_dealsize;
	BY deal_number DESCENDING merged1 DESCENDING merged2 gvkey_acquiror;
	IF FIRST.deal_number;
	DROP merged1 merged2;
RUN;

* Match SDC target with CRSP using historical CUSIP;
PROC SQL;
	CREATE TABLE ma_dealsize AS
	SELECT		a.*, b.permno AS permno_target,
				(anndate >= namedt AND anndate <= nameendt) AS merged,
				ABS(anndate - namedt) AS distance
	FROM		ma_dealsize a LEFT JOIN namefile b
	ON			a.target_cusip = b.ncusip6 
	ORDER BY	deal_number, merged DESC, distance;
QUIT;

DATA ma_dealsize;
	SET ma_dealsize;
	BY deal_number DESCENDING merged distance; 
	IF FIRST.deal_number;
	DROP merged distance;
RUN;

PROC SQL;
	CREATE TABLE ma_dealsize AS
	SELECT		a.*, gvkey AS gvkey_target,
				(a.anndate >= linkdt AND (a.anndate <= linkenddt OR linkenddt = .E)) AS merged1,
				(a.effdate >= linkdt AND (a.effdate <= linkenddt OR linkenddt = .E)) AS merged2
	FROM		ma_dealsize a LEFT JOIN compustat_crsp_link b
	ON			a.permno_target = b.lpermno
	ORDER BY	deal_number, merged1 DESC, merged2 DESC, gvkey_target;
QUIT;

DATA ma_dealsize;
	SET ma_dealsize;
	BY deal_number DESCENDING merged1 DESCENDING merged2 gvkey_target;
	IF FIRST.deal_number;
	DROP merged1 merged2;
RUN;

PROC SQL;
	CREATE TABLE maliquidity AS
	SELECT		gvkey_acquiror AS gvkey, YEAR(anndate) AS year, SUM(deal_size) AS sum_mavalue
	FROM 		ma_dealsize
	WHERE		deal_size ^= .
	GROUP BY	gvkey, year;
QUIT;

* merge with SIC code;
PROC SQL;
	CREATE TABLE maliquidity AS
	SELECT		a.*,
				COALESCE(INT(b.sich / 100), INT(c.sic_num / 100), 0) AS sic2, 
				COALESCE(INT(b.sich / 10), INT(c.sic_num / 10), 0) AS sic3
	FROM		maliquidity (KEEP = gvkey year sum_mavalue) a 
				LEFT JOIN sich b
	ON			a.gvkey = b.gvkey AND (a.year = b.fyear OR a.year = 2016 AND b.fyear = 2015)
				LEFT JOIN sic c
	ON			a.gvkey = c.gvkey;
QUIT;

* calculate MA liquidity, industry total MA divided by industry total assets;
PROC SQL;
	CREATE TABLE ind_at AS
	SELECT		YEAR(datadate) AS year, sic3, AVG(atq) AS at
	FROM		controlq (KEEP = sic3 gvkey datadate atq)
	GROUP BY	gvkey, YEAR(datadate)
	HAVING		datadate = MAX(datadate);
QUIT;

PROC SQL;
	CREATE TABLE maliquidity_sic3 AS
	SELECT		a.sic3, a.year, SUM(sum_mavalue) / sum_at AS maliquidity_sic3
	FROM		maliquidity a, (SELECT sic3, year, SUM(at) AS sum_at FROM ind_at) b
	WHERE		a.sic3 = b.sic3 AND a.year = b.year
	GROUP BY	a.sic3, a.year;
QUIT;

* Merge controls from Compustat;
PROC SQL;
	CREATE TABLE compustatcontrol AS
	SELECT	a.*, 
			COALESCE(f.maliquidity_sic3, 0) AS maliquidity_sic3
	FROM	controlq a 
			LEFT JOIN maliquidity_sic3 f
	ON		a.sic3 = f.sic3 AND YEAR(a.datadate) = f.year;
QUIT;

* goodwill writedown;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME compna '/wrds/comp/sasdata/naa'; 

PROC SQL UNDO_POLICY = NONE;
	CREATE TABLE goodwillq AS
	SELECT 		gvkey, datadate, atq, gdwlq, ABS(gdwlipq) AS gdwlipq
	FROM 		compna.fundq 
	WHERE		indfmt = 'INDL' AND datafmt = 'STD' AND popsrc = 'D' AND consol = 'C' 
				AND fyearq >= 2003
	ORDER BY 	gvkey, datadate, atq DESC, gdwlq DESC, gdwlipq DESC;
QUIT;

* for duplicate gvkey-datadate, use the one with goodwill impairment;
DATA goodwillq;
	SET goodwillq;
	BY gvkey datadate;
	IF FIRST.datadate;
RUN;

* obtain lagged atq and gdwlq for scalers;
PROC SQL;
	CREATE TABLE goodwillq AS
	SELECT		a.*, b.gdwlq AS gdwlq_1, b.atq AS atq_1
	FROM		goodwillq a LEFT JOIN goodwillq b
	ON			a.gvkey = b.gvkey AND a.datadate > b.datadate
	GROUP BY	a.gvkey, a.datadate, a.atq, a.gdwlq, a.gdwlipq
	HAVING		b.datadate = MAX(b.datadate);
QUIT;

%MACRO goodwill;
	* calculate cumulative goodwill impairment in the next two/three/four years;
	%DO j = 2 %TO 4;
		PROC SQL;
			CREATE TABLE goodwillq_&j.y1 AS
			SELECT		a.gvkey, a.datadate, 
						SUM(b.gdwlipq) AS gdwlipq_&j.y1
			FROM		goodwillq a LEFT JOIN goodwillq b
			ON			a.gvkey = b.gvkey AND b.datadate BETWEEN a.datadate + 1 AND a.datadate + &j * 365
			GROUP BY	a.gvkey, a.datadate, a.gdwlq;
		QUIT;
	%END;

	* merge all post window goodwill impairment together;
	PROC SQL;
		CREATE TABLE goodwillq2 AS
		SELECT	*
		FROM	goodwillq a
					%DO j = 2 %TO 4;
						, goodwillq_&j.y1 b&j.1
					%END;
		WHERE	a.gvkey = a.gvkey 
					%DO j = 2 %TO 4;
						AND a.gvkey = b&j.1.gvkey AND a.datadate = b&j.1.datadate
					%END;;
	QUIT;
%MEND goodwill;
%goodwill;
PROC DOWNLOAD DATA = goodwillq2 OUT = goodwillq; RUN;
ENDRSUBMIT;

* Collect age tenure data;
* Process firm-maanger index;
* the complete list of CEOs need age and tenure information;
PROC SQL;
	CREATE TABLE firm_manager_index1 AS
	SELECT		a.executive_id2, a.company_id, b.gvkey, ticker, company, executive, title
	FROM		(SELECT UNIQUE executive_id2, company_id, fy_end, ticker, company, executive, title FROM v.ednd) a, 
				(SELECT UNIQUE company_id, gvkey FROM v.cfcomp) b,
				(SELECT UNIQUE executive_id2, company_id, fy_end FROM newlyvestedshares) c,
				(SELECT UNIQUE executive_id2 FROM qall) d,
				(SELECT UNIQUE executive_id2 FROM v.mm) e
	WHERE		a.company_id = b.company_id	
				AND a.company_id = c.company_id AND a.fy_end = c.fy_end AND a.executive_id2 = c.executive_id2
				AND a.executive_id2 = d.executive_id2 AND a.executive_id2 = e.executive_id2
	GROUP BY	a.executive_id2, a.company_id
	HAVING		a.fy_end = MAX(a.fy_end);
QUIT;

* Organize ceo age tenure data;
* v.agetenure_checked from hand collected CEO age and tenure;
PROC SQL;
	CREATE TABLE agetenure_checked AS
	SELECT		*
	FROM		v.agetenure_checked
	GROUP BY	company_id, executive_id2
	HAVING		fy_end = MIN(fy_end);
QUIT;

PROC SQL;
	CREATE TABLE firm_manager_index2 AS
	SELECT	a.*, becameceo FORMAT MMDDYY10., 
			ceoage, fy_end AS fy_end_ceoage FORMAT MMDDYY10.,
			(becameceo ^= .) AS from_vivian
	FROM	firm_manager_index1 a LEFT JOIN agetenure_checked b
	ON		a.company_id = b.company_id AND a.executive_id2 = b.executive_id2;
QUIT;

%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME execcomp '/wrds/comp/sasdata/execcomp'; 

PROC DOWNLOAD DATA = execcomp.coperol OUT = coperol; RUN;
PROC DOWNLOAD DATA = execcomp.person OUT = person; RUN;
ENDRSUBMIT;

* merge with execucomp;
PROC SQL;
	CREATE TABLE execcompceos AS
	SELECT	*
	FROM	coperol a, 
			person (KEEP = exec_fullname gender nameprefix execid page) b
	WHERE	a.execid = b.execid;
QUIT;

PROC SQL;
	CREATE TABLE firm_manager_index3 AS
	SELECT		a.*,
				b.becameceo AS becameceo_execcomp, b.title AS title_execcomp, b.*
	FROM		firm_manager_index2 a LEFT JOIN execcompceos b
	ON			a.gvkey = b.gvkey AND 
				(COUNT(exec_fullname, SCAN(executive, 1, ','), 'I') OR COUNT(exec_fullname, SCAN(executive, 2, ','), 'I'))
	ORDER BY	executive, company;
QUIT;

* output file for RA;
DATA firm_manager_index3_matched2 
	firm_manager_index3_notmatched2 (KEEP = executive_id2 company_id gvkey ticker company executive title);
	SET firm_manager_index3;
	IF becameceo = . AND (becameceo_execcomp = '' AND page = .) THEN 
		OUTPUT firm_manager_index3_notmatched2;
	ELSE OUTPUT firm_manager_index3_matched2;
RUN;
* this is the file for CEOs not matched to ExecuComp;
PROC SORT DATA = firm_manager_index3_notmatched2 NODUPKEY;
	BY executive_id2 company_id;
RUN;

* this is the file for CEOs matched to ExecuComp, could be one to one or one to more matching, need to be inspected;
DATA firm_manager_index3_execucomp;
	SET firm_manager_index3;
	IF becameceo = . AND (becameceo_execcomp ^= '' OR page ^= .);
	DROP becameceo ceoage fy_end_ceoage from_vivian;
RUN;

PROC EXPORT DATA = firm_manager_index3_notmatched2
	OUTFILE = "D:\Data\ceo_unmatched.xls" REPLACE
	DBMS = EXCELCS; 
	SHEET = "ceo";
RUN;
PROC EXPORT DATA = firm_manager_index3_execucomp
	OUTFILE = "D:\Data\ceo_execucomp.xls" REPLACE
	DBMS = EXCELCS; 
	SHEET = "ceo";
RUN;

* Read in RA data;
* Read in the execucomp matched data;
PROC IMPORT DATAFILE = "D:\Data\ceo_execucomp_ra.xls" 
	OUT = ceo_execucomp_ra
	DBMS = EXCELCS
	REPLACE; 
	SHEET = "ceo";
RUN;

DATA ceo_execucomp_ra2;
	SET ceo_execucomp_ra;
	KEEP company_id executive_id2 becameceo birth_date;
	IF verified = 1 OR executive_id2 = 114202;
	year = SUBSTR(becameceo_filled, 7, 4) * 1;
	month = SUBSTR(becameceo_filled, 4, 2) * 1;
	day = SUBSTR(becameceo_filled, 1, 2) * 1;
	becameceo = COALESCE(becameceo_execcomp, MDY(month, day, year));
	FORMAT becameceo YYMMDD10.;
	RENAME birth_date = ceobirthyear;
RUN;

* Read in originally unmatched data;
PROC IMPORT DATAFILE = "D:\Data\ceo_unmatched_ra.xls" 
	OUT = ceo_unmatched_ra
	DBMS = EXCELCS
	REPLACE; 
	SHEET = "ceo";
RUN;

DATA ceo_unmatched_ra2;
	SET ceo_unmatched_ra;
	RENAME becameceo_filled = becameceo
			birth_date = ceobirthyear;
	KEEP company_id executive_id2 becameceo_filled birth_date;
RUN;

* Process all CEO age and tenure data;
DATA ceo_data_old_ra;
	SET v.agetenure_checked;
	ceobirthyear = YEAR(fy_end) - ceoage;
	KEEP company_id executive_id2 becameceo ceobirthyear;
RUN;

PROC SORT DATA = ceo_data_old_ra NODUPKEY;
	BY company_id executive_id2;
RUN;

DATA ceo_data_all;
	SET ceo_data_old_ra ceo_execucomp_ra2 (IN = a) ceo_unmatched_ra2 (IN = b);
	IF b THEN source = 'Unmatched';
	ELSE IF a THEN source = 'ExecComp';
	ELSE source = 'Old';
RUN;

PROC SQL;
	CREATE TABLE x AS
	SELECT		*
	FROM		ceo_data_all
	GROUP BY	company_id, executive_id2
	HAVING		COUNT(*) > 1;
QUIT;

PROC SORT DATA = ceo_data_all;
	BY company_id executive_id2 source;
RUN;
DATA ceo_age_tenure;
	SET ceo_data_all;
	BY company_id executive_id2 source;
	IF FIRST.executive_id2;
RUN;

* calculate repurchase, M&A and BHAR variables;

* Retrive data from Compustat quarterly file;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME compna '/wrds/comp/sasdata/naa'; 

PROC SQL UNDO_POLICY = NONE;
	CREATE TABLE comvarq_wrds AS
	SELECT 	gvkey, datadate, fyearq, fqtr, fyr, cik, cusip, conm, datacqtr, datafqtr, apdedateq, fdateq, rdq,
			atq, saleq, saley, 
			cshopq, prcraq, cshiq, sstky, spstkcy, scstkcy,
			cshoq, prccq, cshprq, cshfdq, ceqq,
			chq, cheq, niq, oibdpq, oiadpq, nopiq, oancfy, dpq, sretq, actq, lctq,
			ltq, dlcq, dlttq, dltisy, dltry, fincfy, pstkq, txditcq, dvpq, dvpy, pdvcy, cdvcy, dvy,
			tstknq, tstkq,
			ppegtq, ppentq, capxy, xrdq, req
	FROM 	compna.fundq 
	WHERE	indfmt = 'INDL' AND datafmt = 'STD' AND popsrc = 'D' AND consol = 'C' 
			AND fyearq >= 2003; 
QUIT;	

PROC DOWNLOAD DATA = comvarq_wrds OUT = comvarq; RUN;
ENDRSUBMIT;

* Monthly repurchase; 
* Read in RA collected files and clean them;
%MACRO read_monthly_rp;
	%DO i = 1 %TO 10;
		PROC IMPORT DATAFILE = "D:\Data\Monthly Repurchase Collection &i of 10_revised.xlsx" 
			OUT = monthly_rp_&i._of_10
			DBMS = XLSX
			REPLACE; 
		RUN;
	
		DATA monthly_rp_clean_&i._of_10;
			SET monthly_rp_&i._of_10;
			%IF &i ^= 6 %THEN %DO;
				monthly_repurchase2 = INPUT(monthly_repurchase, 8.);
				DROP monthly_repurchase;
				RENAME monthly_repurchase2 = monthly_repurchase;

				avg_price_paid_per_share2 = INPUT(avg_price_paid_per_share, 8.);
				DROP avg_price_paid_per_share;
				RENAME avg_price_paid_per_share2 = avg_price_paid_per_share;
			%END;
			total_program_purchase2 = INPUT(total_program_purchase, 8.);
			DROP total_program_purchase;
			RENAME total_program_purchase2 = total_program_purchase;
			file_num = &i;
			WHERE cik IS NOT MISSING AND month_number IS NOT MISSING;
		RUN;
	%END;

	* aggregate them, only keep relevant variables;
	* define repurchase month variable;
	DATA monthly_rp_collected;
		SET %DO i = 1 %TO 10; monthly_rp_clean_&i._of_10 %END;;
		repurchase_monthend = INTNX('MONTH', compustat_date, month_number - 3, 'E');
		FORMAT repurchase_monthend YYMMDD10.;
		repurchase_month = YEAR(repurchase_monthend) * 100 + MONTH(repurchase_monthend);
		KEEP cik compustat_date qtr_repurchase monthly_repurchase avg_price_paid_per_share repurchase_monthend repurchase_month file_num;
	RUN;
%MEND;
%read_monthly_rp;

PROC SORT DATA = monthly_rp_collected NODUPKEY;
	BY cik compustat_date repurchase_monthend;
RUN;

* there are three observations with duplicate cik-repurchase_monthend, due to change in fiscal year end, pick the later one;
PROC SQL;
	CREATE TABLE monthly_rp_collected AS
	SELECT		*
	FROM		monthly_rp_collected
	GROUP BY	cik, repurchase_monthend
	HAVING		compustat_date = MAX(compustat_date);
QUIT;

* Match with GVKEY;
PROC SQL;
	CREATE TABLE cik_gvkey AS
	SELECT UNIQUE gvkey, cik, datadate
	FROM 	comvarq 
	WHERE 	cshopq > 0 AND cik ^= '';
QUIT;
PROC SORT DATA = cik_gvkey NODUPKEY;
	BY cik datadate;
RUN;

PROC SQL;
	CREATE TABLE monthly_rp_collected AS
	SELECT		a.cik, gvkey, datadate, repurchase_monthend, repurchase_month, 
				monthly_repurchase, avg_price_paid_per_share, file_num
	FROM		monthly_rp_collected a, cik_gvkey b
	WHERE		a.cik = b.cik AND a.compustat_date = b.datadate
	GROUP BY	gvkey, repurchase_monthend
	HAVING		COUNT(*) = 1;;
QUIT;

* Remove duplicate firm-datadate;
* Delete firm-quarters with missing AT;
DATA comvarq;
	SET comvarq;
	mvq = cshoq * prccq;
	IF atq ^= . AND mvq ^= .;
RUN;
* Keep the ones with largest CSHOPQ and SALEQ;
PROC SORT DATA = comvarq;
	BY gvkey datadate DESCENDING cshopq DESCENDING saleq;
RUN;
DATA comvarq;
	SET comvarq;
	BY gvkey datadate DESCENDING cshopq DESCENDING saleq;
	IF FIRST.datadate;
RUN;

* Repurchase variable;
* define repurchase variables using cshopq and prcraq;
PROC SQL;
	CREATE TABLE repurchase_comp AS
	SELECT	a.*, 
			b.cshoq AS lagcshoq, b.prccq AS lagprccq, b.datadate AS lagdatadate
	FROM	comvarq (KEEP = gvkey fyearq fqtr datadate cshopq prcraq cshoq prccq) a,
			comvarq (KEEP = gvkey datadate cshoq prccq) b
	WHERE	a.gvkey = b.gvkey AND INTNX('MONTH', a.datadate, -3, 'END') = b.datadate;
QUIT;

* Populate Qtr Repurchase to Monthly;
DATA repurchase_m;
	SET repurchase_comp (IN = month1) repurchase_comp (IN = month2) repurchase_comp (IN = month3);
	IF month1 THEN DO;
		fmonth = 1;
		monthend = INTNX('MONTH', datadate, -2, 'END');
	END;
	IF month2 THEN DO;
		fmonth = 2;
		monthend = INTNX('MONTH', datadate, -1, 'END');
	END;
	IF month3 THEN DO;
		fmonth = 3;
		monthend = INTNX('MONTH', datadate, 0, 'END');
	END;
	FORMAT monthend YYMMDD10.;
RUN;

PROC SORT DATA = repurchase_m NODUPKEY;
	BY gvkey monthend;
RUN;

* separate into two categories, the ones with SeekEdgar data and the ones without SeekEdgar data;
* only those from match cf_comp with positive compustat cshopq (positive repurchase) with SeekEdgar;
* and then go through manual collection can possibly have monthly repurchase data;
PROC SQL;
	CREATE TABLE repurchase_m2 AS
	SELECT		a.*, b.monthly_repurchase, b.avg_price_paid_per_share, cik, file_num
	FROM		repurchase_m a 
				LEFT JOIN monthly_rp_collected b
	ON			a.gvkey = b.gvkey AND a.monthend = b.repurchase_monthend
	ORDER BY	gvkey, repurchase_monthend;
QUIT;

* Merge with monthly price;
* clean up duplicate months in CRSP (obtained from WRDS web);
* download monthly price;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME crspm '/wrds/crsp/sasdata/a_stock'; 
DATA msf;
	SET crspm.msf;
	WHERE DATE >= MDY(1, 1, 2000);
	KEEP permno date prc cfacshr shrout;
RUN;

PROC DOWNLOAD DATA = msf OUT = crsp_monthly; RUN;

ENDRSUBMIT;

PROC SORT DATA = crsp_monthly NODUPKEY;
	BY permno date;
RUN;

PROC SQL;
	CREATE TABLE repurchase_m2 AS
	SELECT	a.*, c.*
	FROM	repurchase_m2 a LEFT JOIN compustat_crsp_link b
	ON		a.gvkey = b.gvkey  
			AND a.monthend >= b.linkdt AND (a.monthend <= b.linkenddt OR b.linkenddt = .E)
			LEFT JOIN crsp_monthly c
	ON		b.lpermno = c.permno AND INTNX('MONTH', c.date, 1, 'E') = a.monthend
	ORDER BY gvkey, monthend;
QUIT;

* Define monthly repurchase variables;
DATA v.repurchase_m;
	SET repurchase_m2;
	* did not manually collect information of these firms;
	IF cshopq <= 0 THEN DO;
		rpm = 0;
		rpvalpctm = 0;
	END;
	* scale variables by 1000000 because of scale differences in Compustat and CRSP;
	ELSE DO;
		rpm = (monthly_repurchase > 0 OR avg_price_paid_per_share > 0 AND monthly_repurchase = 0);
		* use the # of share and price from Compustat if missing shrout;
		rpvalpctm = (COALESCE(monthly_repurchase * avg_price_paid_per_share, 0) / 1000000) / COALESCE(shrout * prc / 1000, lagcshoq * lagprccq);
	END;
	KEEP gvkey datadate fyearq fqtr fmonth monthend monthly_repurchase avg_price_paid_per_share
		rpm rpvalpctm;
RUN;

* import sdc repurchase announcement;
* read in SDC;
PROC IMPORT DATAFILE = "D:\Data\repurchase data.xls" 
	OUT = rpann
	DBMS = EXCELCS
	REPLACE; 
	SHEET = "Request 1";
RUN;
PROC FREQ DATA = rpann;
	TABLES Purpose_of_Repurchase_Program;
RUN;

DATA rpann2;
	SET rpann;
	IF cusip ^= '' AND Initial_Auth__Date1 ^= .;
	RENAME Initial_Auth__Date1 = anndate
			__Digit_CUSIP = cusip8;
	KEEP Initial_Auth__Date1 __Digit_CUSIP cusip;
RUN;

* Match RP Announcement with CRSP using historical CUSIP;
PROC SQL;
	CREATE TABLE rpann3 AS
	SELECT		anndate, a.cusip, b.*,
				(anndate >= namedt AND anndate <= nameendt) AS merged,
				ABS(anndate - namedt) AS distance
	FROM		(SELECT UNIQUE cusip, anndate FROM rpann2) a, 
				namefile b
	WHERE		a.cusip = b.ncusip6 
	ORDER BY	anndate, cusip, merged DESC, distance;
QUIT;

DATA rpann3;
	SET rpann3;
	BY anndate cusip DESCENDING merged distance; 
	IF FIRST.cusip;
RUN;

PROC SQL;
	CREATE TABLE rpann3 AS
	SELECT	anndate, cusip, permno, gvkey
	FROM	rpann3 a, compustat_crsp_link b
	WHERE	a.permno = b.lpermno;
QUIT;

PROC SORT DATA = rpann3 NODUPKEY OUT = v.rpann;
	BY gvkey anndate;
RUN;

* read in SDC M&A Data;
* Import SDC data;
%MACRO read_in_SDC;
	%DO year = 2006 %TO 2016;
		* read in SDC;
		PROC IMPORT DATAFILE = "D:\Data\ma_&year..xls" 
			OUT = ma&year
			DBMS = EXCELCS
			REPLACE; 
			SHEET = "Request 2";
		RUN;

		* clean up variables;
		DATA ma&year;
			SET ma&year;
			deal_size = Value_of_Transaction___mil_ * 1;
			pct_unknown = TRANWRD(__of_Unknown, ',', '.') * 1;
			RENAME 	Acquiror_Full_Name = acquiror_name
					Acquiror__CUSIP = acquiror_cusip
					__Date_Announced = anndate
					__Date_Effective = effdate
					__of_Cash = pct_cash
					__of_Other = pct_other
					__of_Stock = pct_stock;
			LABEL 	Acquiror_Full_Name = Acquiror_Name
					Acquiror__CUSIP = Acquiror_CUSIP
					__Date_Announced = anndate
					__Date_Effective = effdate
					Target_CUSIP = Target_CUSIP
					Target_Name = Target_Name
					deal_size = Value of Transaction ($mil); 
			* remove one empty observation in the last row;
			IF target_cusip ^= '';
			KEEP Acquiror_Full_Name Target_Name Acquiror__CUSIP Target_CUSIP __Date_Announced __Date_Effective 
				deal_size __of_Cash __of_Other __of_Stock pct_unknown status; 
		RUN;
	%END;
%MEND read_in_SDC;
%read_in_SDC;

* combine all years of SDC;
DATA ma;
	SET ma2006-ma2016;
	IF SUM(pct_other, pct_stock, pct_unknown) >= 99.96 AND SUM(pct_other, pct_stock, pct_unknown) <= 100.05 
		AND pct_cash = . THEN pct_cash = 0;
	IF pct_cash ^= . THEN DO;
		allcash = (pct_cash >= 100);
	END;
	number = _n_;
RUN;

* Download name file from CRSP;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME crspm '/wrds/crsp/sasdata/a_stock'; 
DATA namefile;
	SET crspm.dsenames (KEEP = permno ncusip namedt nameendt cusip);
	ncusip6 = SUBSTR(ncusip, 1, 6);
RUN;
PROC DOWNLOAD DATA = namefile OUT = namefile; RUN;
ENDRSUBMIT;

* Match SDC acquiror with CRSP using historical CUSIP;
PROC SQL;
	CREATE TABLE ma_acquiror AS
	SELECT		a.*, b.*,
				(anndate >= namedt AND anndate <= nameendt) AS merged,
				ABS(anndate - namedt) AS distance
	FROM		(SELECT anndate, effdate, acquiror_cusip, deal_size, status,
				allcash, number FROM ma) a, 
				namefile b
	WHERE		a.acquiror_cusip = b.ncusip6 
	ORDER BY	number, merged DESC, distance;
QUIT;

DATA ma_acquiror;
	SET ma_acquiror;
	BY number DESCENDING merged distance; 
	IF FIRST.number;
RUN;

PROC SQL;
	CREATE TABLE ma_acquiror AS
	SELECT		number, anndate, effdate, acquiror_cusip, deal_size, status,
				allcash, permno, gvkey, 
				(a.anndate >= linkdt AND (a.anndate <= linkenddt OR linkenddt = .E)) AS merged1,
				(a.effdate >= linkdt AND (a.effdate <= linkenddt OR linkenddt = .E)) AS merged2
	FROM		ma_acquiror a, compustat_crsp_link b
	WHERE		a.permno = b.lpermno
	ORDER BY	number, merged1 DESC, merged2 DESC, gvkey;
QUIT;

DATA v.ma_acquiror;
	SET ma_acquiror;
	BY number DESCENDING merged1 DESCENDING merged2 gvkey;
	IF FIRST.number;
	DROP merged1 merged2;
RUN;

* Match SDC target with CRSP using historical CUSIP;
PROC SQL;
	CREATE TABLE ma_target AS
	SELECT		number, anndate, effdate, target_cusip, status, b.*,
				(anndate >= namedt AND anndate <= nameendt) AS merged,
				ABS(anndate - namedt) AS distance
	FROM		(SELECT UNIQUE number, anndate, effdate, target_cusip, status FROM ma) a, 
				namefile b
	WHERE		a.target_cusip = b.ncusip6 
	ORDER BY	number, anndate, effdate, target_cusip, status, merged DESC, distance;
QUIT;

DATA ma_target;
	SET ma_target;
	BY number DESCENDING merged distance; 
	IF FIRST.number;
RUN;
PROC SQL;
	CREATE TABLE ma_target AS
	SELECT		number, anndate, effdate, target_cusip, status, permno, gvkey,
				(a.anndate >= linkdt AND (a.anndate <= linkenddt OR linkenddt = .E)) AS merged1,
				(a.effdate >= linkdt AND (a.effdate <= linkenddt OR linkenddt = .E)) AS merged2
	FROM		ma_target a, compustat_crsp_link b
	WHERE		a.permno = b.lpermno
	ORDER BY	number, merged1 DESC, merged2 DESC, gvkey;
QUIT;

DATA ma_target;
	SET ma_target;
	BY number DESCENDING merged1 DESCENDING merged2 gvkey;
	IF FIRST.number;
	DROP merged1 merged2;
RUN;

* Match SDC acquiror and target, and target financials;
PROC SQL;
	CREATE TABLE ma_acquiror_target AS
	SELECT	a.*, b.gvkey AS gvkey_target
	FROM	ma_acquiror a LEFT JOIN ma_target b
	ON		a.number = b.number;
QUIT;

* list of MA permno-dates for calculating CAR;
PROC SORT DATA = ma_acquiror_target (KEEP = number gvkey_target anndate)
		OUT = ma_target_gvkey NODUPKEY;
	BY gvkey_target anndate;
	WHERE gvkey_target IS NOT MISSING;
RUN;

%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME compna '/wrds/comp/sasdata/naa'; 

PROC UPLOAD DATA = ma_target_gvkey OUT = ma_target_gvkey; RUN;

PROC SQL;
	CREATE TABLE target_financials AS
	SELECT 		gvkey_target, anndate, datadate, fyear,  		
				at AS at_target
	FROM 		ma_target_gvkey a, compna.funda b
	WHERE		a.gvkey_target = b.gvkey AND a.anndate BETWEEN b.datadate + 1 AND b.datadate + 1000
				AND indfmt = 'INDL' AND datafmt = 'STD' AND popsrc = 'D' AND consol = 'C'
	GROUP BY	gvkey_target, anndate
	HAVING		b.datadate = MAX(b.datadate);
QUIT;

PROC SQL;
	CREATE TABLE target_financials AS
	SELECT 		a.*, 
				intanq AS intanq_target, atq AS atq_target, intanq / atq AS intanq_atq_target,
				SUM(cshoq * prccq, dlttq, dlcq) / atq AS mkbk_target,
				saleq AS saleq_target, saley AS saley_target, 	
				cshopq AS cshopq_target, prcraq AS prcraq_target, cshiq AS cshiq_target, sstky AS sstky_target, spstkcy AS spstkcy_target, scstkcy AS scstkcy_target,
				cshoq AS cshoq_target, prccq AS prccq_target, cshprq AS cshprq_target, cshfdq AS cshfdq_target, ceqq AS ceqq_target,
				chq AS chq_target, cheq AS cheq_target, niq AS niq_target, oibdpq AS oibdpq_target, oiadpq AS oiadpq_target, nopiq AS nopiq_target, 
				oancfy AS oancfy_target, dpq AS dpq_target, sretq AS sretq_target, actq AS actq_target, lctq AS lctq_target,
				ltq AS ltq_target, dlcq AS dlcq_target, dlttq AS dlttq_target, dltisy AS dltisy_target, dltry AS dltry_target, fincfy AS fincfy_target, 
				pstkq AS pstkq_target, txditcq AS txditcq_target, dvpq AS dvpq_target, dvpy AS dvpy_target, pdvcy AS pdvcy_target, cdvcy AS cdvcy_target, dvy AS dvy_target,
				tstknq AS tstknq_target, tstkq AS tstkq_target,
				ppegtq AS ppegtq_target, ppentq AS ppentq_target, capxy AS capxy_target, xrdq AS xrdq_target, req AS req_target
	FROM 		target_financials a, compna.fundq b
	WHERE		a.gvkey_target = b.gvkey AND a.anndate BETWEEN b.datadate + 1 AND b.datadate + 1000
				AND indfmt = 'INDL' AND datafmt = 'STD' AND popsrc = 'D' AND consol = 'C'
	GROUP BY	gvkey_target, anndate
	HAVING		b.datadate = MAX(b.datadate)
	ORDER BY	gvkey_target, anndate, at_target DESC, intanq_target DESC, atq_target DESC, saleq_target DESC;
QUIT;

DATA target_financials;
	SET target_financials;
	BY gvkey_target anndate;
	IF FIRST.anndate;
RUN;

PROC DOWNLOAD DATA = target_financials OUT = target_financials; RUN;
ENDRSUBMIT;

* merge with SIC code;
PROC SQL;
	CREATE TABLE v.ma_acquiror_target_financial AS
	SELECT	a.*, atq_target, mkbk_target,
			LOG(saleq_target) AS logsaleq_target, 
			cshoq_target * prccq_target AS mvq_target,
			SUM(dlttq_target, 0) / atq_target AS debt_target, oibdpq_target / atq_target AS operatingroaq_target,
			nopiq_target / atq_target AS noperatingroaq_target,
			COALESCE(INT(c.sich / 100), INT(d.sic_num / 100), 0) AS sic2, 
			COALESCE(INT(c.sich / 10), INT(d.sic_num / 10), 0) AS sic3
	FROM	ma_acquiror_target a LEFT JOIN target_financials b
	ON		a.gvkey_target = b.gvkey_target AND a.anndate = b.anndate
			LEFT JOIN sich c
	ON		a.gvkey = c.gvkey AND (YEAR(a.anndate) = c.fyear OR YEAR(a.anndate) = 2016 AND c.fyear = 2015)
			LEFT JOIN sic d
	ON		a.gvkey = d.gvkey;
QUIT;

* calculate BHAR;
* calculate repurchase market-adjusted returns;
* Output index of firm-period that needs to calculate BHAR;
* link gvkey-period list with PERMNO;
PROC SQL;
	CREATE TABLE gvkeyindexpermno_bharmkt_m AS
	SELECT		UNIQUE a.gvkey, b.lpermno, monthend, 
				INTNX('MONTH', monthend, -3, 'END') AS lag3monthend FORMAT MMDDYY10., 
				INTNX('MONTH', monthend, -2, 'END') AS lag2monthend FORMAT MMDDYY10., 
				INTNX('MONTH', monthend, -1, 'END') AS lag1monthend FORMAT MMDDYY10.,
				INTNX('MONTH', monthend, 12, 'END') AS fwd12monthend FORMAT MMDDYY10.,
				INTNX('MONTH', monthend, 24, 'END') AS fwd24monthend FORMAT MMDDYY10.,
				INTNX('MONTH', monthend, 36, 'END') AS fwd36monthend FORMAT MMDDYY10.,
				INTNX('MONTH', monthend, 48, 'END') AS fwd48monthend FORMAT MMDDYY10.
	FROM		v.repurchase_m a, compustat_crsp_link b
	WHERE		a.gvkey = b.gvkey
	ORDER BY	gvkey, monthend, lpermno;
QUIT;

PROC SORT DATA = gvkeyindexpermno_bharmkt_m NODUPKEY; BY gvkey monthend lpermno; RUN; 

* Calculate market-adjusted returns;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME crspm '/wrds/crsp/sasdata/a_stock'; 

PROC UPLOAD DATA = gvkeyindexpermno_bharmkt_m OUT = gvkeyindexpermno_bharmkt_m; RUN;

%MACRO BHARmkt (v, date1, date2);
	%IF &v = n3_n2 OR &v = n2_n1 OR &v = n1_0 %THEN %LET month = 1;
	%IF &v = 0_12 OR &v = 12_24 OR &v = 24_36 OR &v = 36_48 OR &v = 48_60 %THEN %LET month = 12;

	PROC SQL;
		CREATE TABLE bharmkt_m&v AS
		SELECT		a.*, b.date, 
					prc * vol AS marketcap,
					EXP(SUM(LOG(1 + ret))) - 1 AS rret&v,
					EXP(SUM(LOG(1 + vwretd))) - 1 AS rvwretd&v,
					SUM(ret - vwretd) AS vwcar&v,
					CALCULATED rret&v - CALCULATED rvwretd&v AS rvwexcessret&v
		FROM		gvkeyindexpermno_bharmkt_m (KEEP = gvkey lpermno monthend &date1 &date2) a,
					crspm.msf (KEEP = date permno ret prc vol WHERE = (ret NOT IN (., .C, .B))) b,
					crspm.msi (KEEP = date vwretd ewretd WHERE = (vwretd ^= . AND ewretd ^= .)) c
		WHERE		a.lpermno = b.permno AND b.date BETWEEN a.&date1 + 1 AND a.&date2
					AND b.date = c.date
		GROUP BY	gvkey, monthend, &date1, &date2, lpermno
		HAVING		date = MAX(date) AND COUNT(ret) = &month
		ORDER BY	gvkey, monthend, &date1, &date2, marketcap DESC;
	QUIT;

	* for gvkeys that are matched to multiple PERMNOs, keep the one with larger market cap;
	DATA bharmkt_m&v;
		SET bharmkt_m&v;
		BY gvkey monthend &date1 &date2 DESCENDING marketcap;
		IF FIRST.&date2;
		KEEP gvkey monthend lpermno rret&v vwcar&v rvwexcessret&v;
	RUN;

	PROC DOWNLOAD DATA = bharmkt_m&v OUT = bharmkt_m&v; RUN;
	PROC DATASETS; DELETE bharmkt_m&v; QUIT;
%MEND BHARmkt;

%BHARmkt(n3_n2, lag3monthend, lag2monthend); 
%BHARmkt(n2_n1, lag2monthend, lag1monthend); 
%BHARmkt(n1_0, lag1monthend, monthend);
%BHARmkt(0_12, monthend, fwd12monthend);
%BHARmkt(12_24, fwd12monthend, fwd24monthend);
%BHARmkt(24_36, fwd24monthend, fwd36monthend);
%BHARmkt(36_48, fwd36monthend, fwd48monthend);
ENDRSUBMIT;

PROC SQL;
	CREATE TABLE bharmkt_m AS
	SELECT	COALESCE(a.gvkey, b.gvkey, c.gvkey) AS gvkey,
			COALESCE(a.monthend, b.monthend, c.monthend) AS monthend FORMAT YYMMDDN8.,
			COALESCE(a.lpermno, b.lpermno, c.lpermno) AS lpermno,
			*
	FROM	bharmkt_m0_12 (KEEP = gvkey monthend lpermno rret0_12 rvwexcessret0_12 vwcar0_12) a 
			FULL JOIN bharmkt_mn2_n1 (KEEP = gvkey monthend lpermno rretn2_n1 rvwexcessretn2_n1 vwcarn2_n1) b
	ON		a.gvkey = b.gvkey AND a.monthend = b.monthend 
			FULL JOIN bharmkt_mn1_0 (KEEP = gvkey monthend lpermno rretn1_0 rvwexcessretn1_0 vwcarn1_0) c
	ON		COALESCE(a.gvkey, b.gvkey) = c.gvkey AND COALESCE(a.monthend, b.monthend) = c.monthend 
			LEFT JOIN bharmkt_mn3_n2 (KEEP = gvkey monthend rretn3_n2 rvwexcessretn3_n2 vwcarn3_n2) d
	ON		COALESCE(a.gvkey, b.gvkey, c.gvkey) = d.gvkey AND COALESCE(a.monthend, b.monthend, c.monthend) = d.monthend 
			LEFT JOIN bharmkt_m12_24 (KEEP = gvkey monthend rret12_24 rvwexcessret12_24 vwcar12_24) j
	ON		COALESCE(a.gvkey, b.gvkey, c.gvkey) = j.gvkey AND COALESCE(a.monthend, b.monthend, c.monthend) = j.monthend
			LEFT JOIN bharmkt_m24_36 (KEEP = gvkey monthend rret24_36 rvwexcessret24_36 vwcar24_36) k
	ON		COALESCE(a.gvkey, b.gvkey, c.gvkey) = k.gvkey AND COALESCE(a.monthend, b.monthend, c.monthend) = k.monthend
			LEFT JOIN bharmkt_m36_48 (KEEP = gvkey monthend rret36_48 rvwexcessret36_48 vwcar36_48) l
	ON		COALESCE(a.gvkey, b.gvkey, c.gvkey) = l.gvkey AND COALESCE(a.monthend, b.monthend, c.monthend) = l.monthend;
QUIT;

* calculate repurchase industry-adjusted return;
* Output index of firm-period that needs to calculate BHAR;
* link gvkey-period list with PERMNO;
PROC SQL;
	CREATE TABLE gvkeyindexpermno_bharff_m AS
	SELECT		UNIQUE a.gvkey, b.lpermno, d.ind AS ff49, monthend,
				INTNX('MONTH', monthend, -3, 'END') AS lag3monthend FORMAT MMDDYY10., 
				INTNX('MONTH', monthend, -2, 'END') AS lag2monthend FORMAT MMDDYY10., 
				INTNX('MONTH', monthend, -1, 'END') AS lag1monthend FORMAT MMDDYY10., 
				INTNX('MONTH', monthend, 12, 'END') AS fwd12monthend FORMAT MMDDYY10.,
				INTNX('MONTH', monthend, 24, 'END') AS fwd24monthend FORMAT MMDDYY10.,
				INTNX('MONTH', monthend, 36, 'END') AS fwd36monthend FORMAT MMDDYY10.,
				INTNX('MONTH', monthend, 48, 'END') AS fwd48monthend FORMAT MMDDYY10.
	FROM		v.repurchase_m a, compustat_crsp_link b, 
				sich c, v.ff49 d
	WHERE		a.gvkey = b.gvkey AND a.gvkey = c.gvkey AND a.fyearq = c.fyear AND
				c.sich BETWEEN INPUT(d.ind1, 4.) AND INPUT(d.ind2, 4.)
	ORDER BY	gvkey, monthend, lpermno;
QUIT;

PROC SORT DATA = gvkeyindexpermno_bharff_m NODUPKEY; BY gvkey monthend lpermno; RUN; 

* Import FF industry returns from French Website;
* import value-weighted returns;
PROC IMPORT DATAFILE = "D:\Data\49_Industry_Portfolios.xls" 
	OUT = ff_vw_monthly 
	DBMS = EXCELCS
	REPLACE; 
	SHEET = "vw";
RUN;

* transpose from wide format to long format;
PROC TRANSPOSE DATA = ff_vw_monthly OUT = ff_vw_monthly;
	BY date;
RUN;

DATA ff_vw_monthly;
	SET ff_vw_monthly;
	IF col1 = -99.99 THEN vwretd = .;
	ELSE vwretd = col1 / 100;
	KEEP date _name_ vwretd;
RUN;

* import equal-weighted returns;
PROC IMPORT DATAFILE = "D:\Data\49_Industry_Portfolios.xls" 
	OUT = ff_ew_monthly 
	DBMS = EXCELCS
	REPLACE; 
	SHEET = "ew";
RUN;

* transpose from wide format to long format;
PROC TRANSPOSE DATA = ff_ew_monthly OUT = ff_ew_monthly;
	BY date;
RUN;

DATA ff_ew_monthly;
	SET ff_ew_monthly;
	IF col1 = -99.99 THEN ewretd = .;
	ELSE ewretd = col1 / 100;
	KEEP date _name_ ewretd;
RUN;

* merge two industry returns;
PROC SQL;
	CREATE TABLE ffret AS
	SELECT		INPUT(SUBSTR(a._name_, 4, LENGTH(a._name_) - 3), 4.) AS ff49,
				a.date, 
				vwretd, ewretd
	FROM		ff_vw_monthly a, ff_ew_monthly b
	WHERE		a.date = b.date AND a._name_ = b._name_
	ORDER BY	ff49, date;
QUIT;

%MACRO cumulativeffret(n);	
	%IF &n = 1 %THEN %DO;
		DATA ffret&n;
			SET ffret;
			cvwretd = vwretd;
			sum_vwretd = vwretd;
			cewretd = ewretd;
			sum_ewretd = ewretd;
			KEEP ff49 date cvwretd cewretd sum_vwretd sum_ewretd;
		RUN;
	%END;
	%ELSE %DO;
		DATA ffret&n;
			SET ffret;
			%DO i = 1 %TO &n - 1;
				vwretd_&i = LAG&i(vwretd);
				ewretd_&i = LAG&i(ewretd);
				IF LAG&i(ff49) ^= ff49 THEN DO;
					vwretd_&i = .; ewretd_&i = .;
				END;
			%END;
					
			cvwretd = (1 + vwretd) %DO i = 1 %TO &n - 1; * (1 + vwretd_&i) %END; - 1;
			cewretd = (1 + ewretd) %DO i = 1 %TO &n - 1; * (1 + ewretd_&i) %END; - 1;

			sum_vwretd = SUM(vwretd %DO i = 1 %TO &n - 1; , vwretd_&i %END;);
			sum_ewretd = SUM(ewretd %DO i = 1 %TO &n - 1; , ewretd_&i %END;);

			KEEP ff49 date cvwretd cewretd sum_vwretd sum_ewretd;
		RUN;
	%END;
%MEND cumulativeffret;
%cumulativeffret(1);
%cumulativeffret(12);

* Calculate industry adjusted returns;
%MACRO BHARFF (v, date1, date2);
	%IF &v = n3_n2 OR &v = n2_n1 OR &v = n1_0 %THEN %LET month = 1;
	%IF &v = 0_12 OR &v = 12_24 OR &v = 24_36 OR &v = 36_48 OR &v = 48_60 %THEN %LET month = 12;
	
	* merge cumulative return with FF;
	PROC SQL;
		CREATE TABLE bharff_m&v AS
		SELECT	a.gvkey, a.lpermno, a.monthend, a.&date1, a.&date2, 
				b.rret&v - c.cvwretd AS rvwffret&v
		FROM	gvkeyindexpermno_bharff_m a, bharmkt_m&v b, ffret&month c
		WHERE	a.gvkey = b.gvkey AND a.monthend = b.monthend AND a.lpermno = b.lpermno
				AND a.ff49 = c.ff49 AND YEAR(a.&date2) * 100 + MONTH(a.&date2) = c.date;
	QUIT;
%MEND BHARFF;

%BHARFF(n3_n2, lag3monthend, lag2monthend); 
%BHARFF(n2_n1, lag2monthend, lag1monthend); 
%BHARFF(n1_0, lag1monthend, monthend);
%BHARFF(0_12, monthend, fwd12monthend);
%BHARFF(12_24, fwd12monthend, fwd24monthend);
%BHARFF(24_36, fwd24monthend, fwd36monthend);
%BHARFF(36_48, fwd36monthend, fwd48monthend);

PROC SQL;
	CREATE TABLE bharff_m AS
	SELECT	COALESCE(a.gvkey, b.gvkey, c.gvkey) AS gvkey,
			COALESCE(a.monthend, b.monthend, c.monthend) AS monthend FORMAT YYMMDDN8.,
			COALESCE(a.lpermno, b.lpermno, c.lpermno) AS lpermno,
			*
	FROM	bharff_m0_12 (KEEP = gvkey monthend lpermno rvwffret0_12) a 
			FULL JOIN bharff_mn2_n1 (KEEP = gvkey monthend lpermno rvwffretn2_n1) b
	ON		a.gvkey = b.gvkey AND a.monthend = b.monthend 
			FULL JOIN bharff_mn1_0 (KEEP = gvkey monthend lpermno rvwffretn1_0) c
	ON		COALESCE(a.gvkey, b.gvkey) = c.gvkey AND COALESCE(a.monthend, b.monthend) = c.monthend 
			LEFT JOIN bharff_mn3_n2 (KEEP = gvkey monthend rvwffretn3_n2) d
	ON		COALESCE(a.gvkey, b.gvkey, c.gvkey) = d.gvkey AND COALESCE(a.monthend, b.monthend, c.monthend) = d.monthend 
			LEFT JOIN bharff_m12_24 (KEEP = gvkey monthend rvwffret12_24) j
	ON		COALESCE(a.gvkey, b.gvkey, c.gvkey) = j.gvkey AND COALESCE(a.monthend, b.monthend, c.monthend) = j.monthend
			LEFT JOIN bharff_m24_36 (KEEP = gvkey monthend rvwffret24_36) k
	ON		COALESCE(a.gvkey, b.gvkey, c.gvkey) = k.gvkey AND COALESCE(a.monthend, b.monthend, c.monthend) = k.monthend
			LEFT JOIN bharff_m36_48 (KEEP = gvkey monthend rvwffret36_48) l
	ON		COALESCE(a.gvkey, b.gvkey, c.gvkey) = l.gvkey AND COALESCE(a.monthend, b.monthend, c.monthend) = l.monthend;
QUIT;

/* **********************************************************************************/
/* ************** W R D S   R E S E A R C H   A P P L I C A T I O N S ***************/
/* **********************************************************************************/
/* Summary   : Construct Daniel Grinblatt Titman and Wermers(1997) Benchmarks       */
/* Date      : January, 2011                                                        */
/* Author    : Rabih Moussawi and Gjergji Cici                                      */
/*	Following Wermers (2004):														*/
/*				Calculate industry adjusted B/M 									*/
/*	Following http://terpconnect.umd.edu/~wermers/ftpsite/Dgtw/coverpage.htm :		*/
/*				1. For Jan-May FYE firms, use their B/M in same year June sorting	*/
/*				2. Adjust for delisting returns										*/
/*				3.* Already used here, because it is using WRDS server, not WEB		*/
/* 				4. Use 'USD' to screen out Canadian firms							*/
/* Variables : - BEGDATE: Sample Start Date                                         */
/*             - ENDDATE: Sample End Date                                           */
/* **********************************************************************************/


%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;


/* Step 1. Specifying Options */
%LET begdate = 01JAN1991;
%LET enddate = &SYSDATE9.;

/* Create a CRSP Subsample with Monthly Stock and Event Variables */
/* Restriction on the type of shares (common stocks only) */
%LET sfilter = (shrcd IN (10,11));
/* Selected variables from the CRSP monthly data file (crspa.msf file) */
%LET msfvars = permco prc ret vol shrout cfacpr cfacshr;
%LET msevars = ncusip exchcd shrcd siccd ;
/* This procedure creates a Monthly CRSP dataset named "CRSP_M"  */
%crspmerge(s=m, start=&begdate, end=&enddate, sfvars=&msfvars, sevars=&msevars, filters=&sfilter);

/* Adjust Share and Price in Monthly Data */
data crsp_m; 
set crsp_m;
DATE = INTNX("MONTH",date,0,"E");
P = abs(prc)/cfacpr;
TSO=shrout*cfacshr*1000; 
if TSO<=0 then TSO=.;
ME = P*TSO/1000000;
label P = "Price at Period End, Adjusted";
label TSO = "Total Shares Outstanding, Adjusted";
label ME = "Issue-Level Market Capitalization, x$1m";
drop ncusip prc cfacpr shrout shrcd;
format ret percentn8.4 ME P dollar12.3 TSO comma12.;
run;

/* Create Total Market Capitalization at the Company Level */
proc sql  undo_policy=none;
create table crsp_m
as select *, sum(me) as me_comp "Company-Level Market Cap, $million" format dollar12.3
from crsp_m
group by permco,date
order by permno,date;
quit;

/* Get Book Value of Equity from Compustat to Create B/P Rankings */
data comp1;
set comp.funda (keep=gvkey datadate cusip indfmt datafmt consol popsrc
    SICH SEQ PSTKRV PSTKL PSTK TXDB ITCB curcd);
where indfmt='INDL' and datafmt='STD' and consol='C' and popsrc='D'
  and datadate>="&begdate"d AND curcd = 'USD';
  if SEQ>0;                         /* Shareholders' Equity */
  PREF=PSTKRV;                      /* Preferred stock - Redemption Value */
  if missing(pref) then PREF=PSTKL; /* Preferred stock - Liquidating Value */
  if missing(pref) then PREF=PSTK;  /* Preferred stock - Carrying Value, Stock (Capital) - Total */
  BE = sum(SEQ, TXDB, ITCB, -PREF); /* Deferred taxes  and Investment Tax Credit */
  label BE = "Book Value of Equity";
  if BE>=0;
  /* Daniel and Titman (JF 1997):                                                   */
  /* BE = stockholders' equity + deferred taxes + investment tax credit - Preferred Stock */
 label datadate = "Fiscal Year End Date";
 keep gvkey sich datadate BE;
run;

/* Add Historical PERMCO identifier */
proc sql;
  create table comp2 
  as select a.*, b.lpermco as permco, b.linkprim
  from comp1 as a, crspa.ccmxpf_linktable as b
  where a.gvkey = b.gvkey and
  b.LINKTYPE in ("LU","LC") and 
 (b.LINKDT <= a.datadate) and (a.datadate <= b.LINKENDDT or missing(b.LINKENDDT));
quit;

/* Sorting into Buckets is done in July	of Each Year t               */
/* Additional Requirements:                                          */
/* - Compustat data is available for at least 2 years                */
/* - CRSP data available on FYE of year t-1 and June of year t       */
/* - at least 6 months of returns in CRSP between t-1 and t          */
/* - size weights are constructed using the market value in June     */
/* - B/M Ratio uses the market cap at FYE of the year t-1            */
/* - Modified now that B/M for May is available in June				 */
/* - Momentum factor is the 12 month return with 1 month reversal    */

/* Construct Book to Market Ratio Each Fiscal Year End               */
proc sql;
  create table comp3
  as select distinct b.permno,a.gvkey,year(a.datadate) - (MONTH(a.datadate) <= 5) as YEAR,
  a.datadate,a.linkprim,
  a.BE,b.me,a.sich,b.siccd,a.be/b.me_comp as BM "Book-to-Market Ratio" format comma8.2
  from comp2 as a, crsp_m as b
  where a.permco=b.permco and datadate=intnx("month",date,0,"E") 
order by permno,datadate;
quit;

/* Use linkprim='P' for selecting just one permno-gvkey combination   */
/* Also, if a company changes its FYE month, choose the last report   */
proc sort data=comp3 nodupkey; by permno year datadate linkprim bm; run;
data comp3;
set comp3;
by permno year datadate;
if last.year;
drop linkprim;
run;

/* Industry-Adjust the B/M Ratios using F&F(1997) 48-Industries */
data comp4;
set comp3;
/* First, use historical Compustat SIC Code */
if sich>0 then SIC=sich;
/* Then, if missing, use historical CRSP SIC Code */
else if siccd>0 then sic=siccd;
/* and adjust some SIC code to fit F&F 48 ind delineation */
if SIC in (3990,9995,9997) and siccd>0 and siccd ne SIC then SIC = siccd;
if SIC in (3990,3999) then SIC = 3991;
/* F&F 48 Industry Classification Macro */
%FFI48(sic);
if missing (FFI48) or missing(BM) then delete;
drop sich siccd; /* datadate;*/
run;

 /* Industry adjust BM in Each Period */
 /* difference between firm b/m and industry aggregate b/m, scaled by the standard 
	deviation of the difference in the industry year */

PROC SQL;
	CREATE TABLE comp5 AS
	SELECT		a.*, (LOG(bm) - LOG(bm_ind)) / STD(LOG(bm) - LOG(bm_ind)) 
					AS BM_ADJ "Adjusted Book-to-Market Ratio"
	FROM		comp4 a, 
				(SELECT		ffi48, year, SUM(be) / SUM(me) AS bm_ind
				FROM		comp4
				GROUP BY	ffi48, year) b
	WHERE		a.ffi48 = b.ffi48 AND a.year = b.year
	GROUP BY	a.ffi48, a.year;
QUIT;

*proc printto log=junk; run; 

/* merge with delisting return */

PROC SQL;
	CREATE TABLE crsp_m AS
	SELECT	a.*, 
			b.dlret, b.dlstcd
	FROM	crsp_m a LEFT JOIN crspa.dsedelist b
	ON		a.permno = b.permno AND a.date = INTNX("MONTH",b.dlstdt,0,"E");
QUIT;

/* calculate return incorporating delisting returns */
DATA crsp_m;
	SET crsp_m;
	* define performance delisting;
	IF dlstcd ^= . THEN perf_delist = (dlstcd = 500 OR dlstcd >= 505 AND dlstcd <= 588);
	IF dlret = .S OR dlret = .P OR dlret = .T THEN dlret = .;
	* set missing delisting returns to -30% for NYSE and -55% for others;
	IF perf_delist AND dlret = . THEN DO;
		IF exchcd = 1 THEN dlret = -0.30;
		ELSE dlret = -0.55;
	END;
	* no delisting or no delisting return;
	IF dlstcd = . OR dlstcd = 100 OR dlret = . THEN ret_adj = ret;
	* with delisting but without delisting returns;
	ELSE IF ret = . THEN ret_adj = dlret;
	* with both returns;
	ELSE ret_adj = (1 + ret) * (1 + dlret) - 1;
	DROP perf_delist dlstcd dlret;
RUN;

/* Create (12,1) Momentum Factor with at least 6 months of returns */
proc expand data=crsp_m (keep=permno date ret_adj me exchcd) out=sizmom method=none;
by permno;
id date;
convert ret_adj = cret_12m / transformin=(+1) transformout=(MOVPROD 12 -1 trimleft 6);
quit;
*proc printto; run;

/* Keep Momentum Factor and Size at the End of June - which is the formation date */
data sizmom;
set sizmom;
by permno date;
/* First, add the one month reversal gap */
MOM=lag(cret_12m);
if first.permno then MOM=.;
/* Then, keep Momentum Factor at the End of June */
if month(date)=6;
label MOM="12-Month Momentum Factor with one month reversal";
label date="Formation Date"; format MOM ret_adj percentn8.2;
drop cret_12m; rename me=SIZE;
run;

/* Get Size Breakpoints for NYSE firms */
proc sort data=sizmom nodupkey; by date permno; run;

proc univariate data=sizmom noprint;
where exchcd=1; 
by date;
var size;
output out=NYSE pctlpts = 20 to 80 by 20 pctlpre=dec;
run;

/* Add NYSE Size Breakpoints to the Data*/
data sizmom;
merge sizmom NYSE;
by date;
if size>0 and size < dec20 then group = 1;
else if size >= dec20 and size < dec40 then group =2;
else if size >= dec40 and size < dec60 then group =3;
else if size >= dec60 and size < dec80 then group =4;
else if size >= dec80                  then group =5;
drop dec20 dec40 dec60 dec80;
label group = "Size Portfolio Group";
run;

/* Adjusted BtM from the calendar year preceding the formation date */ 
proc sql;
  create table comp6
  as select distinct a.permno, a.gvkey, a.datadate, b.date, b.group, b.size, b.mom, a.year, a.bm_adj
  from comp5 as a, sizmom as b
  where a.permno=b.permno and year(date)=year+1
   and not missing(size+mom+bm_adj+ret_adj);
quit;

/* Start the Triple Sort on Size, Book-to-Market, and Momentum */
proc sort data=comp6 out=port1 nodupkey; by date group permno; run;
proc rank data=port1 out=port2 group=5;
  by date group;
  var bm_adj;
  ranks bmr;
run;
proc sort data=port2; by date group bmr; run;
proc rank data=port2 out=port3 group=5;
  by date group bmr;
  var mom;
  ranks momr;
run;

/* DGTW_PORT 1 for Bottom Quintile, 5 for Top Quintile */
data port4;
set port3;
bmr=bmr+1;
momr=momr+1;
DGTW_PORT=put(group,1.)||put(bmr,1.)||put(momr,1.);
drop group bmr momr year;
if index(DGTW_PORT, '.') then delete;
label DGTW_PORT="Size, BtM, and Momentum Portfolio Number";
run;

/* Use Size in June as Weights in the Value-Weighted Portfolios */
proc sql;
  create table crsp_m1
  as select a.*, b.date as formdate "Formation Date", b.dgtw_port, b.size as sizew
  from crsp_m (keep=permno date ret_adj) as a, port4 as b
  where a.permno=b.permno and intnx('month', b.date,1,'e')<=a.date<=intnx('month', b.date,12,'e');
quit;

/* Calculate Weighted Average Returns */
proc sort data=crsp_m1 nodupkey;  by date dgtw_port permno; run;
proc means data = crsp_m1 noprint;
by date dgtw_port;
where sizew>0;
var ret_adj / weight=sizew ;
output out = dgtw_vwret(drop=_type_ _freq_)  mean= dgtw_vwret; 
run;

/* Calculate DGTW Excess Return */
proc sql;
  create table dgtw_returns (index=(perm_dat=(permno date)))
  as select a.*,b.DGTW_VWRET format percentn8.4 "DGTW Benchmark Return",
    (a.ret_adj-b.DGTW_VWRET) as DGTW_XRET "DGTW Excess Return" format percentn8.4
  from crsp_m1(drop=sizew) as a left join dgtw_vwret as b
  on a.dgtw_port=b.dgtw_port and a.date=b.date
  order by permno, date;
quit;

PROC DOWNLOAD DATA = dgtw_returns OUT = dgtw_returns_monthly; RUN;
ENDRSUBMIT;

/* Reference: Daniel , Kent , Mark Grinblatt, Sheridan Titman, and Russ Wermers,     */
/*   1997, "Measuring Mutual Fund Performance with Characteristic-Based Benchmarks," */
/*   Journal of Finance , 52, pp. 1035-1058.                                         */

/* ********************************************************************************* */
/* *************  Material Copyright Wharton Research Data Services  *************** */
/* ****************************** All Rights Reserved ****************************** */
/* ********************************************************************************* */

* calculate repurchase DGTW returns;
* Output index of firm-period that needs to calculate BHAR;
* link gvkey-period list with PERMNO;
PROC SQL;
	CREATE TABLE gvkeyindexpermno_dgtw_m AS
	SELECT		UNIQUE a.gvkey, b.lpermno, monthend, 
				INTNX('MONTH', monthend, -3, 'END') AS lag3monthend FORMAT MMDDYY10., 
				INTNX('MONTH', monthend, -2, 'END') AS lag2monthend FORMAT MMDDYY10., 
				INTNX('MONTH', monthend, -1, 'END') AS lag1monthend FORMAT MMDDYY10., 
				INTNX('MONTH', monthend, 12, 'END') AS fwd12monthend FORMAT MMDDYY10.,
				INTNX('MONTH', monthend, 24, 'END') AS fwd24monthend FORMAT MMDDYY10.,
				INTNX('MONTH', monthend, 36, 'END') AS fwd36monthend FORMAT MMDDYY10.,
				INTNX('MONTH', monthend, 48, 'END') AS fwd48monthend FORMAT MMDDYY10.
	FROM		v.repurchase_m a, compustat_crsp_link b
	WHERE		a.gvkey = b.gvkey
	ORDER BY	gvkey, monthend, lpermno;
QUIT;

* Merge with DGTW portfolio returns;
%MACRO BHARDGTW(v, date1, date2);
	%IF &v = n3_n2 OR &v = n2_n1 OR &v = n1_0 OR &v = 0_1 OR &v = 1_2 OR &v = 2_3 %THEN %LET month = 1;
	%IF &v = 0_12 OR &v = 12_24 OR &v = 24_36 OR &v = 36_48 OR &v = 48_60 %THEN %LET month = 12;

	PROC SQL;
		CREATE TABLE bhardgtw_m&v AS
		SELECT		a.*, dgtw_port,
					EXP(SUM(LOG(1 + ret_adj))) - 1 AS rret_adj&v,
					EXP(SUM(LOG(1 + dgtw_vwret))) - 1 AS rret_dgtw&v,
					CALCULATED rret_adj&v - CALCULATED rret_dgtw&v AS rdgtwret&v 
						LABEL = 'Cumulative ret over DGTW portfolio ret',
					SUM(ret_adj - dgtw_vwret) AS dgtwcar&v
		FROM		gvkeyindexpermno_dgtw_m (KEEP = gvkey lpermno monthend &date1 &date2) a, 
					dgtw_returns_monthly (KEEP = permno date ret_adj dgtw_port dgtw_vwret dgtw_xret) b
		WHERE		a.lpermno = b.permno AND b.date BETWEEN a.&date1 + 1 AND a.&date2
		GROUP BY	gvkey, monthend, &date1, &date2, lpermno
		HAVING		date = MAX(date) AND COUNT(ret_adj) = &month
		ORDER BY	gvkey, monthend, &date1, &date2, lpermno, dgtw_port DESC;
	QUIT; 
	DATA bhardgtw_m&v;
		SET bhardgtw_m&v;
		BY gvkey monthend &date1 &date2 lpermno DESCENDING dgtw_port; 
		IF FIRST.monthend;
	RUN;
%MEND bhardgtw;

%BHARDGTW(n3_n2, lag3monthend, lag2monthend); 
%BHARDGTW(n2_n1, lag2monthend, lag1monthend); 
%BHARDGTW(n1_0, lag1monthend, monthend);
%BHARDGTW(0_12, monthend, fwd12monthend);
%BHARDGTW(12_24, fwd12monthend, fwd24monthend);
%BHARDGTW(24_36, fwd24monthend, fwd36monthend);
%BHARDGTW(36_48, fwd36monthend, fwd48monthend);

PROC SQL;
	CREATE TABLE bhardgtw_m AS
	SELECT	COALESCE(a.gvkey, b.gvkey, c.gvkey) AS gvkey,
			COALESCE(a.monthend, b.monthend, c.monthend) AS monthend FORMAT YYMMDDN8.,
			COALESCE(a.lpermno, b.lpermno, c.lpermno) AS lpermno,
			*
	FROM	bhardgtw_m0_12 (KEEP = gvkey monthend lpermno rdgtwret0_12) a 
			FULL JOIN bhardgtw_mn2_n1 (KEEP = gvkey monthend lpermno rdgtwretn2_n1) b
	ON		a.gvkey = b.gvkey AND a.monthend = b.monthend 
			FULL JOIN bhardgtw_mn1_0 (KEEP = gvkey monthend lpermno rdgtwretn1_0) c
	ON		COALESCE(a.gvkey, b.gvkey) = c.gvkey AND COALESCE(a.monthend, b.monthend) = c.monthend 
			LEFT JOIN bhardgtw_mn3_n2 (KEEP = gvkey monthend rdgtwretn3_n2) d
	ON		COALESCE(a.gvkey, b.gvkey, c.gvkey) = d.gvkey AND COALESCE(a.monthend, b.monthend, c.monthend) = d.monthend 
			LEFT JOIN bhardgtw_m12_24 (KEEP = gvkey monthend rdgtwret12_24) j
	ON		COALESCE(a.gvkey, b.gvkey, c.gvkey) = j.gvkey AND COALESCE(a.monthend, b.monthend, c.monthend) = j.monthend
			LEFT JOIN bhardgtw_m24_36 (KEEP = gvkey monthend rdgtwret24_36) k
	ON		COALESCE(a.gvkey, b.gvkey, c.gvkey) = k.gvkey AND COALESCE(a.monthend, b.monthend, c.monthend) = k.monthend
			LEFT JOIN bhardgtw_m36_48 (KEEP = gvkey monthend rdgtwret36_48) l
	ON		COALESCE(a.gvkey, b.gvkey, c.gvkey) = l.gvkey AND COALESCE(a.monthend, b.monthend, c.monthend) = l.monthend;
QUIT;

PROC SQL;
	CREATE TABLE bhar_m AS
	SELECT	a.*, b.*, c.*
	FROM	bharmkt_m a 
			LEFT JOIN bharff_m b
	ON		a.gvkey = b.gvkey AND a.monthend = b.monthend AND a.lpermno = b.lpermno
			LEFT JOIN bhardgtw_m c
	ON		a.gvkey = c.gvkey AND a.monthend = c.monthend AND a.lpermno = c.lpermno;
QUIT;

* calculate MA BHAR market adjusted return;
* Output index of firm-period that needs to calculate BHAR;
* list of permno-dates for calculating BHAR;
PROC SQL;
	CREATE TABLE acquiror_bharmkt1_m AS
	SELECT		UNIQUE permno, anndate, 
				anndate + 0 AS anndate1 FORMAT MMDDYY10.,
				anndate - 30 AS lag1anndate FORMAT MMDDYY10.,
				INTNX('MONTH', CALCULATED lag1anndate, -2, 'SAME') AS lag3anndate FORMAT MMDDYY10.,
				INTNX('MONTH', CALCULATED lag1anndate, -1, 'SAME') AS lag2anndate FORMAT MMDDYY10.,
				INTNX('MONTH', CALCULATED anndate1, 12, 'SAME') AS fwd12anndate FORMAT MMDDYY10.,
				INTNX('MONTH', CALCULATED anndate1, 24, 'SAME') AS fwd24anndate FORMAT MMDDYY10.,
				INTNX('MONTH', CALCULATED anndate1, 36, 'SAME') AS fwd36anndate FORMAT MMDDYY10.,
				INTNX('MONTH', CALCULATED anndate1, 48, 'SAME') AS fwd48anndate FORMAT MMDDYY10.
	FROM		v.ma_acquiror;
QUIT;

* Calculate market-adjusted return on WRDS server;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME crspm '/wrds/crsp/sasdata/a_stock'; 

PROC UPLOAD DATA = acquiror_bharmkt1_m OUT = acquiror_bharmkt1_m; RUN;

%MACRO acquiror_BHARmkt1 (v, date1, date2);
	* calculate cumulative return between &date1 and &date2;

	PROC SQL;
		CREATE TABLE acquiror_bharmkt_m&v AS
		SELECT		a.*, b.date, 
					prc * vol AS marketcap,
					COUNT(*) AS count&v,
					EXP(SUM(LOG(1 + ret))) - 1 AS rret&v,
					EXP(SUM(LOG(1 + vwretd))) - 1 AS rvwretd&v,
					SUM(ret - vwretd) AS vwcar&v,
					CALCULATED rret&v - CALCULATED rvwretd&v AS rvwexcessret&v
		FROM		acquiror_bharmkt1_m (KEEP = permno anndate &date1 &date2) a,
					crspm.dsf (KEEP = date permno ret prc vol WHERE = (ret NOT IN (., .C, .B))) b,
					crspm.dsi (KEEP = date vwretd ewretd WHERE = (vwretd ^= . AND ewretd ^= .)) c
		WHERE		a.permno = b.permno AND b.date BETWEEN a.&date1 + 1 AND a.&date2
					AND b.date = c.date
		GROUP BY	a.permno, anndate, &date1, &date2
		HAVING		date = MAX(date) 
		ORDER BY	a.permno, anndate, &date1, &date2, marketcap DESC;
	QUIT;

	* for permnos that are matched to multiple PERMNOs, keep the one with larger market cap;
	DATA acquiror_bharmkt_m&v;
		SET acquiror_bharmkt_m&v;
		BY permno anndate &date1 &date2 DESCENDING marketcap;
		IF FIRST.&date2;
		KEEP permno anndate rret&v vwcar&v rvwexcessret&v;
	RUN;
	
	PROC DOWNLOAD DATA = acquiror_bharmkt_m&v OUT = acquiror_bharmkt1_m&v; RUN;
	PROC DATASETS; DELETE acquiror_bharmkt_m&v; QUIT;
%MEND acquiror_BHARmkt1;

%acquiror_BHARmkt1(n3_n2, lag3anndate, lag2anndate); 
%acquiror_BHARmkt1(n2_n1, lag2anndate, lag1anndate); 
%acquiror_BHARmkt1(n1_0, lag1anndate, anndate1); 
%acquiror_BHARmkt1(0_12, anndate1, fwd12anndate);
%acquiror_BHARmkt1(12_24, fwd12anndate, fwd24anndate); 
%acquiror_BHARmkt1(24_36, fwd24anndate, fwd36anndate); 
%acquiror_BHARmkt1(36_48, fwd36anndate, fwd48anndate); 

ENDRSUBMIT;

PROC SQL;
	CREATE TABLE acquiror_bharmkt1_m AS
	SELECT	COALESCE(a.permno, b.permno, c.permno) AS permno,
			COALESCE(a.anndate, b.anndate, c.anndate) AS anndate FORMAT YYMMDDN8.,
			*
	FROM	acquiror_bharmkt1_m0_12 (KEEP = permno anndate rret0_12 rvwexcessret0_12 vwcar0_12) a 
			FULL JOIN acquiror_bharmkt1_mn2_n1 (KEEP = permno anndate rretn2_n1 rvwexcessretn2_n1 vwcarn2_n1) b
	ON		a.permno = b.permno AND a.anndate = b.anndate 
			FULL JOIN acquiror_bharmkt1_mn1_0 (KEEP = permno anndate rretn1_0 rvwexcessretn1_0 vwcarn1_0) c
	ON		COALESCE(a.permno, b.permno) = c.permno AND COALESCE(a.anndate, b.anndate) = c.anndate 
			LEFT JOIN acquiror_bharmkt1_mn3_n2 (KEEP = permno anndate rretn3_n2 rvwexcessretn3_n2 vwcarn3_n2) d
	ON		COALESCE(a.permno, b.permno, c.permno) = d.permno AND COALESCE(a.anndate, b.anndate, c.anndate) = d.anndate 
			LEFT JOIN acquiror_bharmkt1_m12_24 (KEEP = permno anndate rret12_24 rvwexcessret12_24 vwcar12_24) j
	ON		COALESCE(a.permno, b.permno, c.permno) = j.permno AND COALESCE(a.anndate, b.anndate, c.anndate) = j.anndate
			LEFT JOIN acquiror_bharmkt1_m24_36 (KEEP = permno anndate rret24_36 rvwexcessret24_36 vwcar24_36) k
	ON		COALESCE(a.permno, b.permno, c.permno) = k.permno AND COALESCE(a.anndate, b.anndate, c.anndate) = k.anndate
			LEFT JOIN acquiror_bharmkt1_m36_48 (KEEP = permno anndate rret36_48 rvwexcessret36_48 vwcar36_48) l
	ON		COALESCE(a.permno, b.permno, c.permno) = l.permno AND COALESCE(a.anndate, b.anndate, c.anndate) = l.anndate;
QUIT;

* calculate MA BHAR FF adjusted return;
* Output index of firm-period that needs to calculate BHAR;
* link gvkey-period list with PERMNO;
PROC SQL;
	CREATE TABLE ma_acquiror_sic AS
	SELECT		a.*, COALESCE(b.sich, c.sic_num) AS sic
	FROM		v.ma_acquiror a 
				LEFT JOIN sich b
	ON			a.gvkey = b.gvkey AND (YEAR(a.anndate) = b.fyear OR YEAR(a.anndate) = 2016 AND b.fyear = 2015)
				LEFT JOIN sic c
	ON			a.gvkey = c.gvkey;
QUIT;

PROC SQL;
	CREATE TABLE acquiror_bharff1_m AS
	SELECT		UNIQUE permno, anndate, ind AS ff49,
				anndate + 0 AS anndate1 FORMAT MMDDYY10.,
				anndate - 30 AS lag1anndate FORMAT MMDDYY10.,
				INTNX('MONTH', CALCULATED lag1anndate, -2, 'SAME') AS lag3anndate FORMAT MMDDYY10.,
				INTNX('MONTH', CALCULATED lag1anndate, -1, 'SAME') AS lag2anndate FORMAT MMDDYY10.,
				INTNX('MONTH', CALCULATED anndate1, 12, 'SAME') AS fwd12anndate FORMAT MMDDYY10.,
				INTNX('MONTH', CALCULATED anndate1, 24, 'SAME') AS fwd24anndate FORMAT MMDDYY10.,
				INTNX('MONTH', CALCULATED anndate1, 36, 'SAME') AS fwd36anndate FORMAT MMDDYY10.,
				INTNX('MONTH', CALCULATED anndate1, 48, 'SAME') AS fwd48anndate FORMAT MMDDYY10.
	FROM		ma_acquiror_sic a, v.ff49 b
	WHERE		a.sic BETWEEN INPUT(b.ind1, 4.) AND INPUT(b.ind2, 4.);
QUIT;

PROC SORT DATA = acquiror_bharff1_m NODUPKEY; BY permno anndate; RUN; 

* Import FF industry returns from French Website;
* import value-weighted returns;
PROC IMPORT DATAFILE = "D:\Data\49_Industry_Portfolios_Daily.xls" 
	OUT = ff_vw_daily 
	DBMS = EXCELCS
	REPLACE; 
	SHEET = "vw";
RUN;

* transpose from wide format to long format;
PROC TRANSPOSE DATA = ff_vw_daily OUT = ff_vw_daily;
	BY date;
RUN;

DATA ff_vw_daily;
	SET ff_vw_daily;
	IF col1 = -99.99 THEN vwretd = .;
	ELSE vwretd = col1 / 100;
	year = INT(date / 10000);
	month = INT((date - year * 10000) / 100);
	day = date - year * 10000 - month * 100;
	date = MDY(month, day, year);
	FORMAT date MMDDYY10.;
	KEEP date _name_ vwretd;
RUN;

* import equal-weighted returns;
PROC IMPORT DATAFILE = "D:\Data\49_Industry_Portfolios_Daily.xls" 
	OUT = ff_ew_daily 
	DBMS = EXCELCS
	REPLACE; 
	SHEET = "ew";
RUN;

* transpose from wide format to long format;
PROC TRANSPOSE DATA = ff_ew_daily OUT = ff_ew_daily;
	BY date;
RUN;

DATA ff_ew_daily;
	SET ff_ew_daily;
	IF col1 = -99.99 THEN ewretd = .;
	ELSE ewretd = col1 / 100;
	year = INT(date / 10000);
	month = INT((date - year * 10000) / 100);
	day = date - year * 10000 - month * 100;
	date = MDY(month, day, year);
	FORMAT date MMDDYY10.;
	KEEP date _name_ ewretd;
RUN;

* merge two industry returns;
PROC SQL;
	CREATE TABLE ffret_daily AS
	SELECT		INPUT(SUBSTR(a._name_, 4, LENGTH(a._name_) - 3), 4.) AS ff49, 
				a.date, 
				vwretd, ewretd
	FROM		ff_vw_daily a, ff_ew_daily b
	WHERE		a.date = b.date AND a._name_ = b._name_
	ORDER BY	ff49, date;
QUIT;

* Calculate industry adjusted returns;
%MACRO acquiror_bharff1_m (v, date1, date2);
	* calculate FF return;
	PROC SQL;
		CREATE TABLE acquiror_bharff1_m&v AS
		SELECT		a.*, 
					EXP(SUM(LOG(1 + vwretd))) - 1 AS rvwretd&v
		FROM		acquiror_bharff1_m (KEEP = permno ff49 anndate &date1 &date2) a, ffret_daily b
		WHERE		a.ff49 = b.ff49 AND b.date BETWEEN a.&date1 + 1 AND a.&date2
		GROUP BY	permno, a.ff49, anndate, &date1, &date2;
	QUIT;

	* merge firm return with FF return;
	PROC SQL;
		CREATE TABLE acquiror_bharff1_m&v AS
		SELECT	a.permno, a.anndate,
				rret&v - b.rvwretd&v AS rvwffret&v
		FROM	acquiror_bharmkt1_m a, acquiror_bharff1_m&v b
		WHERE	a.permno = b.permno AND a.anndate = b.anndate;
	QUIT;
%MEND acquiror_bharff1_m;

%acquiror_bharff1_m(n3_n2, lag3anndate, lag2anndate); 
%acquiror_bharff1_m(n2_n1, lag2anndate, lag1anndate); 
%acquiror_bharff1_m(n1_0, lag1anndate, anndate1); 
%acquiror_bharff1_m(0_12, anndate1, fwd12anndate);
%acquiror_bharff1_m(12_24, fwd12anndate, fwd24anndate); 
%acquiror_bharff1_m(24_36, fwd24anndate, fwd36anndate); 
%acquiror_bharff1_m(36_48, fwd36anndate, fwd48anndate); 

PROC SQL;
	CREATE TABLE acquiror_bharff1_m AS
	SELECT	COALESCE(a.permno, b.permno, c.permno) AS permno,
			COALESCE(a.anndate, b.anndate, c.anndate) AS anndate FORMAT YYMMDDN8.,
			*
	FROM	acquiror_bharff1_m0_12 (KEEP = permno anndate rvwffret0_12) a 
			FULL JOIN acquiror_bharff1_mn2_n1 (KEEP = permno anndate rvwffretn2_n1) b
	ON		a.permno = b.permno AND a.anndate = b.anndate 
			FULL JOIN acquiror_bharff1_mn1_0 (KEEP = permno anndate rvwffretn1_0) c
	ON		COALESCE(a.permno, b.permno) = c.permno AND COALESCE(a.anndate, b.anndate) = c.anndate 
			LEFT JOIN acquiror_bharff1_mn3_n2 (KEEP = permno anndate rvwffretn3_n2) d
	ON		COALESCE(a.permno, b.permno, c.permno) = d.permno AND COALESCE(a.anndate, b.anndate, c.anndate) = d.anndate 
			LEFT JOIN acquiror_bharff1_m12_24 (KEEP = permno anndate rvwffret12_24) j
	ON		COALESCE(a.permno, b.permno, c.permno) = j.permno AND COALESCE(a.anndate, b.anndate, c.anndate) = j.anndate
			LEFT JOIN acquiror_bharff1_m24_36 (KEEP = permno anndate rvwffret24_36) k
	ON		COALESCE(a.permno, b.permno, c.permno) = k.permno AND COALESCE(a.anndate, b.anndate, c.anndate) = k.anndate
			LEFT JOIN acquiror_bharff1_m36_48 (KEEP = permno anndate rvwffret36_48) l
	ON		COALESCE(a.permno, b.permno, c.permno) = l.permno AND COALESCE(a.anndate, b.anndate, c.anndate) = l.anndate;
QUIT;

/* **********************************************************************************/
/* ************** W R D S   R E S E A R C H   A P P L I C A T I O N S ***************/
/* **********************************************************************************/
/* Summary   : Construct Daniel Grinblatt Titman and Wermers(1997) Benchmarks       */
/* Date      : January, 2011                                                        */
/* Author    : Rabih Moussawi and Gjergji Cici                                      */
/*	Following Wermers (2004):														*/
/*				Calculate industry adjusted B/M 									*/
/*	Following http://terpconnect.umd.edu/~wermers/ftpsite/Dgtw/coverpage.htm :		*/
/*				1. For Jan-May FYE firms, use their B/M in same year June sorting	*/
/*				2. Adjust for delisting returns										*/
/*				3.* Already used here, because it is using WRDS server, not WEB		*/
/* 				4. Use 'USD' to screen out Canadian firms							*/
/* Variables : - BEGDATE: Sample Start Date                                         */
/*             - ENDDATE: Sample End Date                                           */
/* **********************************************************************************/


%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;


/* Step 1. Specifying Options */
%LET begdate = 01JAN1991;
%LET enddate = &SYSDATE9.;

/* Create a CRSP Subsample with Monthly Stock and Event Variables */
/* Restriction on the type of shares (common stocks only) */
%LET sfilter = (shrcd IN (10,11));
/* Selected variables from the CRSP monthly data file (crspa.msf file) */
%LET msfvars = permco prc ret vol shrout cfacpr cfacshr;
%LET msevars = ncusip exchcd shrcd siccd ;
/* This procedure creates a Monthly CRSP dataset named "CRSP_M"  */
%crspmerge(s=m, start=&begdate, end=&enddate, sfvars=&msfvars, sevars=&msevars, filters=&sfilter);
%crspmerge(s=d, start=&begdate, end=&enddate, sfvars=&msfvars, sevars=&msevars, filters=&sfilter);


/* Adjust Share and Price in Monthly Data */
data crsp_m; 
set crsp_m;
DATE = INTNX("MONTH",date,0,"E");
P = abs(prc)/cfacpr;
TSO=shrout*cfacshr*1000; 
if TSO<=0 then TSO=.;
ME = P*TSO/1000000;
label P = "Price at Period End, Adjusted";
label TSO = "Total Shares Outstanding, Adjusted";
label ME = "Issue-Level Market Capitalization, x$1m";
drop ncusip prc cfacpr shrout shrcd;
format ret percentn8.4 ME P dollar12.3 TSO comma12.;
run;


/* Create Total Market Capitalization at the Company Level */
proc sql  undo_policy=none;
create table crsp_m
as select *, sum(me) as me_comp "Company-Level Market Cap, $million" format dollar12.3
from crsp_m
group by permco,date
order by permno,date;
quit;

/* Get Book Value of Equity from Compustat to Create B/P Rankings */
data comp1;
set comp.funda (keep=gvkey datadate cusip indfmt datafmt consol popsrc
    SICH SEQ PSTKRV PSTKL PSTK TXDB ITCB curcd);
where indfmt='INDL' and datafmt='STD' and consol='C' and popsrc='D'
  and datadate>="&begdate"d AND curcd = 'USD';
  if SEQ>0;                         /* Shareholders' Equity */
  PREF=PSTKRV;                      /* Preferred stock - Redemption Value */
  if missing(pref) then PREF=PSTKL; /* Preferred stock - Liquidating Value */
  if missing(pref) then PREF=PSTK;  /* Preferred stock - Carrying Value, Stock (Capital) - Total */
  BE = sum(SEQ, TXDB, ITCB, -PREF); /* Deferred taxes  and Investment Tax Credit */
  label BE = "Book Value of Equity";
  if BE>=0;
  /* Daniel and Titman (JF 1997):                                                   */
  /* BE = stockholders' equity + deferred taxes + investment tax credit - Preferred Stock */
 label datadate = "Fiscal Year End Date";
 keep gvkey sich datadate BE;
run;

/* Add Historical PERMCO identifier */
proc sql;
  create table comp2 
  as select a.*, b.lpermco as permco, b.linkprim
  from comp1 as a, crspa.ccmxpf_linktable as b
  where a.gvkey = b.gvkey and
  b.LINKTYPE in ("LU","LC") and 
 (b.LINKDT <= a.datadate) and (a.datadate <= b.LINKENDDT or missing(b.LINKENDDT));
quit;

/* Sorting into Buckets is done in July	of Each Year t               */
/* Additional Requirements:                                          */
/* - Compustat data is available for at least 2 years                */
/* - CRSP data available on FYE of year t-1 and June of year t       */
/* - at least 6 months of returns in CRSP between t-1 and t          */
/* - size weights are constructed using the market value in June     */
/* - B/M Ratio uses the market cap at FYE of the year t-1            */
/* - Modified now that B/M for May is available in June				 */
/* - Momentum factor is the 12 month return with 1 month reversal    */

/* Construct Book to Market Ratio Each Fiscal Year End               */
proc sql;
  create table comp3
  as select distinct b.permno,a.gvkey,year(a.datadate) - (MONTH(a.datadate) <= 5) as YEAR,
  a.datadate,a.linkprim,
  a.BE,b.me,a.sich,b.siccd,a.be/b.me_comp as BM "Book-to-Market Ratio" format comma8.2
  from comp2 as a, crsp_m as b
  where a.permco=b.permco and datadate=intnx("month",date,0,"E") 
order by permno,datadate;
quit;

/* Use linkprim='P' for selecting just one permno-gvkey combination   */
/* Also, if a company changes its FYE month, choose the last report   */
proc sort data=comp3 nodupkey; by permno year datadate linkprim bm; run;
data comp3;
set comp3;
by permno year datadate;
if last.year;
drop linkprim;
run;

/* Industry-Adjust the B/M Ratios using F&F(1997) 48-Industries */
data comp4;
set comp3;
/* First, use historical Compustat SIC Code */
if sich>0 then SIC=sich;
/* Then, if missing, use historical CRSP SIC Code */
else if siccd>0 then sic=siccd;
/* and adjust some SIC code to fit F&F 48 ind delineation */
if SIC in (3990,9995,9997) and siccd>0 and siccd ne SIC then SIC = siccd;
if SIC in (3990,3999) then SIC = 3991;
/* F&F 48 Industry Classification Macro */
%FFI48(sic);
if missing (FFI48) or missing(BM) then delete;
drop sich siccd; /* datadate;*/
run;

 /* Industry adjust BM in Each Period */
 /* difference between firm b/m and industry aggregate b/m, scaled by the standard 
	deviation of the difference in the industry year */

PROC SQL;
	CREATE TABLE comp5 AS
	SELECT		a.*, (LOG(bm) - LOG(bm_ind)) / STD(LOG(bm) - LOG(bm_ind)) 
					AS BM_ADJ "Adjusted Book-to-Market Ratio"
	FROM		comp4 a, 
				(SELECT		ffi48, year, SUM(be) / SUM(me) AS bm_ind
				FROM		comp4
				GROUP BY	ffi48, year) b
	WHERE		a.ffi48 = b.ffi48 AND a.year = b.year
	GROUP BY	a.ffi48, a.year;
QUIT;

*proc printto log=junk; run; 

/* merge with delisting return */

PROC SQL;
	CREATE TABLE crsp_m AS
	SELECT	a.*, 
			b.dlret, b.dlstcd
	FROM	crsp_m a LEFT JOIN crspa.dsedelist b
	ON		a.permno = b.permno AND a.date = INTNX("MONTH",b.dlstdt,0,"E");
QUIT;

/* calculate return incorporating delisting returns */
DATA crsp_m;
	SET crsp_m;
	* define performance delisting;
	IF dlstcd ^= . THEN perf_delist = (dlstcd = 500 OR dlstcd >= 505 AND dlstcd <= 588);
	IF dlret = .S OR dlret = .P OR dlret = .T THEN dlret = .;
	* set missing delisting returns to -30% for NYSE and -55% for others;
	IF perf_delist AND dlret = . THEN DO;
		IF exchcd = 1 THEN dlret = -0.30;
		ELSE dlret = -0.55;
	END;
	* no delisting or no delisting return;
	IF dlstcd = . OR dlstcd = 100 OR dlret = . THEN ret_adj = ret;
	* with delisting but without delisting returns;
	ELSE IF ret = . THEN ret_adj = dlret;
	* with both returns;
	ELSE ret_adj = (1 + ret) * (1 + dlret) - 1;
	DROP perf_delist dlstcd dlret;
RUN;

/* Create (12,1) Momentum Factor with at least 6 months of returns */
proc expand data=crsp_m (keep=permno date ret_adj me exchcd) out=sizmom method=none;
by permno;
id date;
convert ret_adj = cret_12m / transformin=(+1) transformout=(MOVPROD 12 -1 trimleft 6);
quit;
*proc printto; run;

/* Keep Momentum Factor and Size at the End of June - which is the formation date */
data sizmom;
set sizmom;
by permno date;
/* First, add the one month reversal gap */
MOM=lag(cret_12m);
if first.permno then MOM=.;
/* Then, keep Momentum Factor at the End of June */
if month(date)=6;
label MOM="12-Month Momentum Factor with one month reversal";
label date="Formation Date"; format MOM ret_adj percentn8.2;
drop cret_12m; rename me=SIZE;
run;

/* Get Size Breakpoints for NYSE firms */
proc sort data=sizmom nodupkey; by date permno; run;

proc univariate data=sizmom noprint;
where exchcd=1; 
by date;
var size;
output out=NYSE pctlpts = 20 to 80 by 20 pctlpre=dec;
run;

/* Add NYSE Size Breakpoints to the Data*/
data sizmom;
merge sizmom NYSE;
by date;
if size>0 and size < dec20 then group = 1;
else if size >= dec20 and size < dec40 then group =2;
else if size >= dec40 and size < dec60 then group =3;
else if size >= dec60 and size < dec80 then group =4;
else if size >= dec80                  then group =5;
drop dec20 dec40 dec60 dec80;
label group = "Size Portfolio Group";
run;

/* Adjusted BtM from the calendar year preceding the formation date */ 
proc sql;
  create table comp6
  as select distinct a.permno, a.gvkey, a.datadate, b.date, b.group, b.size, b.mom, a.year, a.bm_adj
  from comp5 as a, sizmom as b
  where a.permno=b.permno and year(date)=year+1
   and not missing(size+mom+bm_adj+ret_adj);
quit;

/* Start the Triple Sort on Size, Book-to-Market, and Momentum */
proc sort data=comp6 out=port1 nodupkey; by date group permno; run;
proc rank data=port1 out=port2 group=5;
  by date group;
  var bm_adj;
  ranks bmr;
run;
proc sort data=port2; by date group bmr; run;
proc rank data=port2 out=port3 group=5;
  by date group bmr;
  var mom;
  ranks momr;
run;

/* DGTW_PORT 1 for Bottom Quintile, 5 for Top Quintile */
data port4;
set port3;
bmr=bmr+1;
momr=momr+1;
DGTW_PORT=put(group,1.)||put(bmr,1.)||put(momr,1.);
drop group bmr momr year;
if index(DGTW_PORT, '.') then delete;
label DGTW_PORT="Size, BtM, and Momentum Portfolio Number";
run;

/* Use Size in June as Weights in the Value-Weighted Portfolios */
proc sql;
  create table crsp_m1
  as select a.*, b.date as formdate "Formation Date", b.dgtw_port, b.size as sizew
  from crsp_m (keep=permno date ret_adj) as a, port4 as b
  where a.permno=b.permno and intnx('month', b.date,1,'e')<=a.date<=intnx('month', b.date,12,'e');
quit;


/* daily return merge with delisting return */

PROC SQL;
	CREATE TABLE crsp_d AS
	SELECT	a.*, 
			b.dlret, b.dlstcd
	FROM	crsp_d a LEFT JOIN crspa.dsedelist b
	ON		a.permno = b.permno AND a.date = b.dlstdt;
QUIT;


/* calculate return incorporating delisting returns */
DATA crsp_d;
	SET crsp_d;
	* define performance delisting;
	IF dlstcd ^= . THEN perf_delist = (dlstcd = 500 OR dlstcd >= 505 AND dlstcd <= 588);
	IF dlret = .S OR dlret = .P OR dlret = .T THEN dlret = .;
	* set missing delisting returns to -30% for NYSE and -55% for others;
	IF perf_delist AND dlret = . THEN DO;
		IF exchcd = 1 THEN dlret = -0.30;
		ELSE dlret = -0.55;
	END;
	* no delisting or no delisting return;
	IF dlstcd = . OR dlstcd = 100 OR dlret = . THEN ret_adj = ret;
	* with delisting but without delisting returns;
	ELSE IF ret = . THEN ret_adj = dlret;
	* with both returns;
	ELSE ret_adj = (1 + ret) * (1 + dlret) - 1;
	KEEP permno date ret ret_adj;
RUN;

PROC SQL;
	CREATE TABLE crsp_d1 AS
	SELECT		a.*, b.date as formdate "Formation Date", b.dgtw_port, b.size as sizew
	FROM		crsp_d a, port4 b
	WHERE		a.permno = b.permno AND INTNX('MONTH', b.date, 1, 'B') <= a.date <= INTNX('MONTH', b.date, 12, 'E')
	ORDER BY	date, dgtw_port, permno;
QUIT;
/* Calculate Weighted Average Returns */
proc means data = crsp_d1 noprint;
by date dgtw_port;
where sizew>0;
var ret_adj / weight=sizew ;
output out = dgtw_vwret(drop=_type_ _freq_)  mean= dgtw_vwret; 
run;

PROC DOWNLOAD DATA = dgtw_vwret OUT = dgtw_vwret_daily; RUN;
PROC DOWNLOAD DATA = port4 OUT = dgtw_portfolio; RUN;

ENDRSUBMIT;

/* Reference: Daniel , Kent , Mark Grinblatt, Sheridan Titman, and Russ Wermers,     */
/*   1997, "Measuring Mutual Fund Performance with Characteristic-Based Benchmarks," */
/*   Journal of Finance , 52, pp. 1035-1058.                                         */

/* ********************************************************************************* */
/* *************  Material Copyright Wharton Research Data Services  *************** */
/* ****************************** All Rights Reserved ****************************** */
/* ********************************************************************************* */

* calculate MA DGTW adjusted return;
* Output index of firm-period that needs to calculate BHAR;
* list of permno-dates for calculating CAR;
PROC SQL;
	CREATE TABLE acquiror_bhardgtw1_m AS
	SELECT		UNIQUE permno, anndate, 
				anndate + 0 AS anndate1 FORMAT MMDDYY10.,
				anndate - 30 AS lag1anndate FORMAT MMDDYY10.,
				INTNX('MONTH', CALCULATED lag1anndate, -2, 'SAME') AS lag3anndate FORMAT MMDDYY10.,
				INTNX('MONTH', CALCULATED lag1anndate, -1, 'SAME') AS lag2anndate FORMAT MMDDYY10.,
				INTNX('MONTH', CALCULATED anndate1, 12, 'SAME') AS fwd12anndate FORMAT MMDDYY10.,
				INTNX('MONTH', CALCULATED anndate1, 24, 'SAME') AS fwd24anndate FORMAT MMDDYY10.,
				INTNX('MONTH', CALCULATED anndate1, 36, 'SAME') AS fwd36anndate FORMAT MMDDYY10.,
				INTNX('MONTH', CALCULATED anndate1, 48, 'SAME') AS fwd48anndate FORMAT MMDDYY10.
	FROM		v.ma_acquiror;
QUIT;

* Calculate market-adjusted returns;
%MACRO acquiror_bhardgtw1_m(v, date1, date2);

	PROC SQL;
		CREATE TABLE acquiror_bhardgtw1_m&v AS
		SELECT		a.*, b.dgtw_port,
					EXP(SUM(LOG(1 + dgtw_vwret))) - 1 AS rret_dgtw&v
		FROM		acquiror_bhardgtw1_m (KEEP = permno anndate &date1 &date2) a, 
					dgtw_portfolio (KEEP = permno date dgtw_port) b,
					dgtw_vwret_daily (KEEP = date dgtw_port dgtw_vwret) c
		WHERE		a.permno = b.permno AND INTNX('MONTH', b.date, 1, 'B') <= a.anndate <= INTNX('MONTH', b.date, 12, 'E')
					AND b.dgtw_port = c.dgtw_port AND c.date BETWEEN a.&date1 + 1 AND a.&date2
		GROUP BY	a.permno, a.anndate, &date1, &date2, b.dgtw_port
		ORDER BY	permno, anndate, &date1, &date2, dgtw_port DESC;
	QUIT; 
	DATA acquiror_bhardgtw1_m&v;
		SET acquiror_bhardgtw1_m&v;
		BY permno anndate &date1 &date2 DESCENDING dgtw_port; 
		IF FIRST.&date2;
	RUN;
	PROC SQL;
		CREATE TABLE acquiror_bhardgtw1_m&v AS
		SELECT	a.permno, a.anndate, a.&date1, a.&date2, 
				rret&v - rret_dgtw&v AS rdgtwret&v 
		FROM	acquiror_bhardgtw1_m&v a, acquiror_bharmkt1_m b
		WHERE	a.permno = b.permno AND a.anndate = b.anndate;
	QUIT;
%MEND acquiror_bhardgtw1_m;

%acquiror_bhardgtw1_m(n3_n2, lag3anndate, lag2anndate); 
%acquiror_bhardgtw1_m(n2_n1, lag2anndate, lag1anndate); 
%acquiror_bhardgtw1_m(n1_0, lag1anndate, anndate1);  
%acquiror_bhardgtw1_m(0_12, anndate1, fwd12anndate);
%acquiror_bhardgtw1_m(12_24, fwd12anndate, fwd24anndate); 
%acquiror_bhardgtw1_m(24_36, fwd24anndate, fwd36anndate); 
%acquiror_bhardgtw1_m(36_48, fwd36anndate, fwd48anndate); 

PROC SQL;
	CREATE TABLE acquiror_bhardgtw1_m AS
	SELECT	COALESCE(a.permno, b.permno, c.permno) AS permno,
			COALESCE(a.anndate, b.anndate, c.anndate) AS anndate FORMAT YYMMDDN8.,
			*
	FROM	acquiror_bhardgtw1_m0_12 (KEEP = permno anndate rdgtwret0_12) a 
			FULL JOIN acquiror_bhardgtw1_mn2_n1 (KEEP = permno anndate rdgtwretn2_n1) b
	ON		a.permno = b.permno AND a.anndate = b.anndate 
			FULL JOIN acquiror_bhardgtw1_mn1_0 (KEEP = permno anndate rdgtwretn1_0) c
	ON		COALESCE(a.permno, b.permno) = c.permno AND COALESCE(a.anndate, b.anndate) = c.anndate 
			LEFT JOIN acquiror_bhardgtw1_mn3_n2 (KEEP = permno anndate rdgtwretn3_n2) d
	ON		COALESCE(a.permno, b.permno, c.permno) = d.permno AND COALESCE(a.anndate, b.anndate, c.anndate) = d.anndate
			LEFT JOIN acquiror_bhardgtw1_m12_24 (KEEP = permno anndate rdgtwret12_24) j
	ON		COALESCE(a.permno, b.permno, c.permno) = j.permno AND COALESCE(a.anndate, b.anndate, c.anndate) = j.anndate
			LEFT JOIN acquiror_bhardgtw1_m24_36 (KEEP = permno anndate rdgtwret24_36) k
	ON		COALESCE(a.permno, b.permno, c.permno) = k.permno AND COALESCE(a.anndate, b.anndate, c.anndate) = k.anndate
			LEFT JOIN acquiror_bhardgtw1_m36_48 (KEEP = permno anndate rdgtwret36_48) l
	ON		COALESCE(a.permno, b.permno, c.permno) = l.permno AND COALESCE(a.anndate, b.anndate, c.anndate) = l.anndate;
QUIT;

* calculate MA CAR;
* list of permno-dates for calculating CAR;
PROC SORT DATA = v.ma_acquiror OUT = ma_acquiror_permnodate (KEEP = permno anndate) NODUPKEY;
	BY permno anndate;
RUN;

* Calculate market reaction to acquirors;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME crsp '/wrds/crsp/sasdata/a_stock'; 

PROC UPLOAD DATA = ma_acquiror_permnodate OUT = ma_acquiror_permnodate; RUN;

%MACRO MAcar;
	* retrieve daily returns;
	PROC SQL;
		CREATE TABLE ma_acquiror_car AS
		SELECT		a.*, b.date, ret, ret - vwretd AS mar
		FROM		ma_acquiror_permnodate a,
					crsp.dsf (KEEP = date permno ret 
						WHERE = (date BETWEEN MDY(12, 1, 2005) AND MDY(1, 31, 2017)
							AND ret NOT IN (., .C, .B))) b,
					crsp.dsi (KEEP = date vwretd  
						WHERE = (date BETWEEN MDY(12, 1, 2005) AND MDY(1, 31, 2017)
							AND vwretd ^= .)) c
		WHERE		a.permno = b.permno AND ABS(a.anndate - b.date) < 21 AND b.date = c.date
		ORDER BY	permno, anndate, date DESC;
	QUIT;
		
	* for POST announcement window;
	DATA ma_acquiror_car;
		SET ma_acquiror_car;
		BY permno anndate DESCENDING date;
		ret0 = ret;
		mar0 = mar;
		%DO i = 1 %TO 1;
			ret&i = LAG&i(ret);
			mar&i = LAG&i(mar);
		%END;
		%DO i = 1 %TO 1;
			IF permno = LAG&i(permno) AND anndate = LAG&i(anndate) THEN DO;
				ret0&i = (1 + ret) %DO j = 1 %TO &i; * (1 + ret&j) %END; - 1;
				mar0&i = (1 + mar0) %DO j = 1 %TO &i; * (1 + mar&j) %END; - 1;
				car0&i = SUM(mar0 %DO j = 1 %TO &i; ,mar&j %END;);
			END;
		%END;
	RUN;

	* for PRE announcement window;
	PROC SORT DATA = ma_acquiror_car;
		BY permno anndate date;
	RUN;

	DATA ma_acquiror_car;
		SET ma_acquiror_car;
		BY permno anndate date;
		%DO i = 1 %TO 1;
			ret_lag&i = LAG&i(ret);
			mar_lag&i = LAG&i(mar);
		%END;
		* calculate (-i, i);
		%DO i = 1 %TO 1;
			IF permno = LAG&i(permno) AND anndate = LAG&i(anndate) THEN DO;
				ret_&i.&i = (1 + ret0&i) %DO j = 1 %TO &i; * (1 + ret_lag&j) %END; - 1;
				mar_&i.&i = (1 + mar0&i) %DO j = 1 %TO &i; * (1 + mar_lag&j) %END; - 1;
				car_&i.&i = SUM(car0&i %DO j = 1 %TO &i; ,mar_lag&j %END;);
			END;
		%END;
		DROP ret_lag1 mar_lag1 ret ret1 mar mar1;
	RUN;

	DATA ma_acquiror_car;
		SET ma_acquiror_car (WHERE = (date >= anndate));
		BY permno anndate date;
		IF FIRST.anndate;
	RUN;

	PROC DOWNLOAD DATA = ma_acquiror_car OUT = v.ma_acquiror_car; RUN;
%MEND MAcar;
%MAcar;
ENDRSUBMIT;

* merge all MA BHAR;
PROC SQL;
	CREATE TABLE v.acquiror_bhar1_m AS
	SELECT	a.*, b.*, c.*, d.*, ABS(e.car_11) AS macar_11_abs
	FROM	v.ma_acquiror (WHERE = (deal_size >= 5 OR deal_size = .)) a 
			LEFT JOIN acquiror_bharmkt1_m b
	ON		a.permno = b.permno AND a.anndate = b.anndate
			LEFT JOIN acquiror_bharff1_m c
	ON		a.permno = c.permno AND a.anndate = c.anndate
			LEFT JOIN acquiror_bhardgtw1_m d
	ON		a.permno = d.permno AND a.anndate = d.anndate
			LEFT JOIN v.ma_acquiror_car e
	ON		a.permno = e.permno AND a.anndate = e.anndate;
QUIT;

* merge to final sample; 
* Merge repurchase with controls;
PROC SQL;
	CREATE TABLE samplem1 AS
	SELECT		a.*, b.datadate AS lagdatadate, 
				b.*
	FROM		v.repurchase_m a LEFT JOIN compustatcontrol (DROP = fyearq) b
	ON			a.gvkey = b.gvkey AND INTNX('MONTH', b.datadate, 3, 'END') = a.datadate;
QUIT;

PROC SQL;
	CREATE TABLE samplem1 AS
	SELECT	a.*, b.*
	FROM	samplem1 a LEFT JOIN goodwillq (DROP = atq) b
	ON		a.gvkey = b.gvkey AND a.lagdatadate = b.datadate;
QUIT;

PROC SORT DATA = samplem1 NODUPKEY; BY gvkey monthend; RUN;

*  Merge with vesting equity;
PROC SQL;
	CREATE TABLE samplem2 AS
	SELECT	a.*, company_id, company_name, executive, executive_id2, fy_end,
			SUM(vestingshares * lagprc, delta1 * lagprc, 0) AS vestingval1
	FROM	samplem1 a, mnewlyvesting_v1 b
	WHERE	a.gvkey = b.gvkey AND a.monthend = b.datadate;
QUIT;

PROC SORT DATA = samplem2 NODUPKEY; BY gvkey monthend executive_id2; RUN;

* Merge with vested and unvested;
* calcualte annual vested and unvested;
DATA sample_annual;
	SET v.mm;
	IF lagexist;
	KEEP fyear gvkey company_id fy_end executive_id2 unvestedtotalvaladj1 alreadyvestedtotalval1;
	alreadyvestedoptionnumber1 = MAX(alreadyvestedsum_delta1, 0);
	alreadyvestedstocknumber = MAX(alreadyvestedshares, 0);
	newlyvestingoptionnumber1 = MAX(newlyvestedsum_delta1, 0);
	unvestedstocknumber = SUM(total_unvested_shares, total_unvested_ip_shares, 0);
	newlyvestingstocknumber = MAX(shares_acquired_on_vesting_of_st, 0);
	unvestedoptionnumber1 = MAX(unvestedsum_delta1, 0);
	alreadyvestedtotalnumber1 = alreadyvestedstocknumber + alreadyvestedoptionnumber1;
	unvestedstockval = unvestedstocknumber * lagfinalprc;
	newlyvestingstockval = newlyvestingstocknumber * lagfinalprc;
	unvestedoptionval1 = unvestedoptionnumber1 * lagfinalprc;
	newlyvestingoptionval1 = newlyvestingoptionnumber1 * lagfinalprc;
	unvestedtotalvaladj1 = MAX(unvestedstockval - newlyvestingstockval, 0) + MAX(unvestedoptionval1 - newlyvestingoptionval1, 0);
	alreadyvestedtotalval1 = alreadyvestedtotalnumber1 * lagfinalprc;
RUN;

PROC SQL;
	CREATE TABLE sample_annual AS
	SELECT	a.*, b.sich
	FROM	sample_annual a, sich b
	WHERE	a.gvkey = b.gvkey AND a.fyear = b.fyear;
QUIT;

PROC SQL;
	CREATE TABLE sample_annual AS
	SELECT		a.*, b2.ind AS ff49
	FROM		sample_annual a 
				LEFT JOIN v.ff49 b2
	ON			a.sich BETWEEN INPUT(b2.ind1, 4.) AND INPUT(b2.ind2, 4.);
QUIT;

PROC SORT DATA = sample_annual NODUPKEY; BY company_id fy_end executive_id2; RUN;

PROC SQL;
	CREATE TABLE samplem3 AS
	SELECT	*
	FROM	samplem2 a, sample_annual b
	WHERE	a.executive_id2 = b.executive_id2 AND a.fy_end = b.fy_end AND a.company_id = b.company_id;
QUIT;

PROC SORT DATA = samplem3 NODUPKEY; BY gvkey executive_id2 monthend; RUN;

* Merge with salary/bonus and age/tenure;
* merge index with current and lagged salary/bonus;
PROC SQL;
	CREATE TABLE salarybonus AS
	SELECT	a.*,
			MAX(b.base_salary, 0) AS salary, MAX(b.bonus, 0) AS bonus,
			MAX(c.base_salary, 0) AS lagsalary LABEL = 'lagsalary', 
			MAX(c.bonus, 0) AS lagbonus LABEL = 'lagbonus'
	FROM	v.mm (KEEP = company_id executive_id2 executive fy_end lagfy_end lagexist
					WHERE = (lagexist = 1)) a, 
			v.ednd b,
			v.ednd c
	WHERE	a.company_id = b.company_id AND a.executive_id2 = b.executive_id2 
			AND a.fy_end = b.fy_end
			AND a.company_id = c.company_id AND a.executive_id2 = c.executive_id2 
			AND a.lagfy_end = c.fy_end;
QUIT;

* merge with salary/bonus;
PROC SQL;
	CREATE TABLE samplem4 AS
	SELECT	*
	FROM	samplem3 a, salarybonus (DROP = executive) b
	WHERE	a.executive_id2 = b.executive_id2 AND a.fy_end = b.fy_end 
			AND a.company_id = b.company_id;
QUIT;

PROC SORT DATA = samplem4 NODUPKEY; BY gvkey executive_id2 monthend; RUN;

* merge with age/tenure;
PROC SORT DATA = ceo_age_tenure NODUPKEY; BY company_id executive_id2; RUN;

PROC SQL;
	CREATE TABLE samplem5 AS
	SELECT	a.*, 
			YEAR(datadate) - ceobirthyear AS ceoage, YEAR(datadate) - YEAR(becameceo) + 1 AS ceotenure,
			becameceo
	FROM	samplem4 a, ceo_age_tenure b
	WHERE	a.company_id = b.company_id AND a.executive_id2 = b.executive_id2;
QUIT;

* define new CEO;
DATA samplem5;
	SET samplem5;
	IF ceotenure < 1 THEN ceotenure = 1;
	lagceoage = ceoage - 1;
	lagceotenure = ceotenure - 1;
	newceo = (becameceo <= datadate AND becameceo >= datadate - 366);
	DROP ceoage ceotenure;
RUN;

* Merge with momentum;
PROC SQL;
	CREATE TABLE samplem6 AS
	SELECT	a.*, b.*			
	FROM	samplem5 a, 
			bharmkt_m (KEEP = gvkey monthend lpermno rvwexcessretn1_0 
			RENAME = (lpermno = permno)) b
	WHERE	a.gvkey = b.gvkey AND a.monthend = b.monthend;
QUIT;

PROC SORT DATA = samplem6 NODUPKEY; BY gvkey executive_id2 monthend; RUN;

* For Sensitivity Test, Different Version of Vesting;
PROC SQL;
	CREATE TABLE vesting_other_version AS
	SELECT	b.gvkey, b.datadate, b.executive_id2, 
			SUM(b.vestingshares * b.lagprc, b.delta1 * b.lagprc, 0) AS vestingval_v2,
			SUM(c.vestingshares * c.lagprc, c.delta1 * c.lagprc, 0) AS vestingval_v3,
			SUM(d.vestingshares * d.lagprc, d.delta1 * d.lagprc, 0) AS vestingval_v4
	FROM	mnewlyvesting_v2 (KEEP = gvkey datadate executive_id2 vestingshares lagprc delta1) b, 
			mnewlyvesting_v3 (KEEP = gvkey datadate executive_id2 vestingshares lagprc delta1) c, 
			mnewlyvesting_v4 (KEEP = gvkey datadate executive_id2 vestingshares lagprc delta1) d
	WHERE	b.gvkey = c.gvkey AND b.datadate = c.datadate AND b.executive_id2 = c.executive_id2 AND
			b.gvkey = d.gvkey AND b.datadate = d.datadate AND b.executive_id2 = d.executive_id2;
QUIT;

PROC SQL;
	CREATE TABLE samplem7 AS
	SELECT	a.*, b.*			
	FROM	samplem6 a, vesting_other_version b
	WHERE	a.gvkey = b.gvkey AND a.monthend = b.datadate AND a.executive_id2 = b.executive_id2;
QUIT;

* Merge with investment;
PROC SORT DATA = qtrinvestment NODUPKEY; BY gvkey datadate; RUN;

PROC SQL;
	CREATE TABLE samplem8 AS
	SELECT	a.*, b.*
	FROM	samplem7 a LEFT JOIN qtrinvestment (KEEP = gvkey datadate deltardatq deltacapxatq deltardcapxatq) b
	ON 		a.gvkey = b.gvkey AND a.datadate = b.datadate;
QUIT;

* Merge with M&A Announcement CAR;
* merge ma_acquiror with market reaction;
PROC SQL;
	CREATE TABLE ma_acquiror AS
	SELECT	a.*, b.car_11 AS macar_11
	FROM	v.ma_acquiror_target_financial a LEFT JOIN v.ma_acquiror_car b
	ON		a.permno = b.permno AND a.anndate = b.anndate;
QUIT;

* merge MA with firm-month index;
PROC SQL;
	CREATE TABLE ma_acquiror AS
	SELECT 		a.gvkey, a.monthend, macar_11, 
				deal_size, allcash
	FROM		samplem8 (KEEP = gvkey monthend) a, ma_acquiror b
	WHERE		a.gvkey = b.gvkey AND 
				b.anndate BETWEEN INTNX('MONTH', monthend, -1, 'END') + 1 AND monthend
	ORDER BY	a.gvkey, a.monthend, deal_size DESC;
QUIT;

* total deal size;
PROC SQL;
	CREATE TABLE ma_acquiror_summary AS
	SELECT		gvkey, monthend, 
				SUM(deal_size) AS deal_size_sum,
				COUNT(*) AS deal_number
	FROM		ma_acquiror
	GROUP BY	gvkey, monthend;
QUIT;

* quarterly deal size;
PROC SQL;
	CREATE TABLE deal_size_sum_q AS
	SELECT 		a.gvkey, a.datadate, SUM(deal_size) AS deal_size_sum_q
	FROM		(SELECT UNIQUE gvkey, datadate FROM samplem8) a, v.ma_acquiror_target_financial b
	WHERE		a.gvkey = b.gvkey AND 
				b.anndate BETWEEN INTNX('MONTH', datadate, -3, 'END') + 1 AND datadate
	GROUP BY	a.gvkey, a.datadate;
QUIT;

* largest (-1, 1) absolute market reaction to deal;
PROC SQL;
	CREATE TABLE pick1_carabs11_large5m2 AS
	SELECT		gvkey, monthend, 
				allcash AS allcash_maxabs_large5m2
	FROM		ma_acquiror
	WHERE		deal_size >= 5 OR deal_size = .
	GROUP BY	gvkey, monthend
	HAVING		ABS(macar_11) = MAX(ABS(macar_11));
QUIT;

PROC SORT DATA = pick1_carabs11_large5m2 NODUPKEY; BY gvkey monthend; RUN;

PROC SQL;
	CREATE TABLE samplem9 AS
	SELECT	a.*, b.*,  
			(b.monthend ^= .) AS acquiror LABEL = 'Has acquisition', 
			(b.monthend ^= . AND (deal_size_sum >= 5 OR deal_size_sum = .)) AS acquiror LABEL = 'Has acquisition (exclu <$5M)', 
			c.*, 
			d.deal_size_sum_q,
			(d.datadate ^= . AND (deal_size_sum_q >= 5 OR deal_size_sum_q = .)) AS acquiror_q LABEL = 'Has acquisition (exclu <$5M)'
	FROM	samplem8 a 
		LEFT JOIN ma_acquiror_summary b
	ON		a.gvkey = b.gvkey AND a.monthend = b.monthend
		LEFT JOIN pick1_carabs11_large5m2 c
	ON		a.gvkey = c.gvkey AND a.monthend = c.monthend
		LEFT JOIN deal_size_sum_q d
	ON		a.gvkey = d.gvkey AND a.datadate = d.datadate;
QUIT;

* Goodwill writedowns;
DATA samplem10;
	SET samplem9;
	IF deal_size_sum = 0 THEN mavalpct100 = 0;
	ELSE mavalpct100 = deal_size_sum / mvq;
	mavalpct100 = COALESCE(mavalpct100, 0);
	gdwlipq_2y1_maval = COALESCE(gdwlipq_2y1 / deal_size_sum_q, 0);
	gdwlipq_3y1_maval = COALESCE(gdwlipq_3y1 / deal_size_sum_q, 0);
	gdwlipq_4y1_maval = COALESCE(gdwlipq_4y1 / deal_size_sum_q, 0);
	IF acquiror = 0 THEN deal_number = 0;
RUN;

* Merge with RP Announcement;
PROC SQL;
	CREATE TABLE samplem11 AS
	SELECT 		a.*, (b.monthend ^= .) AS rpann
	FROM		samplem10 a LEFT JOIN 
				(SELECT UNIQUE a.gvkey, a.monthend
				FROM	samplem10 (KEEP = gvkey monthend) a, v.rpann b
				WHERE	a.gvkey = b.gvkey AND 
						b.anndate BETWEEN INTNX('MONTH', monthend, -1, 'END') + 1 AND monthend) b
	ON			a.gvkey = b.gvkey AND a.monthend = b.monthend;
QUIT;

* Process variables for regression;
DATA samplem12;
	SET samplem11;
	KEEP fyearq fqtr datadate lagdatadate gvkey executive executive_id2 company_name company_id fy_end lagfy_end ff49 permno monthend
		vestingval1 vestingval_v2 vestingval_v3 vestingval_v4 unvestedtotalvaladj1 alreadyvestedtotalval1
		rpm rpvalpctm rpann	acquiror acquiror_q allcash_maxabs_large5m2 mavalpct100 deal_number
		lagceoage lagceotenure newceo lagsalary lagbonus logsaleq mkbk debt operatingroaq noperatingroaq mklev
		deltardatq deltacapxatq deltardcapxatq hhisic3 maliquidity_sic3 rvwexcessretn1_0
		gdwlipq_2y1_maval gdwlipq_3y1_maval gdwlipq_4y1_maval;
RUN;

%MACRO winsorize(var, var_out, by_var, left, right, input, output);
	* Read in variables to winsorize;
	%LET var_number = 1; 
	%LET var&var_number = %QSCAN(&var, &var_number, %STR( ));
	%DO %WHILE (&&var&var_number NE);
		%LET var_number = %EVAL(&var_number + 1);
		%LET var&var_number = %QSCAN(&var, &var_number, %STR( ));
	%END;
	%LET var_number = %EVAL(&var_number - 1);

  * check if both left and right percentile are missing;
  %IF &left = AND &right = %THEN %DO;
    %PUT ERROR: Both top and bottom percentiles to winsorize are missing, no winsorize is needed!!!;
    %GOTO exit;
  %END;

	* if variables after winsorize is 'AUTONAME', then automatically assign names to 
		variables after winsorize as the original variable name + _left tail % + _right tail %;
  	* for example, atq winsorized at 1% and 99% will become atq_1_99;
	%IF &var_out = AUTONAME %THEN %DO;
		%DO i = 1 %TO &var_number;
			%LET &&var_out_&i = &&var&i.._w_&left._&right;
		%END;
	%END;

	%ELSE %DO; 
		* Read in variable names after winsorize;
		%LET var_out_number = 1;
		%LET var_out&var_out_number = %QSCAN(&var_out, &var_out_number, %STR( ));
	  	%DO %WHILE (&&var_out&var_out_number NE);
			%LET var_out_number = %EVAL(&var_out_number + 1);
	 		%LET var_out&var_out_number = %QSCAN(&var_out, &var_out_number, %STR( ));
		%END;
		%LET var_out_number = %EVAL(&var_out_number - 1);
		
		* check if number of variables to winsorize is the same as number
		of variable names after winsorize, output ERROR if it is not the same;
		%IF &var_number ^= &var_out_number %THEN %DO;
			%PUT ERROR: The Number of variables to winsorize does not equal the number of output variables!!!;
			%PUT variables to be winsorize:;
			%DO i = 1 %TO &var_number;	
				%PUT    &i: &&var&i;		
			%END;
			%PUT variable names after winsorize:;
			%DO i = 1 %TO &var_out_number; 
				%PUT    &i: &&var_out&i; 
			%END;		
				%GOTO exit;
		%END;
	%END;

	* set by variable to a dummy if it is missing, to help with merging afterward;
	%IF &by_var =  %THEN %DO;
		%LET by_var = just_for_merge;
		DATA _winsorize_temp;
			SET &input;
			just_for_merge = 1;
		RUN;
	%END;
	%ELSE %DO;
		DATA _winsorize_temp;
			SET &input;
		RUN;
	%END;
	PROC SORT data = _winsorize_temp;
		BY &by_var;
	RUN;
	
	* determine value at the cut-off percentile;
	PROC UNIVARIATE DATA = _winsorize_temp NOPRINT;
		BY &by_var;
		VAR %DO i = 1 %TO &var_number; &&var&i %END;;
		OUTPUT OUT = winsorized PCTLPTS = %IF &left ^= %THEN %DO; &left %END; %IF &right ^= %THEN %DO; &right %END;
			PCTLPRE = %DO i=1 %TO &var_number; var_&i %END;
			PCTLNAME = %IF &left ^= %THEN %DO;  _pct_&left %END; %IF &right ^= %THEN %DO;  _pct_&right %END;;
	RUN;

	* merge the cut-off percentile back to original database and winsorize variables;
	DATA &output;
		MERGE _winsorize_temp winsorized;
		BY &by_var;
		%DO i=1 %TO &var_number;
			IF &&var&i ^= . THEN DO;
				%IF &left ^= AND &right^= %THEN %DO;
					&&var_out&i = MIN(var_&i._pct_&right, MAX(var_&i._pct_&left, &&var&i));
					DROP var_&i._pct_&left var_&i._pct_&right;
				%END;
		        %ELSE %IF &left ^= %THEN %DO;
	  	        			&&var_out&i = MAX(var_&i._pct_&left, &&var&i);
							DROP var_&i._pct_&left;
	      	  	%END;
				%ELSE %IF &right ^= %THEN %DO;
  	        		&&var_out&i = MIN(var_&i._pct_&right, &&var&i);
					DROP var_&i._pct_&right;
				%END;
			END;
		%END;
		%IF &by_var = just_for_merge %THEN DROP just_for_merge;;
	RUN;
	%exit:
%MEND winsorize;

%winsorize(
	var 	= 	vestingval1 vestingval_v2 vestingval_v3 vestingval_v4
				unvestedtotalvaladj1 alreadyvestedtotalval1	rpvalpctm mavalpct100
				lagceoage lagceotenure lagsalary lagbonus logsaleq mkbk debt operatingroaq noperatingroaq mklev 
				deltardatq deltacapxatq deltardcapxatq hhisic3 maliquidity_sic3	rvwexcessretn1_0
				gdwlipq_2y1_maval gdwlipq_3y1_maval gdwlipq_4y1_maval,
	var_out = 	vestingval1 vestingval_v2 vestingval_v3 vestingval_v4
				unvestedtotalvaladj1 alreadyvestedtotalval1	rpvalpctm mavalpct100
				lagceoage lagceotenure lagsalary lagbonus logsaleq mkbk debt operatingroaq noperatingroaq mklev 
				deltardatq deltacapxatq deltardcapxatq hhisic3 maliquidity_sic3	rvwexcessretn1_0
				gdwlipq_2y1_maval gdwlipq_3y1_maval gdwlipq_4y1_maval,
	by_var 	= , 
	left 	= 1, 
	right 	= 99, 
	input 	= samplem12,
	output 	= samplem13);

DATA samplem;
	SET samplem13;
	rpvalpctm100 = rpvalpctm * 100;
	acquiror_allcash_maxabs5m = (allcash_maxabs_large5m2 = 1 AND acquiror);
	acquiror_notallcash_maxabs5m = (acquiror = 1 & acquiror_allcash_maxabs5m = 0);	
	newlyvesting = vestingval1 / 1000000000;	
	newlyvesting_v2 = vestingval_v2 / 1000000000;
	newlyvesting_v3 = vestingval_v3 / 1000000000;
	newlyvesting_v4 = vestingval_v4 / 1000000000;	
	unvested = unvestedtotalvaladj1 / 1000000000;
	alreadyvested = alreadyvestedtotalval1 / 1000000000;
	fyearqtr = fyearq * 10 + fqtr;
	lagsalary_sc = lagsalary / 1000000;
	lagbonus_sc = lagbonus / 1000000;
	ceoage_sc = lagceoage / 100;
	ceotenure_sc = lagceotenure/100;
	yearmonth = YEAR(monthend) * 100 + MONTH(monthend);
RUN;

PROC EXPORT DATA = samplem 
	OUTFILE = 'D:\Data\samplem.dta' 
	REPLACE;
RUN;

* BHAR Surrounding Repurchase;
PROC SORT DATA = bhar_m NODUPKEY; BY gvkey monthend; RUN;

* merge with returns;
%MACRO merge_rp_bhar;
	%LET v1 = n3_n2;
	%LET v2 = n2_n1;
	%LET v3 = n1_0;
	%LET v4 = 0_12;
	%LET v5 = 12_24;
	%LET v6 = 24_36;	
	%LET v7 = 36_48;

	* merge with returns;
	PROC SQL;
		CREATE TABLE samplem_rp_bhar AS
		SELECT	a.*, b.*
		FROM	samplem (DROP = rvwexcessretn1_0 ) a 
				LEFT JOIN bhar_m (KEEP = gvkey monthend 
					%DO i = 1 %TO 7; 
						rvwexcessret&&v&i rvwffret&&v&i rdgtwret&&v&i 
						vwcar&&v&i 
					%END;) b
		ON		a.gvkey = b.gvkey AND a.monthend = b.monthend;
	QUIT;

	* winsorize;
	%winsorize(
		var 	= 	%DO i = 1 %TO 7; 
						rvwexcessret&&v&i rvwffret&&v&i rdgtwret&&v&i 
						vwcar&&v&i 
					%END;,
		var_out = 	%DO i = 1 %TO 7; 
						rvwexcessret&&v&i rvwffret&&v&i rdgtwret&&v&i 
						vwcar&&v&i 
					%END;,
		by_var 	= , 
		left 	= 1, 
		right 	= 99, 
		input 	= samplem_rp_bhar,
		output 	= samplem_rp_bhar);

	DATA samplem_rp_bhar;
		SET samplem_rp_bhar;
		KEEP newlyvesting newlyvesting_v2 newlyvesting_v3 newlyvesting_v4
			rvwexcessretn3_n2 rvwexcessretn2_n1 rvwexcessretn1_0 rvwexcessret0_12 rvwexcessret12_24 rvwexcessret24_36 rvwexcessret36_48
			rvwffretn3_n2 rvwffretn2_n1 rvwffretn1_0 rvwffret0_12 rvwffret12_24 rvwffret24_36 rvwffret36_48
			rdgtwretn3_n2 rdgtwretn2_n1 rdgtwretn1_0 rdgtwret0_12 rdgtwret12_24 rdgtwret24_36 rdgtwret36_48
			vwcar0_12 vwcar12_24 vwcar24_36 vwcar36_48
			deltardatq deltacapxatq deltardcapxatq
			yearmonth gvkey executive_id2;
		WHERE rpm = 1;
	RUN;

	* output to STATA file;
	PROC EXPORT DATA = samplem_rp_bhar
		OUTFILE = "D:\Data\samplem_rp_bhar.dta" 
		REPLACE;
	RUN;
%MEND merge_rp_bhar;
%merge_rp_bhar;

* BHAR Surrounding MA;
* merge with returns;
%MACRO merge_ma_bhar;
	%LET v1 = 0_12;
	%LET v2 = 12_24;
	%LET v3 = 24_36;	
	%LET v4 = 36_48;	

	* merge with returns;
	PROC SQL;
		CREATE TABLE samplem_ma_bhar AS
		SELECT		a.*, b.*
		FROM		samplem (DROP = rvwexcessretn1_0) a, 
					v.acquiror_bhar1_m (KEEP = gvkey anndate macar_11_abs
						%DO i = 1 %TO 4; 
							rvwexcessret&&v&i rvwffret&&v&i rdgtwret&&v&i vwcar&&v&i
						%END;) b
		WHERE		a.gvkey = b.gvkey AND 
					b.anndate BETWEEN INTNX('MONTH', a.monthend, -3, 'END') + 1 AND a.monthend ;
	QUIT;

	* winsorize;
	%winsorize(
		var 	= 	%DO i = 1 %TO 4; 
						rvwexcessret&&v&i rvwffret&&v&i rdgtwret&&v&i vwcar&&v&i
					%END;,
		var_out = 	%DO i = 1 %TO 4; 
						rvwexcessret&&v&i rvwffret&&v&i rdgtwret&&v&i vwcar&&v&i
					%END;,
		by_var 	= , 
		left 	= 1, 
		right 	= 99, 
		input 	= samplem_ma_bhar,
		output 	= samplem_ma_bhar);

	* select the M&A with the highest absolute market reaction;
	PROC SORT DATA = samplem_ma_bhar;
		BY gvkey monthend DESCENDING macar_11_abs anndate newlyvesting;
	RUN;


	DATA samplem_ma_bhar;
		SET samplem_ma_bhar;
		BY gvkey monthend DESCENDING macar_11_abs anndate newlyvesting;
		IF FIRST.monthend;
	RUN;

	DATA samplem_ma_bhar;
		SET samplem_ma_bhar;
		KEEP newlyvesting newlyvesting_v2 newlyvesting_v3 newlyvesting_v4
			rvwexcessret0_12 rvwexcessret12_24 rvwexcessret24_36 rvwexcessret36_48
			rvwffret0_12 rvwffret12_24 rvwffret24_36 rvwffret36_48
			rdgtwret0_12 rdgtwret12_24 rdgtwret24_36 rdgtwret36_48
			vwcar0_12 vwcar12_24 vwcar24_36 vwcar36_48
			deltardatq deltacapxatq deltardcapxatq yearmonth gvkey;
	RUN;

	PROC EXPORT DATA = samplem_ma_bhar 
		OUTFILE = 'D:\Data\samplem_ma_bhar.dta' 
		REPLACE;
	RUN;
%MEND merge_ma_bhar;
%merge_ma_bhar;

* Merge with Insider Trading;
PROC SORT DATA = mergedname OUT = test NODUPKEY; BY company_id executive_id2 fy_end; RUN;

PROC SQL;
	CREATE TABLE indexcusip AS
	SELECT	*, INTNX('MONTH', datadate, 1, 'END') AS fwddatedate FORMAT = YYMMDDN8.
	FROM	v.mallp a, 
			mergedname (DROP = conm company_name executive) b
	WHERE	a.company_id = b.company_id AND a.fy_end = b.fy_end
			AND a.executive_id2 = b.executive_id2;
QUIT;

* Download insider trading data from TFN;
%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME insider '/wrds/tfn/sasdata/insiders'; 

PROC DOWNLOAD DATA = insider.table1 (WHERE = (YEAR(cdate) >= 2005)) OUT = insider_table1; RUN;

ENDRSUBMIT;

* Stocks sold - Match names with stock transactions;
PROC SQL; 
	CREATE TABLE stocksold (KEEP = gvkey executive_id2 company_id fy_end lagfy_end datadate lagdatadate 
		cusip executive cusip6 conm company_name personid ownermatched rolecodematched
		formtype owner rolecode1 trancode acqdisp trandate tprice ownership
		lagprc shares sharesheld secdate sigdate trancode_ar acqdisp_ar trandate_ar dcn seqnum) AS 
	SELECT 	*
	FROM	indexcusip a, insider_table1 b
	WHERE	a.cusip6 = b.cusip6 AND a.personid = b.personid 
			AND b.trandate BETWEEN a.lagdatadate AND a.datadate
			AND shares ^= . AND (acqdisp = 'D' OR acqdisp_ar = 'D') AND TRANCODE ^= 'F';
QUIT;

PROC SORT DATA = stocksold NODUPKEY;
	BY gvkey executive_id2 company_id fy_end lagfy_end datadate personid dcn seqnum; 
RUN;

PROC SQL;
	CREATE TABLE stocksold AS
	SELECT		gvkey, company_id, executive_id2, fy_end, datadate, lagprc,
				SUM(shares * lagprc) AS stocksoldval
	FROM		stocksold
	GROUP BY	gvkey, company_id, executive_id2, fy_end, datadate, lagprc;
QUIT;

* Merge stocks sold into the sample;
PROC SQL;
	CREATE TABLE samplem_stocksold AS
	SELECT	a.*, COALESCE(stocksoldval, 0) AS stocksoldval
	FROM	samplem a LEFT JOIN stocksold b
	ON		a.gvkey = b.gvkey AND a.executive_id2 = b.executive_id2 AND a.company_id = b.company_id 
			AND a.fy_end = b.fy_end AND a.datadate=b.datadate;
QUIT;

%winsorize(
	var 	= stocksoldval,
	var_out = stocksoldval,
	by_var 	= , 
	left 	= 1, 
	right 	= 99, 
	input 	= samplem_stocksold,
	output 	= samplem_stocksold);

DATA samplem_stocksold;
	SET samplem_stocksold;
	stocksold_sc = stocksoldval / 1000000000;
	KEEP gvkey fyearq fqtr ff49 yearmonth
		rpm rpvalpctm100 
		lagsalary_sc lagbonus_sc newceo
		ceoage_sc ceotenure_sc mkbk rvwexcessretn1_0   
		stocksold_sc stocksoldval
		newlyvesting unvested alreadyvested
		vestingval1 unvestedtotalvaladj1 alreadyvestedtotalval1
		lagsalary lagbonus lagceoage lagceotenure
		logsaleq operatingroaq noperatingroaq debt
		acquiror mklev logsaleq mkbk operatingroaq maliquidity_sic3 hhisic3	
		gdwlipq_2y1_maval gdwlipq_3y1_maval gdwlipq_4y1_maval;
RUN;

PROC EXPORT DATA = samplem_stocksold 
	OUTFILE = 'D:\Data\samplem_stocksold.dta' 
	REPLACE;
RUN;

* insider trading surrounding repurchase and MA announcements;
* Firm-month list with vesting equity;
PROC SQL;
	CREATE TABLE vesting_month AS
	SELECT 	UNIQUE a.gvkey, c.cusip6, a.datadate, a.lagdatadate, a.lagprc, 
			SUM(vestingshares * b.lagprc, delta1 * b.lagprc, 0) AS vestingval1,
			CALCULATED vestingval1 / 1000000000 AS newlyvesting
	FROM	v.mallp a, mnewlyvesting_v1 b, mergedname c
	WHERE	a.gvkey = b.gvkey AND a.datadate = b.datadate 
			AND a.executive_id2 = b.executive_id2 
			AND a.company_id = c.company_id AND a.fy_end = c.fy_end 
			AND a.executive_id2 = c.executive_id2
			AND a.lagprc ^= .;
QUIT;

%winsorize(
	var 	= vestingval1 newlyvesting,
	var_out = vestingval1 newlyvesting,
	by_var 	= , 
	left 	= 1, 
	right 	= 99, 
	input 	= vesting_month,
	output 	= vesting_month);

* Aggregate insider selling at the firm-person-date level;
* DCN and SEQNUM allow us to identify any unique stock transaction;
* aggregate insider selling at the firm-person-date level;
PROC SQL;
	CREATE TABLE sharesold AS
	SELECT		UNIQUE a.cusip6, personid, trandate, 
				SUM(lagprc * shares) AS sharesoldval
	FROM		insider_table1 a, 
				(SELECT UNIQUE cusip6, datadate, lagdatadate, lagprc FROM vesting_month) b
	WHERE		a.cusip6 = b.cusip6
				AND a.trandate BETWEEN b.lagdatadate + 1 AND b.datadate 
				AND YEAR(trandate) BETWEEN 2005 AND 2016 AND (acqdisp = 'D' OR acqdisp_ar = 'D') AND trancode ^= 'F'
				AND a.cusip6 IS NOT MISSING AND shares > 0 AND lagprc > 0
	GROUP BY	a.cusip6, personid, trandate;
QUIT;

* Calculate MVE prior to RP & MA announcement;
* list of RP permno-dates for calculating CAR;
PROC SORT DATA = v.rpann (KEEP = permno anndate cusip) OUT = rpann_permnodate NODUPKEY;
	BY permno anndate cusip;
	WHERE YEAR(anndate) BETWEEN 2006 AND 2015;
RUN;

* list of MA permno-dates for calculating CAR;
PROC SORT DATA = v.ma_acquiror (KEEP = permno anndate acquiror_cusip deal_size RENAME = (acquiror_cusip = cusip)) 
		OUT = ma_permnodate (DROP = deal_size) NODUPKEY;
	BY permno anndate cusip;
	WHERE YEAR(anndate) BETWEEN 2006 AND 2015 AND (deal_size >= 5 OR deal_size = .);
RUN;

%LET wrds = wrds.wharton.upenn.edu 4016;
OPTIONS COMAMID = TCP REMOTE = wrds; 
SIGNON USERNAME = _PROMPT_;
RSUBMIT;

LIBNAME crsp '/wrds/crsp/sasdata/a_stock'; 

* RP;
PROC UPLOAD DATA = rpann_permnodate OUT = rpann_permnodate; RUN;

* retrieve stock price 90 days prior to RP announcement;
* shrout is in thousands;
PROC SQL; 
	CREATE TABLE rpann_mve AS
	SELECT		a.*, ABS(prc * shrout) * 1000 AS mve
	FROM		rpann_permnodate a,
				crsp.dsf (KEEP = date permno prc shrout) b
	WHERE		a.permno = b.permno AND a.anndate - b.date >= 90 AND prc ^= . AND shrout ^= .
	GROUP BY	a.permno, anndate, cusip
	HAVING		date = MAX(date)
	ORDER BY	cusip, anndate, mve DESC;
QUIT;

DATA rpann_mve;
	SET rpann_mve;
	BY cusip anndate DESCENDING mve;
	IF FIRST.anndate;
RUN;

PROC DOWNLOAD DATA = rpann_mve OUT = rpann_mve; RUN;

* MA;
PROC UPLOAD DATA = ma_permnodate OUT = ma_permnodate; RUN;
	
* retrieve MVE at least 90 days prior to the MA announcement;
* shrout is in thousands;
PROC SQL; 
	CREATE TABLE ma_mve AS
	SELECT		a.*, ABS(prc * shrout) * 1000 AS mve
	FROM		ma_permnodate a,
				crsp.dsf (KEEP = date permno prc shrout) b
	WHERE		a.permno = b.permno AND a.anndate - b.date >= 90 AND prc ^= . AND shrout ^= .
	GROUP BY	a.permno, anndate, cusip
	HAVING		date = MAX(date)
	ORDER BY	cusip, anndate, mve DESC;
QUIT;

DATA ma_mve;
	SET ma_mve;
	BY cusip anndate DESCENDING mve;
	IF FIRST.anndate;
RUN;

PROC DOWNLOAD DATA = ma_mve OUT = ma_mve; RUN;

* trading days;
PROC SORT DATA = crsp.dsf (KEEP = date) OUT = trading_days NODUPKEY;
	BY date;
	WHERE YEAR(date) BETWEEN 1990 AND 2017;
RUN;

PROC DOWNLOAD DATA = trading_days OUT = trading_days; RUN;
ENDRSUBMIT;

* Aggregate insider selling in the pre- and post- window;
* assign trading day;
DATA trading_days;
	SET trading_days;
	IF _n_ = 1 THEN trading_day_count = 0;
	trading_day_count + 1;
RUN;

PROC SQL;
	CREATE TABLE ma_mve AS
	SELECT		a.*, b.trading_day_count
	FROM		ma_mve a LEFT JOIN trading_days b
	ON			a.anndate BETWEEN b.date - 7 AND b.date
	GROUP BY	cusip, anndate
	HAVING		b.date = MAX(b.date);
QUIT;

PROC SQL;
	CREATE TABLE rpann_mve AS
	SELECT		a.*, b.trading_day_count
	FROM		rpann_mve a LEFT JOIN trading_days b
	ON			a.anndate BETWEEN b.date - 7 AND b.date
	GROUP BY	cusip, anndate
	HAVING		b.date = MAX(b.date);
QUIT;

PROC SQL;
	CREATE TABLE sharesold AS
	SELECT		a.*, b.trading_day_count
	FROM		sharesold a LEFT JOIN trading_days b
	ON			trandate BETWEEN b.date - 7 AND b.date
	GROUP BY	a.cusip6, personid, trandate
	HAVING		b.date = MAX(b.date);
QUIT;

%MACRO sharesold_pre_post;
	%LET window1 = 2;
	%LET window2 = 5;
	%LET window3 = 10;
	%LET window4 = 15;
	%LET window5 = 20;
	%LET window_number = 5;
	%DO w = 1 %TO &window_number;	
		* match repurchase with the insider trading in the pre- and post- window;
		PROC SQL;
			CREATE TABLE rp_sharesold_&&window&w..day AS
			SELECT		cusip, anndate, personid, trandate, 
						sharesoldval, sharesoldval / MVE * 100 AS sharesoldpct,
						(trandate - anndate < 0) AS pre_rpann,
						(trandate - anndate > 0) AS post_rpann
			FROM		(SELECT UNIQUE cusip, anndate, mve, trading_day_count FROM rpann_mve) a, 
						sharesold b
			WHERE		a.cusip = b.cusip6 AND ABS(a.trading_day_count - b.trading_day_count) <= &&window&w;
		QUIT;

		* calculate insider trading intensity in the pre- and post- window;
		PROC SQL;
			CREATE TABLE rp_sharesold_&&window&w..day AS
			SELECT		a.*, 
						SUM(b1.sharesoldval) > 0 AS stocksold_dummy_pre&&window&w..day,
						COUNT(UNIQUE b1.personid) AS stocksold_pcount_pre&&window&w..day,
						COUNT(UNIQUE b1.trandate) AS stocksold_dcount_pre&&window&w..day,
						COALESCE(SUM(b1.sharesoldval), 0) AS stocksoldval_total_pre&&window&w..day,
						COALESCE(SUM(b1.sharesoldpct), 0) AS stocksoldpct_total_pre&&window&w..day,
						SUM(b2.sharesoldval) > 0 AS stocksold_dummy_post&&window&w..day,
						COUNT(UNIQUE b2.personid) AS stocksold_pcount_post&&window&w..day,
						COUNT(UNIQUE b2.trandate) AS stocksold_dcount_post&&window&w..day,
						COALESCE(SUM(b2.sharesoldval), 0) AS stocksoldval_total_post&&window&w..day,
						COALESCE(SUM(b2.sharesoldpct), 0) AS stocksoldpct_total_post&&window&w..day
			FROM		(SELECT UNIQUE cusip, anndate FROM rpann_mve) a 
						LEFT JOIN rp_sharesold_&&window&w..day (WHERE = (pre_rpann = 1)) b1
			ON			a.cusip = b1.cusip AND a.anndate = b1.anndate
						LEFT JOIN rp_sharesold_&&window&w..day (WHERE = (post_rpann = 1)) b2
			ON			a.cusip = b2.cusip AND a.anndate = b2.anndate
			GROUP BY	a.cusip, a.anndate;
		QUIT;

		* winsorize the continuous variables;
		%winsorize(
				var = 		stocksoldval_total_pre&&window&w..day stocksoldpct_total_pre&&window&w..day
							stocksoldval_total_post&&window&w..day stocksoldpct_total_post&&window&w..day,
				var_out = 	stocksoldval_total_pre&&window&w..day stocksoldpct_total_pre&&window&w..day
							stocksoldval_total_post&&window&w..day stocksoldpct_total_post&&window&w..day,
				by_var = , 
				left = 1, 
				right = 99, 
				input = rp_sharesold_&&window&w..day,
				output = rp_sharesold_&&window&w..day);

		* match MA with the insider trading in the pre- and post- window;
		PROC SQL;
			CREATE TABLE ma_sharesold_&&window&w..day AS
			SELECT		cusip, anndate, personid, trandate, 
						sharesoldval, sharesoldval / mve * 100 AS sharesoldpct,
						(trandate - anndate < 0) AS pre_ma,
						(trandate - anndate > 0) AS post_ma
			FROM		(SELECT UNIQUE cusip, anndate, mve, trading_day_count FROM ma_mve) a, 
						sharesold b
			WHERE		a.cusip = b.cusip6 AND ABS(a.trading_day_count - b.trading_day_count) <= &&window&w;
		QUIT;

		* calculate insider trading intensity in the pre- and post- window;
		PROC SQL;
			CREATE TABLE ma_sharesold_&&window&w..day AS
			SELECT		a.*, 
						SUM(b1.sharesoldval) > 0 AS stocksold_dummy_pre&&window&w..day,
						COUNT(UNIQUE b1.personid) AS stocksold_pcount_pre&&window&w..day,
						COUNT(UNIQUE b1.trandate) AS stocksold_dcount_pre&&window&w..day,
						COALESCE(SUM(b1.sharesoldval), 0) AS stocksoldval_total_pre&&window&w..day,
						COALESCE(SUM(b1.sharesoldpct), 0) AS stocksoldpct_total_pre&&window&w..day,
						SUM(b2.sharesoldval) > 0 AS stocksold_dummy_post&&window&w..day,
						COUNT(UNIQUE b2.personid) AS stocksold_pcount_post&&window&w..day,
						COUNT(UNIQUE b2.trandate) AS stocksold_dcount_post&&window&w..day,
						COALESCE(SUM(b2.sharesoldval), 0) AS stocksoldval_total_post&&window&w..day,
						COALESCE(SUM(b2.sharesoldpct), 0) AS stocksoldpct_total_post&&window&w..day
			FROM		(SELECT UNIQUE cusip, anndate FROM ma_mve) a 
						LEFT JOIN ma_sharesold_&&window&w..day (WHERE = (pre_ma = 1)) b1
			ON			a.cusip = b1.cusip AND a.anndate = b1.anndate
						LEFT JOIN ma_sharesold_&&window&w..day (WHERE = (post_ma = 1)) b2
			ON			a.cusip = b2.cusip AND a.anndate = b2.anndate
			GROUP BY	a.cusip, a.anndate;
		QUIT;

		* winsorize the continuous variables;
		%winsorize(
				var = 		stocksoldval_total_pre&&window&w..day stocksoldpct_total_pre&&window&w..day
							stocksoldval_total_post&&window&w..day stocksoldpct_total_post&&window&w..day,
				var_out = 	stocksoldval_total_pre&&window&w..day stocksoldpct_total_pre&&window&w..day
							stocksoldval_total_post&&window&w..day stocksoldpct_total_post&&window&w..day,
				by_var = , 
				left = 1, 
				right = 99, 
				input = ma_sharesold_&&window&w..day,
				output = ma_sharesold_&&window&w..day);

	%END;
		
	* merge all RP windows together;
	PROC SQL;
		CREATE TABLE rp_sharesold_allwindow AS
		SELECT	*
		FROM	%DO i = 1 %TO &window_number-1; rp_sharesold_&&window&i..day a&i, %END;
				rp_sharesold_&&window&window_number..day a&window_number
		WHERE	a1.cusip = a2.cusip AND a1.anndate = a2.anndate
				%DO i = 3 %TO &window_number; 
					AND a1.cusip = a&i..cusip AND a1.anndate = a&i..anndate
				%END;;
	QUIT;
	
	* merge all MA windows together;
	PROC SQL;
		CREATE TABLE ma_sharesold_allwindow AS
		SELECT	*
		FROM	%DO i = 1 %TO &window_number-1; ma_sharesold_&&window&i..day a&i, %END;
				ma_sharesold_&&window&window_number..day a&window_number
		WHERE	a1.cusip = a2.cusip AND a1.anndate = a2.anndate
				%DO i = 3 %TO &window_number; 
					AND a1.cusip = a&i..cusip AND a1.anndate = a&i..anndate
				%END;;
	QUIT;
%MEND sharesold_pre_post;
%sharesold_pre_post;

* merge with newlyvesting and firm-year;
PROC SQL;
	CREATE TABLE rp_sharesold_allwindow_m AS
	SELECT	a.*, 
			stocksoldpct_total_post20day - stocksoldpct_total_pre20day AS stocksoldpct_total_diff20day,
			b.*
	FROM	rp_sharesold_allwindow a, 
			(SELECT		cusip6, datadate, lagdatadate, SUM(newlyvesting) AS newlyvesting
			FROM 		vesting_month 
			GROUP BY	cusip6, datadate, lagdatadate) b
	WHERE	a.cusip = b.cusip6 AND a.anndate BETWEEN b.lagdatadate + 1 AND b.datadate;
QUIT;

PROC EXPORT DATA = rp_sharesold_allwindow_m
	OUTFILE = 'D:\Data\rp_sharesold_allwindow_m.dta' 
	REPLACE;
RUN;

PROC SQL;
	CREATE TABLE ma_sharesold_allwindow_m AS
	SELECT	a.*, 
			stocksoldpct_total_post20day - stocksoldpct_total_pre20day AS stocksoldpct_total_diff20day,
			b.*
	FROM	ma_sharesold_allwindow a, 
			(SELECT		cusip6, datadate, lagdatadate, SUM(newlyvesting) AS newlyvesting
			FROM 		vesting_month 
			GROUP BY	cusip6, datadate, lagdatadate) b
	WHERE	a.cusip = b.cusip6 AND a.anndate BETWEEN b.lagdatadate + 1 AND b.datadate;
QUIT;

PROC EXPORT DATA = ma_sharesold_allwindow_m
	OUTFILE = 'D:\Data\ma_sharesold_allwindow_m.dta' 
	REPLACE;
RUN;
